      subroutine nvsbom(shapp,cartb,hessb,baloc,ofram,elvel,elpre,
     .                  elmat,elrhs,grave,force,veloc,corio,dsurf,
     .                  ndime,nnode,nnods,nnodt,nevab,nevat,ntens,
     .                  nhess,denac,visac,porac,timom,dtins,frano,
     .                  ievns,iecns,jlins,fcons,zeroc)
c************************************************************************
c
c**** This routine computes the contribution to ELMAT for the Navier-
c**** Stokes equations due to the normal component of the residual at
c**** the boundary for the GLS approach. This term appears when one
c**** adds
c       _               _               _
c      |               |               |
c      |  q div(R) = - |  grad(q).R +  |   q n.R 
c     _|D             _|D             _|dD
c     
c**** as stabilization term for the pressure (it's the second term
c**** in the RHS). The temporal variation in the residual is neglected.
c
c************************************************************************
      implicit none
      integer   ndime,nnode,nnods,nnodt,nevab,nevat,ntens,
     .          inods,ievat,jnods,jevat,kdime,ldime,jevab,
     .          jnode,ievns,iecns,jlins,nhess
      real*8    denac,timom,visac,porac,dtins,fcons,
     .          produ,xmuit,dsurf,alapl,frano(4,2),xmui1,float,dilas,
     .          zeroc,vecdot,vecave
      real*8    shapp(nnodt),             cartb(ndime,nnodt),
     .          hessb(ntens,nnode),       baloc(ndime),
     .          elvel(nevab,*),           elpre(nnods,*),
     .          elmat(nevat,nevat),       elrhs(nevat),
     .          grave(ndime,ndime),       force(ndime),
     .          veloc(ndime),             
     .          ofram(3,2),               corio(3)
c
c*** Initializations
c
      call veczer(3,corio)
      call veczer(ndime*ndime,grave)
      call vecasi(ndime,baloc,corio)
      call vecpro(corio,ofram(1,2),corio)                   ! n x w
c
c*** Computations needed for the convective term
c
      if (iecns.eq.1) then
        dilas=0.0
        do kdime=1,ndime
          do ldime=1,ndime
            jevab=ldime
            do jnode=1,nnode
              grave(kdime,ldime)=grave(kdime,ldime)+        ! grad(u)
     .          cartb(kdime,jnode)*elvel(jevab,1)
              jevab=jevab+ndime
            end do
          end do
          dilas=dilas+grave(kdime,kdime)                    ! div(u)
        end do
        dilas=dilas*fcons
      end if
c
c*** Main calculations
c
      produ=timom*dsurf*dtins/denac                         
      do inods=1,nnods                                      
        ievat=nevab+inods                                   
        xmuit=shapp(nnode+inods)*produ  
c
c*** Contribution to ELRHS from f.n
c
        elrhs(ievat)=elrhs(ievat)+xmuit* 
     .    vecdot(force,baloc,ndime)      
c
c*** Contribution to ELMAT from grad(p).n
c        
        do jnods=1,nnods
          jevat=nevab+jnods
          elmat(ievat,jevat)=elmat(ievat,jevat)+xmuit* 
     .      vecdot(cartb(1,nnode+jnods),baloc,ndime)   
        end do                                         
c
c*** Contribution from the convective term  u.grad(u) according to the
c*** type of linearization. The case in which the convective term is treated
c*** explicitly is treated as a RHS linearization (JLINS = 0)        
c
        if (iecns.eq.1) then 
          xmui1=xmuit*denac
          if(jlins.eq.0) then                               ! RHS linearization
            do ldime=1,ndime
              elrhs(ievat)=elrhs(ievat)-xmui1
     .          *(vecdot(veloc,grave(1,ldime),ndime)*baloc(ldime)+
     .          dilas*veloc(ldime)*baloc(ldime))
            end do
          else if(jlins.ge.1) then                          ! Picard 
            do ldime=1,ndime
              jevab=ldime
              do jnode=1,nnode
                elmat(ievat,jevab)=elmat(ievat,jevab)+xmui1
     .            *(vecdot(veloc,cartb(1,jnode),ndime)
     .            +dilas*shapp(jnode))*baloc(ldime)
                jevab=jevab+ndime
              end do
            end do
          else if (jlins.eq.2) then                         ! Newton-Rhapson
            do ldime=1,ndime
              elrhs(ievat)=elrhs(ievat)+xmui1
     .          *(vecdot(veloc,grave(1,ldime),ndime)*baloc(ldime)+
     .          dilas*veloc(ldime)*baloc(ldime))
              jevab=ldime
              do jnode=1,nnode
                do kdime=1,ndime
                  elmat(ievat,jevab)=elmat(ievat,jevab)+
     .              xmui1*(shapp(jnode)*grave(ldime,kdime)+
     .              cartb(ldime,jnode)*veloc(kdime))*baloc(kdime)
                end do
                jevab=jevab+ndime
              end do
            end do
          end if
        end if
c
c*** Contribution to ELMAT due to (2*rho*(Ome x u) + poros*u).n 
c        
        if (frano(1,2).gt.zeroc) then
          jevab=0
          do jnode=1,nnode
            xmui1=2.0d0*xmuit*denac*shapp(jnode)*frano(1,2)
            do kdime=1,ndime                           
              jevab=jevab+1                            
              elmat(ievat,jevab)=elmat(ievat,jevab)   
     .          + corio(kdime)*xmui1
     .          + baloc(kdime)*porac*shapp(jnode)
            end do
          end do
        end if
c
c*** Contribution to ELMAT due to -mu*div(grad(u)).n
c        
        if((ievns.eq.1).and.(nhess.ne.0)) then
          xmui1=xmuit*visac
          jevab=0
          do jnode=1,nnode                   
            alapl=vecave(hessb(1,jnode),ndime)*xmui1
            do ldime=1,ndime                        
              jevab=jevab+1
              elmat(ievat,jevab)=elmat(ievat,jevab)-
     .          alapl*baloc(ldime)
            end do
          end do
        end if
      end do
            
      end
