      subroutine boulib(objbo,ilebo,objel,ileel,lboel,lnods,iffix,
     .                  shapb,shaga,shape,tload,ivabo,elpro,elcod,
     .                  elvel,elpre,eltem,elfrs,elmat,elrhs,work1,
     .                  itask)
c************************************************************************
c
c**** This routine performs computations at the boundary level according
c**** to the following tasks:
c
c     itask =  1   -->  Empty task
c     itask =  2   -->  Compute the RHS for the Navier-Stokes equations
c                       due to surface contributions
c     itask =  3   -->  Compute force and torque
c     itask =  4   -->  Compute the RHS for the heat equation due to 
c                       surface heat flux
c     itask =  5   -->  Put atmosferic stress on required boundaries
c
c**** The objects (now boundary elements) are organized as follows:
c
c     objbo(ilebo(fdsur)) : Pointer to surface element            DSURF
c     objbo(ilebo(fgpcb)) : Pointer to integration point coords.  GPCOB
c     objbo(ilebo(fbalo)) : Pointer to local basis vectors        BALOC
c
c************************************************************************
      implicit real*8(a-h,o-z)
      include 'com_prob.f'
      include 'com_auxi.f'
      integer   ilebo(*), ileel(*) 
      integer   ivabo(*), lboel(*), lnods(*), iffix(*)
      real*8    objbo(*), objel(*), shapb(*), tload(*),  
     .          elvel(*), elpre(*), eltem(*), elfrs(*),    
     .          elmat(*), elrhs(*), work1(*), shaga(*),  
     .          elpro(*), elcod(*), shape(*)
c
c*** Direct to appropriate processor
c
      go to ( 10 , 20 , 30 ,40 ,50 ) , itask
c
c*** Empty task
c
   10 continue
      return
c
c*** Compute the element matrix and RHS for the Navier-Stokes equations
c*** due to surface contributions
c
   20 call nvsbou(%val(lboty_p),lboel,elpro,shapb,shaga,shape,
     .            tload,ivabo  ,elmat,elrhs,elvel,elpre,
     .            objbo(ilebo(fdsur)),objbo(ilebo(fgpcb)),
     .            objbo(ilebo(fbalo)),
     .            objel(ileel(fcart)),objel(ileel(fhess)),
     .            objel(ileel(fhlen)),objel(ileel(ftrag)),
     .            work1(  invsb(    1)),work1(  invsb(    2)),
     .            work1(  invsb(    3)),work1(  invsb(    4)),
     .            work1(  invsb(    5)),work1(  invsb(    6)),
     .            work1(  invsb(    7)),work1(  invsb(    8)),
     .            work1(  invsb(   14)),work1(  invsb(   15)))
      call nvsmod(%val(lpoty_p),%val(lroty_p),%val(presv_p),
     .            %val(exnor_p),%val(skcos_p),
     .            iffix,lnods,elmat,elrhs,elfrs,elvel,work1,1)
      return
c
c*** Compute force and torque on the boundaries
c
   30 call nvsbof(%val(lboty_p),lboel,elpro,shapb,shaga,shape,
     .            elvel,elpre,%val(fflow_p),
     .            objbo(ilebo(fdsur)),objbo(ilebo(fgpcb)),
     .            objbo(ilebo(fbalo)),
     .            objel(ileel(fcart)),
     .            work1(  invsb(    4)),work1(  invsb(    5)),
     .            work1(  invsb(    6)),work1(  invsb(    9)))
      return
c
c*** Compute the element matrix and RHS for the temperature equation
c*** due to surface contributions
c
   40 call tembou(lboel,shapb,tload,ivabo,elmat,elrhs,eltem,
     .            objbo(ilebo(fdsur)),
     .            work1(  itemb(    1)))
      return
c
c*** Put the atmosferic stress
c
   50 call nvsatm(%val(lboty_p),lboel,ivabo,tload,elcod,
     .            objbo(ilebo(fbalo)),objbo(ilebo(fgpcb)),
     .            work1(  invsb(   10)),work1(  invsb(   11)),
     .            work1(  invsb(   12)),work1(  invsb(   13)))
      return

      end
