      subroutine genass(
     .  lnods,elmat,elrhs,rhsid,nnode,nnods,nnodt,
     .  npoin,nvalo,nevaw,ntotw,ndofn,ncrhs,iablo,
     .  iprob,ielem,nelem,kfait,value,idest)
c*****************************************************************************
c
c**** This routine performs the assembly of the element stiffness matrix
c**** ELMAT in the global matrix GLMAT and the element right-hand-side ELRHS
c**** in the global right-hand-side RHSID. This routine is also used for the
c**** assembly of boundary arrays.
c
c**** The need for assembling more than one block is identified by IABLO,
c**** according to the following possibilities:
c
c     IABLO = 0.  No matrix needs to be assembled
c     IABLO = 1.  The matrix to be assembled is composed of one block
c     IABLO = 2.  The matrix to be assembled is composed of two blocks
c
c*****************************************************************************
      implicit none
      include   'MatMan.h'
      integer
     .  nnode,nnods,nnodt,npoin,nvalo,nevaw,ntotw,
     .  ndofn,ncrhs,iablo,iprob,ielem,nelem,kfait,
     .  nnodw,icrhs,idumy                    
      integer    lnods(nnodt),       idest(2*nvalo*nvalo)
      real*8     elmat(nvalo,nvalo), elrhs(nvalo,ncrhs),
     .           rhsid(ntotw,ncrhs), value(nvalo*nvalo)
      real*8     dummy
c
c*** Matrices are only assembled if KFAIT = 1
c      
      if(kfait.eq.1) then
c
c*** Assembly of the main block for direct solvers
c
        if(iablo.gt.0.and.kmsip(iprob).ge.0)
     .    call assmat(
     .    lnods,elmat,    0,nnode,   0,    0,ndofn,
     .        0,nnode,    0,         0,ndofn,nvalo,
     .    nnodt,iprob,value,idest)
c
c*** For the Navier-Stokes equations, if pressure is not eliminated the
c*** associated matrices are assembled, or the elemental matrix is stored
c*** for iterative solvers. 
c
        if(iablo.eq.2.and.kmsip(iprob).ge.0) then
          call assmat(
     .      lnods,elmat,    0,nnode,    0,    0,ndofn,
     .      nnode,nnodt,npoin,ndofn,    1,nvalo,nnodt,
     .      iprob,value,idest)
          call assmat(
     .      lnods,elmat,nnode,nnodt,npoin,ndofn,    1,
     .          0,nnode,    0,    0,ndofn,nvalo,nnodt,
     .      iprob,value,idest)
          call assmat(
     .      lnods,elmat,nnode,nnodt,npoin,ndofn,    1,
     .      nnode,nnodt,npoin,ndofn,    1,nvalo,nnodt,
     .      iprob,value,idest)
        else if (kmsip(iprob).lt.0) then
          nnodw=nnode
          if(nevaw.gt.ndofn*nnode) nnodw=nnodt
          call MatMan(
     .      idumy,idumy,dummy,dummy,ndofn,nnode,nnodt,
     .      nnodw,nelem,npoin,idumy,    1,idumy,kfait,
     .      idumy,iprob,nvalo,ielem,elmat,    3)
        end if
      end if
c
c*** Scater of the RHS
c      
      do icrhs=1,ncrhs
        call scater(rhsid(1,icrhs),elrhs(1,icrhs),lnods,
     .              ndofn,nnode,npoin)
      end do      
      if(iablo.eq.2) then
        call rlnods(lnods,nnodt,nnode, 1,-npoin)            ! Scater the rest
        call scater(rhsid(npoin*ndofn+1,1),                 ! of RHSID
     .              elrhs(nevaw+1-nnods,1),
     .              lnods(nnode+1),1,nnods,
     .              ntotw-npoin*ndofn      )
        call rlnods(lnods,nnodt,nnode, 1, npoin)
      end if

      end
