      subroutine repnod(lnods,coord,nnode,node0,node1,ndime,
     .                  nelem,npoii,lenpo,imcoo,npoif)
c*************************************************************************
c
c*** This routine eliminates repeated nodes of a finite element mesh.
c*** The input is:
c
c    NNODE:              Number of nodes per element
c    NDIME:              Number of space dimensions
c    NELEM:              Number of elements
c    NPOII:              Initial number of nodal points (some repeated)
c    LENPO:              Length of the pointers
c    NODE0:              Initial node number to check repetitions      
c    NODE1:              Final node number to check repetitions      
c    IMCOO:              Flag to modify (=1) or not (=0) the coordinates
c      
c    LNODS(NNODE,NELEM): Initial list of nodal connections
c    COORD(NDIME,NPOII): Initial list of coordinates
c
c*** and the output is
c
c    NPOIF:              Final number of nodal points
c    LNODS(NNODE,NELEM): Final list of nodal connections
c    COORD(NDIME,NPOIF): Final list of coordinates
c
c*** The algorithm of this routine is based on the fact that repeated 
c*** nodes belong to elements that share at least one node
c
c*** The basic tree is:
c
c    REPNOD                Main routine
c          MCELPO          List of elements per point
c                MCNEPO    Number of elements per point
c                MCIDNU    Identification of a number in the list
c                MCFINU    Filling of a number in the list
c          COINOD          Check of coincident nodes
c          ELRENO          Elimination of repeated nodes
c      
c*************************************************************************
      implicit none
      integer  nnode,node0,node1,ndime,nelem,npoii,npoif,
     .         lenpo,imcoo,lbyts,ipoin,nlist,idumy
      integer  lnods(nnode,nelem)
      integer  p_nepoi,p_lelpo,p_norep,iadd1,iadd2          ! pointers
      real*8   coord(ndime,npoii)
      real*8   zeroc
c
c*** Construct first the list of elements to which each nodal points belong
c
      call mcelpo(nelem,nnode,node0,node1,npoii,lenpo,
     .            lnods,p_nepoi,p_lelpo)
c
c*** Compute the point with which a given one coincides with lowest nodal
c*** number (array NOREP)
c
      zeroc=1.0e-15
      lbyts=lenpo*npoii
      call admemo(0,p_norep,lbyts)
      call lvecze(npoii,%val(p_norep))
      do ipoin=1,npoii
        call mcidnu(%val(p_nepoi),npoii,ipoin,nlist)
        if(nlist.gt.1) then
          iadd1=p_lelpo+(ipoin-1)*lenpo
          call poiass(%val(iadd1),iadd2)
          call coinod(%val(iadd2),%val(p_norep),lnods,coord,nnode,
     .                node0,node1,nelem,ndime,npoii,nlist,zeroc)
        end if
      end do
c
c*** Eliminate repeated nodes and redefine the vectors LNODS and COORD
c
      call elreno(%val(p_norep),lnods,coord,nnode,node0,node1,
     .            nelem,ndime,npoii,imcoo,npoif)
c
c*** Deallocate memory
c
      call admemo(2,p_nepoi,idumy)
      call admemo(2,p_lelpo,idumy)
      call admemo(2,p_norep,idumy)
      
      end
