      subroutine invmtx(a,b,deter,nsize)
c***********************************************************************
c
c**** This routine inverts a square matrix A -> Mat(nsize,nsize). The
c**** inverse is stored in B. Its determinant is DETER
c
c***********************************************************************
      implicit real*8(a-h,o-z)
      real*8 a(nsize,*), b(nsize,*)
c
c*** Invers of a 1*1 matrix
c
      if(nsize.eq.1) then
        deter=a(1,1)
        if(deter.eq.0.) return
        b(1,1) = 1./a(1,1)
        return
      endif
c
c*** Invers of a 2*2 matrix
c
      if(nsize.eq.2) then
        deter=a(1,1)*a(2,2)-a(2,1)*a(1,2)
        if(deter.eq.0.) return
        denom=1.0/deter
        b(1,1) = a(2,2)*denom
        b(2,2) = a(1,1)*denom
        b(2,1) =-a(2,1)*denom
        b(1,2) =-a(1,2)*denom
        return
      endif
c
c*** Inverse of a 3*3 matrix
c
      if(nsize.eq.3) then
        t1  = a(2,2)*a(3,3) - a(3,2)*a(2,3)
        t2  =-a(2,1)*a(3,3) + a(3,1)*a(2,3)
        t3  = a(2,1)*a(3,2) - a(3,1)*a(2,2)
        deter = a(1,1)*t1 + a(1,2)*t2 + a(1,3)*t3
        if(deter.eq.0.0) return
        denom = 1./deter
        b(1,1) = t1*denom
        b(2,1) = t2*denom
        b(3,1) = t3*denom
        b(2,2) = ( a(1,1)*a(3,3) - a(3,1)*a(1,3))*denom
        b(3,2) = (-a(1,1)*a(3,2) + a(1,2)*a(3,1))*denom
        b(3,3) = ( a(1,1)*a(2,2) - a(2,1)*a(1,2))*denom
        b(1,2) = (-a(1,2)*a(3,3) + a(3,2)*a(1,3))*denom
        b(1,3) = ( a(1,2)*a(2,3) - a(2,2)*a(1,3))*denom
        b(2,3) = (-a(1,1)*a(2,3) + a(2,1)*a(1,3))*denom
        return
      endif
c
c*** Inverse of a 4*4 matrix
c
      if(nsize.eq.4) then
        t1= a(2,2)*a(3,3)*a(4,4) + a(2,3)*a(3,4)*a(4,2)
     .    + a(2,4)*a(3,2)*a(4,3) - a(2,3)*a(3,2)*a(4,4)
     .    - a(2,2)*a(3,4)*a(4,3) - a(2,4)*a(3,3)*a(4,2)
        t2=-a(2,1)*a(3,3)*a(4,4) - a(2,3)*a(3,4)*a(4,1)
     .    - a(2,4)*a(3,1)*a(4,3) + a(2,4)*a(3,3)*a(4,1)
     .    + a(2,3)*a(3,1)*a(4,4) + a(2,1)*a(3,4)*a(4,3)
        t3=+a(2,1)*a(3,2)*a(4,4) + a(2,2)*a(3,4)*a(4,1)
     .    + a(2,4)*a(3,1)*a(4,2) - a(2,4)*a(3,2)*a(4,1)
     .    - a(2,2)*a(3,1)*a(4,4) - a(2,1)*a(3,4)*a(4,2)
        t4=-a(2,1)*a(3,2)*a(4,3) - a(2,2)*a(3,3)*a(4,1)
     .    - a(2,3)*a(3,1)*a(4,2) + a(2,3)*a(3,2)*a(4,1)
     .    + a(2,2)*a(3,1)*a(4,3) + a(2,1)*a(3,3)*a(4,2)
        deter= a(1,1)*t1 + a(1,2)*t2 + a(1,3)*t3 + a(1,4)*t4
        if(deter.eq.0.) return
        denom=1./deter
        b(1,1) = t1*denom
        b(2,1) = t2*denom
        b(3,1) = t3*denom
        b(4,1) = t4*denom
        b(1,2) =(- a(1,2)*a(3,3)*a(4,4) - a(1,3)*a(3,4)*a(4,2)
     .    - a(1,4)*a(3,2)*a(4,3) + a(1,3)*a(3,2)*a(4,4)
     .    + a(1,2)*a(3,4)*a(4,3) + a(1,4)*a(3,3)*a(4,2))*denom
        b(2,2) =(  a(1,1)*a(3,3)*a(4,4) + a(1,3)*a(3,4)*a(4,1)
     .    + a(1,4)*a(3,1)*a(4,3) - a(1,4)*a(3,3)*a(4,1)
     .    - a(1,3)*a(3,1)*a(4,4) - a(1,1)*a(3,4)*a(4,3))*denom
        b(3,2) =(- a(1,1)*a(3,2)*a(4,4) - a(1,2)*a(3,4)*a(4,1)
     .    - a(1,4)*a(3,1)*a(4,2) + a(1,4)*a(3,2)*a(4,1)
     .    + a(1,2)*a(3,1)*a(4,4) + a(1,1)*a(3,4)*a(4,2))*denom
        b(4,2) =(  a(1,1)*a(3,2)*a(4,3) + a(1,2)*a(3,3)*a(4,1)
     .    + a(1,3)*a(3,1)*a(4,2) - a(1,3)*a(3,2)*a(4,1)
     .    - a(1,2)*a(3,1)*a(4,3) - a(1,1)*a(3,3)*a(4,2))*denom
        b(1,3) =(  a(1,2)*a(2,3)*a(4,4) + a(1,3)*a(2,4)*a(4,2)
     .    + a(1,4)*a(2,2)*a(4,3) - a(1,3)*a(2,2)*a(4,4)
     .    - a(1,2)*a(2,4)*a(4,3) - a(1,4)*a(2,3)*a(4,2))*denom
        b(2,3) =(- a(1,1)*a(2,3)*a(4,4) - a(1,3)*a(2,4)*a(4,1)
     .    - a(1,4)*a(2,1)*a(4,3) + a(1,4)*a(2,3)*a(4,1)
     .    + a(1,3)*a(2,1)*a(4,4) + a(1,1)*a(2,4)*a(4,3))*denom
        b(3,3) =(  a(1,1)*a(2,2)*a(4,4) + a(1,2)*a(2,4)*a(4,1)
     .    + a(1,4)*a(2,1)*a(4,2) - a(1,4)*a(2,2)*a(4,1)
     .    - a(1,2)*a(2,1)*a(4,4) - a(1,1)*a(2,4)*a(4,2))*denom
        b(4,3) =(- a(1,1)*a(2,2)*a(4,3) - a(1,2)*a(2,3)*a(4,1)
     .    - a(1,3)*a(2,1)*a(4,2) + a(1,3)*a(2,2)*a(4,1)
     .    + a(1,2)*a(2,1)*a(4,3) + a(1,1)*a(2,3)*a(4,2))*denom
        b(1,4) =(- a(1,2)*a(2,3)*a(3,4) - a(1,3)*a(2,4)*a(3,2)
     .    - a(1,4)*a(2,2)*a(3,3) + a(1,4)*a(2,3)*a(3,2)
     .    + a(1,3)*a(2,2)*a(3,4) + a(1,2)*a(2,4)*a(3,3))*denom
        b(2,4) =(  a(1,1)*a(2,3)*a(3,4) + a(1,3)*a(2,4)*a(3,1)
     .    + a(1,4)*a(2,1)*a(3,3) - a(1,4)*a(2,3)*a(3,1)
     .    - a(1,3)*a(2,1)*a(3,4) - a(1,1)*a(2,4)*a(3,3))*denom
        b(3,4) =(- a(1,1)*a(2,2)*a(3,4) - a(1,2)*a(2,4)*a(3,1)
     .    - a(1,4)*a(2,1)*a(3,2) + a(1,4)*a(2,2)*a(3,1)
     .    + a(1,2)*a(2,1)*a(3,4) + a(1,1)*a(2,4)*a(3,2))*denom
        b(4,4) =(  a(1,1)*a(2,2)*a(3,3) + a(1,2)*a(2,3)*a(3,1)
     .    + a(1,3)*a(2,1)*a(3,2) - a(1,3)*a(2,2)*a(3,1)
     .    - a(1,2)*a(2,1)*a(3,3) - a(1,1)*a(2,3)*a(3,2))*denom
        return
      endif                                                 
c
c*** Inverse of a nsize*nsize matrix
c
      do isize=1,nsize
        do jsize=1,nsize
          b(isize,jsize)=a(isize,jsize)
        enddo
      enddo
      call invert(b,nsize,nsize)

      end
