      subroutine listen(subna)
c***********************************************************************
c
c*** Reads a string and interprets it as words and parameters.
c
c   - Maximum number of words and parameters = maxwp.
c   - Only the first five characters of each word are decoded.
c   - The underline characters '_' are discarted.
c   - Lower case letters are converted to upper case.
c   - Each word or parameter must be separated by ' ', '=', ':' or ','
c   - A "comment" begins with '$', '!', '/'  or '/' in any place.
c   - A line that has a comment beginning with '/' or '/'
c     continuates in the next line.
c   - A line beginning with title is not decoded. title directive.
c   - A line beginning with include is an include directive.
c   - A line beginning with echo turns 'on' or 'off' the echo.
c
c***********************************************************************
      implicit none
      include 'com_inou.f'
      include 'def_logi.f'
c
c***  Var
c
        character   subna*6
        real*8      digit
        integer     first,firsp,i,
     .              last,lastp,nunit,ptrwo,npptr,nwptr
        integer     leng,lnbln1,flag,resum
        logical*1   newline,echo
c
c***  Save
c
        save echo         ! save echo status
        save nunit        ! save input file
c
c***  Data
        data echo/.false./! default echo off. to change it use: echo on
        data nunit/ludat/ ! initial data file
c
c***  Format
c
   10   format(a80)
   20   format(1x,a6,' <-- ',a)
c
c***  Begin
c
        ccard=' '
        nnwor=0                                    ! initialize.
        nnpar=0
        nwptr=0
        npptr=0
        resum=0
        do i=1,maxwp
          words(i)=' '
          param(i)=0.0d0
        end do
  100   continue
        do while(((nnwor.eq.0).and.(nnpar.eq.0)   ! don't return without answer
     .              .or.newline).or.resum.eq.1)   ! continue reading if / or \

          if (resum.eq.0) then
            newline=.false.                       ! initialize.
            last=0
            lastp=0
          end if
          resum=0

          read(nunit,10,end=101,err=1) ccard                ! read a card
          leng=lnbln1(ccard)                      ! calculate the length.

          do while(last.lt.leng)                  ! decode all the card.
            first=last+1
            do while(first.lt.leng           .and.
     .               ccard(first:first).eq.'_'.or.! jump null character (_)
     .               ccard(first:first).eq.' '.or.! jump separators ( =:,)
     .               ccard(first:first).eq.'='.or.
     .               ccard(first:first).eq.':'.or.
     .               ccard(first:first).eq.','    )
              first=first+1
            end do
            if(last.eq.0) firsp=first             ! save first to print card
            last=first
            do while((last.le.leng)        .and.  ! look for end (last=leng).
     .               ccard(last:last).ne.' '.and. ! look for separator ( =:,).
     .               ccard(last:last).ne.'='.and.
     .               ccard(last:last).ne.':'.and.
     .               ccard(last:last).ne.','.and.
     .               ccard(last:last).ne.'$'.and. ! look for coment ($!/\).
     .               ccard(last:last).ne.'!'.and.
     .               ccard(last:last).ne.'/'.and.
     .               ccard(last:last).ne.'/'     )
              last=last+1
            end do
            if(ccard(last:last).eq.'$'.or.
     .         ccard(last:last).eq.'!') leng=last-1
            if(ccard(last:last).eq.'/'.or.
     .         ccard(last:last).eq.'/') then
              leng=last-1                         ! deal with continuation
              newline=.true.                      ! set new line flag.
            end if
            last=last-1
            if(last.ge.first) then                ! is it a word or a parameter
              lastp=last                          ! save last to print card
              call decod1(last-first+1,'(f20.0)',
     .        ccard(first:last),flag,digit)
              if(flag.eq.0) then                  ! it is a parameter.
                nnpar=nnpar+1                     ! # of parameters
                npptr=npptr+1                     ! integer to next parameter
                if(npptr.gt.maxwp) go to 4        ! error.
                if(nwptr.gt.npptr) npptr=nwptr
                param(npptr)=digit
              else                                ! it is a word.
                nnwor=nnwor+1                     ! # of words
                nwptr=nwptr+1                     ! integer to next word
                if(nwptr.gt.maxwp) go to 5        ! error.
                if(npptr.ge.nwptr) nwptr=npptr+1
                ptrwo=1
                do while ((first.le.last).and.(ptrwo.le.5))
                  words(nwptr)(ptrwo:ptrwo)=ccard(first:first)
                  ptrwo=ptrwo+1
                  first=first+1
                  do while (ccard(first:first).eq.'_') ! jump null character
                    first=first+1
                  end do
                end do
                call upcase(words(nwptr))         ! convert to upper case.
              end if
            end if ! (last.ge.first)
            if((nnwor.eq.1).and.(nnpar.eq.0).and. ! deal with title or include
     .        ((words(1).eq.'TITLE').or.((words(1).eq.'INCLU').and.
     .        (subna.ne.'NOREAD')))) then
              if(echo.and.(subna.ne.'noecho'))
     .          write(lures,20) 'listen',ccard(firsp:leng)
              last=last+2
              do while(ccard(last:last).eq.' ')   ! remove blank spaces
                last=last+1
              end do
              if(leng.lt.last) go to 6            ! error
              ccard=ccard(last:leng)              ! remove words(1) from ccard
              leng=leng-last+1
              if(words(1).eq.'TITLE') then        ! deal with titles
                firsp=1
                lastp=leng
              else                                ! deal with include directive
                if(nunit.eq.luinc) go to 3        ! error
                last=1                            ! remove tail comments
                do while((last.le.leng).and.      ! look for end (last=leng)
     .               ccard(last:last).ne.' ')     ! look for separator ( )
                  last=last+1
                end do
                ccard=ccard(1:last-1)             ! remove tail comments
                nunit=luinc
                open(unit=nunit,file=ccard,
     .               form='formatted',status='old')
                lastp=0                           ! to ignore the echo
                nnwor=0                           ! forget all
                nwptr=0
                words(1)=' '
              end if
              last=leng                           ! to break the do while
            else if(subna.eq.'NOREAD') then
              newline=.false.
              nnwor=1
            end if                       
          end do ! while (last.lt.leng)

          if((words(1).eq.'ECHO') .and.
     .       (nnpar.eq.0).and.(nnwor.eq.2)) then  ! deal with echo
            if(words(2).eq.'OFF') then
              echo=.false.
              if(subna.ne.'noecho') write(lures,20) 'LISTEN','ECHO OFF'
            else
              echo=.true.
              if(subna.ne.'noecho') write(lures,20) 'LISTEN','ECHO ON'
            endif
            nnwor=0                               ! forget all
            nwptr=0
            do i=1,maxwp
              words(i)=' '
            end do
          else                                    ! print card
            if((echo).and.(firsp.le.lastp).and.(subna.ne.'noecho')) then
              if(newline) then
                lastp=lastp+2
                ccard(lastp-1:lastp)=' /'
              end if
              write(lures,20) subna,ccard(firsp:lastp)
            end if
          end if

        end do ! while ((nnwor.eq.0).and.(nnpar.eq.0).or.newline)

        nwopa=max(npptr,nwptr)
        return
c
c*** End of include
c
  101   if(nunit.ne.luinc) go to 2 ! error
        close(unit=nunit,status='keep')
        nunit=ludat
        if(echo.and.(subna.ne.'noecho'))
     .    write(lures,20) 'LISTEN','END OF INCLUDE FILE'
        resum=1
        go to 100 ! for resume the error return to the same place.
c
c*** Errors:
c
    1   call runend('LISTEN: ERROR DETECTED WHEN READING')
    2   call runend('LISTEN: END OF FILE DETECTED       ')
    3   call runend('LISTEN: ERROR: INCLUDE FROM INCLUDE')
    4   call runend('LISTEN: TOO MANY PARAM IN COMMAND  ')
    5   call runend('LISTEN: TOO MANY WORDS IN COMMAND  ')
    6   call runend('LISTEN: BLANK IS ILEGAL HEAR       ')

      end

      subroutine decod1(nstri,forma,strin,lflag,digit)
      implicit none
c***********************************************************************
c
c*** This routine decodified a string
c
c***********************************************************************
      integer     nstri,lflag,istri,decim
      real*8      digit,atof1
      character*7 forma
      character*1 strin(nstri+1),stri1(81)

      lflag=0                                               ! It is a parameter
      istri=1
      do while(istri.le.nstri)
        decim=ichar(strin(istri))                           ! decimal value
        if (decim.lt.48.or.decim.gt.57) then                ! It is not a num.
          if (decim.ne.43.and.decim.ne.45                   ! discard + -
     .      .and.decim.ne.68.and.decim.ne.69                ! discard D E
     .      .and.decim.ne.100.and.decim.ne.101              ! discard d e
     .      .and.decim.ne.46) then                          ! discard .
            lflag=1
            istri=nstri
          end if
        end if
        istri=istri+1
      end do

      if (lflag.eq.0) then                                  ! It's a number
        do istri=1,nstri
          stri1(istri)=strin(istri)
        end do
        stri1(nstri+1)=char(0)
        digit=atof1(stri1)                                  ! DIGIT<-'STRIN'
      end if

      end
      
      integer function lnbln1(ccard)
c***********************************************************************
c
c*** Compute the length of ccard
c
c***********************************************************************
      implicit none
      character*1 ccard(81)

      lnbln1=81
      do while (ccard(lnbln1).eq.' '.and.lnbln1.ne.1)
        lnbln1=lnbln1-1
      end do

      if (lnbln1.eq.1.and.ccard(1).eq.' ') lnbln1=0

      end
