      subroutine cderda
c***********************************************************************
c       
c**** This routine computes some derivated parameters
c
c***********************************************************************
      implicit none
      include 'com_prob.f'
      include 'com_auxi.f'
      logical*1 confl
      real*8    sftot
c
c*** Number of integration points and pressure nodes
c
      if(ngaus.eq.0) then
        if(nelty.eq.1) then
          ngaus=nnode
          if((ndime.eq.2).and.(ngaus.gt.4)) ngaus= 9
          if((ndime.eq.3).and.(ngaus.gt.8)) ngaus=27
        else if(nelty.eq.2) then
          ngaus=nnode
          if((ndime.eq.2).and.(ngaus.gt.3)) ngaus= 7
          if((ndime.eq.3).and.(ngaus.gt.4)) ngaus=14
        end if
      end if
      if(nnods.eq.0) then
        inpco=0
        if(nelty.eq.1) then
          nnods=1
          if((ndime.eq.2).and.(nnode.eq. 9)) nnods=3
          if((ndime.eq.3).and.(nnode.eq.27)) nnods=4
        else if(nelty.eq.2) then
          nnods=1
          if((ndime.eq.2).and.(nnode.eq. 7)) nnods=3 
          if((ndime.eq.3).and.(nnode.eq.15)) nnods=4
        end if
      end if
c
c*** Number of nodes and integration points per boundary element. For
c*** open integration rules, the number of boundary integration points
c*** is determined in order to keep the accuracy on the integration rule,
c*** whereas in the case of closed integration it is determined by matching
c*** the integration points as belonging to the element and to the boundary
c
      if(ndime.eq.2) then
        nnodb=2                       
        if(nelty.eq.1) then                                 !quadrilateral
          ngaub=int(sqrt(float(ngaus)))
          if(nnode.ge.8) nnodb=3
        else if(nelty.eq.2) then                            !simplex
          if(inqua.eq.0) then                               !open rule
            if(ngaus.le.1) then       
              ngaub=1                 !exact for P1 
            else if(ngaus.le.4) then  
              ngaub=2                 !exact for P3
            else if(ngaus.le.7) then  
              ngaub=3                 !exact for P5
            else if(ngaus.le.13) then 
              ngaub=4                 !exact for P7
            end if 
          else if(inqua.eq.1) then                          !closed rule
            if(ngaus.le.4) then
              ngaub=2
            else if(ngaus.le.7) then
              ngaub=3
            else if(ngaus.le.10) then
              ngaub=4
            end if
          end if
          if(nnode.ge.6) nnodb=3
        end if
      else if(ndime.eq.3) then
        if(nelty.eq.1) then                                 !brick
          nnodb=4
          ngaub=nint(float(ngaus)**(2.0d0/3.0d0))
          if(nnode.eq.20) nnodb=8
          if(nnode.eq.27) nnodb=9
      	else if(nelty.eq.2) then                            !simplex
          if(inqua.eq.0) then                               !open rule
            if(ngaus.eq.1) then
              ngaub=1                 !exact for P1
            else if(ngaus.le.5) then   
              ngaub=ngaus-1           !exact for P2/P3
            else if(ngaus.eq.11) then
              ngaub=6                 !exact for P4
            else if(ngaus.eq.14) then
              ngaub=7                 !exact for P5
            end if
          else if(inqua.eq.1) then                          !closed rule
            if(ngaus.le.5) then
              ngaub=3
            else if(ngaus.le.11) then
              ngaub=6
            else if(ngaus.eq.15) then
              ngaub=7
            else if(ngaus.eq.20) then
              ngaub=10
            end if
          end if
          nnodb=3
          if(nnode.eq.10) nnodb=6
          if(nnode.eq.11) nnodb=6          
          if(nnode.eq.15) nnodb=7
        end if
      end if
      ndimb=ndime-1                    !space dimension on the boundary 
c
c*** Set the number of some dimensions
c
      nnodt=nnode+iepns*nnods          !number of total nodes for the n.s. eqs.
      if(nnodt.eq.0) nnodt=nnode
      nevab=iupns*nnode*ndime          !number of velocity var. per element
      nevat=nevab+iepns*nnods          !number of vel. & pres. var. per element
      ntotv=iupns*npoin*ndime          !total velocity variables
      ntotp=iepns*nnods*nelem          !total pressure variables (maximum)
      ntott=iepns*ntotv+iepns*ntotp    !total velocity-pressure variables
      ntens=3
      if(ndime.eq.3) ntens=6	       !total number of stress component
      nltot=ntotv+iuphe*npoin
     .  +iupfs*npoin                   !total number of main variables
     .  +ietns*ntotv
      nvfix=nltot+ntotp                !as before plus pressure variables
      ntloa=(ndime*iepns+3*iephe)      !total number of load components
     .      *nnodb
      ncotl=1
      if (ietns.eq.1 .or. iethe.eq.1) ncotl=2
      mvalo=max(nevab+
     .  iepns*nnods,nnode)             !maximum number of element variables
c
c*** Flag to check if a transient problem exists 
c
      ietra=max(ietns,iethe,ietfs)      
c
c*** Number of physical properties
c
      nproe=1                         ! Density is always needed
     .  +2*iupns                      ! Viscosity and porosity
     .  +(2+ieshe)*iuphe              ! Cv, k and maybe Q for the heat equation
      nprod=nproe*(1+ietra)
c
c*** Defines some logical flags
c
      ksyns=0                          !N-S problem is symmetric by default
      ksyhe=0                          !heat problem is symmetric by default
      if((iecns.eq.1).and.
     .   (ilins.ne.0))       ksyns=1
      if(lsnsc.ne.0)         ksyns=1
      if(frano(1,1).gt.zero2)ksyns=1
      if((lsnsp.ne.0).and.
     .  (nnode.ne.ndime+1))  ksyns=1
      if(lsnsb.ne.0)         ksyns=1
      if(ieche.eq.1) ksyhe=1
      neres=0                          !no residuals have to be computed 
      sftot=sfnsc+sfnsp+sfnsr          !check if a stabilization exists
     .  +sfnss+sfhec+sfdmo+shcns
     .  +shche+sftuc+shctu
      if(sftot.gt.zero2 .and.
     .  lsnsb.eq.0) neres=1
      ngpco=0                          !integ. coordinates not to be stored
      if((naxis.eq.1).or.                   !axisymmetric problem
     .  (frano(1,1).ge.zero2).or.           !Coriolis force exists
     .  (oufer.gt.0).or.                    !exact solution to be computed
     .  (kpola.gt.0)) ngpco=1               !point dependent porosity
      klsns=0                          !no SGS technique is needed (NS eqs.)
      if(sfnsc+sfnsp+sfnsr
     .  +sfnss+sfnsd+sfdmo
     .  .gt.zero2) klsns=1
      if ((sfdmo.gt.zero2)
     .  .and.(frano(1,1).gt.zero2))
     .  ldimo=1
      nerep=0                          !Laplacian of pressure is not needed
      if((ldimo.eq.1).and.
     .  (nnode.ne.nnods))
     .  nerep=1
      if (nnods.eq.ndime+1) nerep=0    !linear elements
      nhess=nerep*nnods                !second dimension of HESSI
      if(nnode.gt.ndime+1)
     .  nhess=nhess+neres*nnode 
      klshe=0                          !no SGS technique is needed (heat eq.)
      if(sfhec.gt.zero2) klshe=1
      klspc=0                          !no SGS technique is needed (free-sur.)
      if(sfpcc.gt.zero2) klspc=1
      kpeel=0                          !pressure cannot be eliminated
      if((iepns.eq.1).and.
     .  (inpco.eq.0).and.
     .  (inpes.ge.1)) kpeel=1
      ntovp=ntotv+(1-kpeel)*ntotp      !velocity+pressure variables
      kcons=0                          !properties do not change
      if((kdela.ne.0).or.
     .   (kvila.ne.0).or.
     .   (kpola.ne.0).or.
     .   (kcvla.ne.0).or.
     .   (kdila.ne.0).or.
     .   (iejhe.ne.0))     kcons=1
c
c*** Other
c
      nrule=1                          !number of integration rule
      if(nelty.eq.1) then
        if(inqua.eq.1) nrule=2
      else if(nelty.eq.2) then
        if(inqua.eq.0) nrule=3
        if(inqua.eq.1) nrule=4
      end if
      miter=max(micns,miche,micpc)     !maximum number of iterations
      nodiv='2  '                      !norm of div(u)
      if(inons.eq.1) nodiv='1  '
      if(inons.eq.3) nodiv='inf'
      if((ietns+iethe+ietfs).eq.0) then
        tstep=1.0d0
        tinit=0.0d0
        tfina=0.1d0
      end if
      if (ietns.eq.0) thens=1.0d0
      if (iethe.eq.0) thehe=1.0d0
      if (ietfs.eq.0) thepc=1.0d0
      if((iepns.eq.0).and.(igopo.eq.1)) then
        if(iephe.eq.1) then
          igopo=2
        else
          igopo=3
        end if
      end if
      if(kturb.le.2) tucou=1
c
c*** Defines the length of some principal arrays
c
      ncove=max(iupns,iepns*(2+ietns)) !velocity:        VELOC(NTOTV,NCOVE)
      ncopr=iepns*(2+ietns)            !pressure:        PRESU(NTOTP,NCOPR)
      ncote=max(iuphe,iephe*(2+iethe)) !temperature:     TEMPE(NPOIN,NCOTE)
      ncofs=max(iupfs,iepfs*(2+ietfs)) !free-surface:    FRESU(NPOIN,NCOFS)
      nrhsi=max(ntovp,npoin)           !right-hand-side: RHSID(NRHSI)
      if(twost.ne.0) ncove=ncove+1
c
c*** Factorization flags (these can be moved to the iterative process, if
c*** needed). The need for modifying the RHS due to the temporal variation
c*** of the fixity conditions is verified in UPDDIR. The flags are (A: system
c*** matrix, A_p matrix affecting prescriptions, x_p: vector of prescriptions):
c
c    0  -->  Fixity conditions modified ( A_p x_p re-computed)
c    1  -->  A factored & A_p x_p re-computed
c    2  -->  Neither A is factored nor A_p x_p re-computed      
c
      kfans=2                          !Navier-Stokes
      confl=.false.
      if(iecns.eq.1) then
        if(ilins.ne.0) then
          if(ietns.eq.0) then
            confl=.true.
          else
            if(thcon.gt.0.1) confl=.true.
          end if
        end if
      end if
      if((confl)                                            !convection on
     .  .or.(lsnsc.ne.0.and.twost.ne.-1)                    !convective stab.
     .  .or.(sfnsd.gt.zero1)                                !divergence stab.
     .  .or.(sfdmo.gt.zero1)                                !DRS technique
     .  .or.(shcns.gt.zero1)                                !DC technique
     .  .or.((kdela+kvila+kpola+kturb).ne.0)                !variable prop.
     .  .or.(iepfs.eq.1)                                    !air + liquid
     .  .or.(nscon)) then                                   !condensation     
        kfans=1 
      end if
      kfans=1                                               !Set to 1 for safety (changeable)
      kfahe=2                                               !Heat equation
      if((ieche.eq.1.and.iepns.eq.1)                        !convection on
     .  .or.(lshec.ne.0.and.sfhec.gt.zero1)                 !convective stab.
     .  .or.(shche.gt.zero1)                                !DC technique
     .  .or.((kdela+kdila+kcvla).ne.0)) then                !variable prop.
        kfahe=1
      end if

      kfapc=2                                               !Free surface
      if((iepns.eq.1)                                       !convection on
     .  .or.(lspcc.ne.0.and.sfpcc.gt.zero1)) then           !convective stab.
        kfapc=1
      end if
      kfasm=2                                               !Smoothing eqs.
      kfatu=1                                               !Turbulence eqs.
c
c*** Compute the number of subelements per element
c
      call nusube
      
      end
