      subroutine ruqclo(ndime,ngaus,posgp,weigp)
c***********************************************************************
c
c**** This routine sets up the integration constants of closed 
c**** integration rules for brick elements:
c
c         NDIME = 1             NDIME = 2             NDIME = 3
c
c     NGAUS  EXACT POL.     NGAUS  EXACT POL.     NGAUS  EXACT POL. 
c     -----  ----------     -----  ----------     -----  ----------  
c	2      q1           2 x 2     q1          2x2x2     q1   
c	3      q3           3 x 3     q3          3x3x3     q3
c	4      q3           4 x 4     q3          4x4x4     q3
c                             8       p2            20      p2 
c
c***********************************************************************
      implicit real*8(a-h,o-z)
      real*8    posgl(4), posgp(ndime,*), weigl(4), weigp(*)
      integer   inoga(27)
c
c*** 2D 8-point integration
c
      if(ndime.eq.2.and.ngaus.eq.8) then
        do igaus=5,8
          weigp(igaus-4)=-1.0/3.0
          weigp(igaus  )= 4.0/3.0
        end do
        posgp(1,5)= 0.0 
        posgp(1,6)= 1.0
        posgp(1,7)= 0.0
        posgp(1,8)=-1.0
        posgp(2,5)=-1.0
        posgp(2,6)= 0.0
        posgp(2,7)= 1.0
        posgp(2,8)= 0.0
        return
      end if
c
c*** 3D 20-point integration
c
      if(ndime.eq.3.and.ngaus.eq.20) then
        nlocs=nint(float(ngaus)**(1./3.))
        do igaus=1,8
          weigp(igaus)=-1.0
        end do
        do igaus=9,20
          weigp(igaus)= 4.0/3.0
        end do
        posgl(1)=-1.0
        posgl(2)= 0.0
        posgl(3)= 1.0
        call chaord(inoga,27)
        igaus=0
        do ilocs=1,nlocs
          do jlocs=1,nlocs
            do klocs=1,nlocs
              igaus=igaus+1
              if(inoga(igaus).le.20) then
                posgp(1,inoga(igaus))=posgl(ilocs)
                posgp(2,inoga(igaus))=posgl(jlocs)
                posgp(3,inoga(igaus))=posgl(klocs)
              end if
            end do
          end do
        end do
        return
      end if
c
c*** Rules obtained from one-dimensional integration
c
      if(ndime.eq.1) then
        nlocs=ngaus
      else if(ndime.eq.2) then
        nlocs=nint(sqrt(float(ngaus)))
      else
        nlocs=nint(float(ngaus)**(1./3.))
      end if

      call chaord(inoga,nlocs**ndime)

      if(nlocs.eq.2) then
        posgl(1)=-1.0
        posgl(2)= 1.0
        weigl(1)= 1.0
        weigl(2)= 1.0
      else if(nlocs.eq.3) then
        posgl(1)=-1.0
        posgl(2)= 0.0
        posgl(3)= 1.0
        weigl(1)= 1.0/3.0
        weigl(2)= 4.0/3.0
        weigl(3)= 1.0/3.0
      else if(nlocs.eq.4) then
        posgl(1)=-1.0
        posgl(2)=-1.0/3.0
        posgl(3)= 1.0/3.0
        posgl(4)= 1.0
        weigl(1)= 1.0/4.0
        weigl(2)= 3.0/4.0
        weigl(3)= 3.0/4.0
        weigl(4)= 1.0/4.0
      end if

      if(ndime.eq.1) then
        igaus=0
        do ilocs=1,nlocs
          igaus=igaus+1
          weigp(  igaus)=weigl(ilocs)
          posgp(1,igaus)=posgl(ilocs)
        end do
      else if(ndime.eq.2) then
        igaus=0
        do ilocs=1,nlocs
          do jlocs=1,nlocs
            igaus=igaus+1
            weigp(  inoga(igaus))=weigl(ilocs)*weigl(jlocs)
            posgp(1,inoga(igaus))=posgl(ilocs)
            posgp(2,inoga(igaus))=posgl(jlocs)
          end do
        end do
        if(ngaus.eq.5) then                                 !  4       3
          do igaus=1,4                                      !
            weigp(igaus)=1.0/3.0                            !      5
          end do                                            !
          weigp(  5)=8.0/3.0                                !  1       2
          posgp(1,5)=0.0
          posgp(2,5)=0.0
        end if
      else if(ndime.eq.3) then
        igaus=0
        do ilocs=1,nlocs
          do jlocs=1,nlocs
            do klocs=1,nlocs
              igaus=igaus+1
              weigp(  inoga(igaus))=weigl(ilocs)*weigl(jlocs)
     .                             *weigl(klocs)
              posgp(1,inoga(igaus))=posgl(ilocs)
              posgp(2,inoga(igaus))=posgl(jlocs)
              posgp(3,inoga(igaus))=posgl(klocs)
            end do
          end do
        end do
        if(ngaus.eq.9) then                                 !    8------7
          do igaus=1,8                                      !   /|     /|
            weigp(igaus)=weigp(igaus)/3.0                   !  5------6 |
          end do                                            !  | | 9  | |
          weigp(  9)=16.0/3.0                               !  | 4----|-3
          posgp(1,9)=0.0                                    !  |/     |/
          posgp(2,9)=0.0                                    !  1------2
          posgp(3,9)=0.0
        end if
      end if

      end
