      subroutine shcos3(    s,    t,    z,nelty,nnode,
     .                  shape,deriv,heslo,neres)
c***********************************************************************
c
c**** This routine evaluates shape functions and their first and
c**** second derivatives 3-d standar continuous interpolation
c**** elements.
c
c               TETRAHEDRA:  4  5  10  11  15  &  20  nodes
c               HEXAHEDRA:   8  9  20      27  &  64  nodes
c
c***********************************************************************
      implicit real*8(a-h,o-z)
      real*8 deriv(3,nnode), shape(nnode), heslo(6,nnode)

      istop=0
c
c*** Linear & Linear + bubble tetrahedron 
c
      if(nnode.eq.4.or.nnode.eq.5) then
        shape(   1) = 1.0-s-t-z
        deriv(1, 1) =-1.0
        deriv(2, 1) =-1.0
        deriv(3, 1) =-1.0
        shape(   2) = s
        deriv(1, 2) = 1.0
        shape(   3) = t
        deriv(2, 3) = 1.0
        shape(   4) = z
        deriv(3, 4) = 1.0
        if(nnode.eq.5) then
          a1=1.-s-t-z
          shape(   5) = 256.*a1*s*t*z           
          deriv(1, 5) = 256.*t*z*(1.0-2.0*s-t-z)
          deriv(2, 5) = 256.*s*z*(1.0-2.0*t-s-z)
          deriv(3, 5) = 256.*s*t*(1.0-2.0*z-s-t)
          if(neres.eq.1) then
            heslo(1,5)=-512.0*t*z
            heslo(2,5)=-512.0*s*z
            heslo(3,5)=-512.0*s*t
            heslo(4,5)= 256.0*z*(1.-2.*s-2.*t-z)
            heslo(5,5)= 256.0*t*(1.-2.*s-2.*z-t)
            heslo(6,5)= 256.0*s*(1.-2.*t-2.*z-s)
          end if
          do i = 1,4
            shape(i) = shape(i)-shape(5)/4.
            do j = 1,3
              deriv(j,i) = deriv(j,i)-deriv(j,5)/4.
            enddo
            if(neres.eq.1) then
              do j = 1,6
                heslo(j,i) = -heslo(j,5)/4.
              enddo
            endif
          enddo
        endif
c
c*** Quadratic & quadratic + bubble tetrahedron 
c
      else if(nnode.eq.10.or.nnode.eq.11) then
        a1=1.-s-t-z
        a2=s
        a3=t
        a4=z
        shape(   1) = (2.*a1-1.)*a1
        deriv(1, 1) = 1.-4.*a1
        deriv(2, 1) = 1.-4.*a1
        deriv(3, 1) = 1.-4.*a1
        if(neres.eq.1) then
          do i = 1,6
            heslo(i, 1) = 4.
          enddo
        end if
        shape(   2) = (2.*a2-1.)*a2
        deriv(1, 2) = 4.*a2-1.
        if(neres.eq.1) heslo(1, 2) = 4.
        shape(   3) = (2.*a3-1.)*a3
        deriv(2, 3) = 4.*a3-1.
        if(neres.eq.1) heslo(2, 3) = 4.
        shape(   4) = (2.*a4-1.)*a4
        deriv(3, 4) = 4.*a4-1.
        if(neres.eq.1) heslo(3, 4) = 4.
        shape(   5) = 4.*a1*a2
        deriv(1, 5) = 4.*(a1-a2)
        deriv(2, 5) =-4.*a2
        deriv(3, 5) =-4.*a2
        if(neres.eq.1) then
          heslo(1, 5) =-8.
          heslo(4, 5) =-4.
          heslo(5, 5) =-4.
        end if
        shape(   6) = 4.*a2*a3
        deriv(1, 6) = 4.*a3
        deriv(2, 6) = 4.*a2
        if(neres.eq.1) heslo(4, 6) = 4.
        shape(   7) = 4.*a1*a3
        deriv(1, 7) =-4.*a3
        deriv(2, 7) = 4.*(a1-a3)
        deriv(3, 7) =-4.*a3
        if(neres.eq.1) then
          heslo(2, 7) =-8.
          heslo(4, 7) =-4.
          heslo(6, 7) =-4.
        end if
        shape(   8) = 4.*a1*a4
        deriv(1, 8) =-4.*a4
        deriv(2, 8) =-4.*a4
        deriv(3, 8) = 4.*(a1-a4)
        if(neres.eq.1) then
          heslo(3, 8) =-8.
          heslo(5, 8) =-4.
          heslo(6, 8) =-4.
        end if
        shape(   9) = 4.*a2*a4
        deriv(1, 9) = 4.*a4
        deriv(3, 9) = 4.*a2
        if(neres.eq.1) heslo(5, 9) = 4.
        shape(  10) = 4.*a3*a4
        deriv(2,10) = 4.*a4
        deriv(3,10) = 4.*a3
        if(neres.eq.1) heslo(6,10) = 4.
        if(nnode.eq.11) then
          shape(  11) = 256.*a1*a2*a3*a4
          deriv(1,11) = 256.*(-a2*a3*a4+a1*a3*a4)
          deriv(2,11) = 256.*(-a2*a3*a4+a1*a2*a4)
          deriv(3,11) = 256.*(-a2*a3*a4+a1*a2*a3)
          if(neres.eq.1) then
            do i = 1,6
              heslo(i,11) =-64.*(2.*a1-1.)
            enddo
          endif
          do i = 1,10
            if(i.le.4) then
              shape(i) = shape(i)+shape(11)/8.
              do j = 1,3
                deriv(j,i) = deriv(j,i)+deriv(j,11)/8.
              enddo
              if(neres.eq.1) then
                do j = 1,6
                  heslo(j,i) = heslo(j,i)+heslo(j,11)/8.
                enddo
              endif
            else
              shape(i) = shape(i)-shape(11)/4.
              do j = 1,3
                deriv(j,i) = deriv(j,i)-deriv(j,11)/4.
              enddo
              if(neres.eq.1) then
                do j = 1,6
                  heslo(j,i) = heslo(j,i)-heslo(j,11)/4.
                enddo
              endif  
            endif
          enddo
        endif
c
c*** Quadratic tetrahedron with central & face bubbles
c
      else if(nnode.eq.15) then
        a1=1.-s-t-z
        a2=s
        a3=t
        a4=z
        shape(  11) = 27.*a1*a2*a3
        deriv(1,11) = 27.*a3*(a1-a2)
        deriv(2,11) = 27.*a2*(a1-a3)
        deriv(3,11) =-27.*a2*a3
        if(neres.eq.1) then
          heslo(1,11) =-54.*a3
          heslo(2,11) =-54.*a2
          heslo(4,11) = 27.*(a1-a2-a3)
          heslo(5,11) =-27.*a3
          heslo(6,11) =-27.*a2
        endif
        shape(  12) = 27.*a1*a2*a4
        deriv(1,12) = 27.*a4*(a1-a2)
        deriv(2,12) =-27.*a2*a4
        deriv(3,12) = 27.*a2*(a1-a4)
        if(neres.eq.1) then
          heslo(1,12) =-54.*a4
          heslo(3,12) =-54.*a2
          heslo(4,12) =-27.*a4
          heslo(5,12) = 27.*(a1-a1-a4)
          heslo(6,12) =-27.*a2
        endif
        shape(  13) = 27.*a2*a3*a4
        deriv(1,13) = 27.*a3*a4
        deriv(2,13) = 27.*a2*a4
        deriv(3,13) = 27.*a2*a3
        if(neres.eq.1) then
          heslo(4,13) = 27.*a4
          heslo(5,13) = 27.*a3
          heslo(6,13) = 27.*a2
        endif
        shape(  14) = 27.*a1*a3*a4
        deriv(1,14) =-27.*a3*a4
        deriv(2,14) = 27.*a4*(a1-a3)
        deriv(3,14) = 27.*a3*(a1-a4)
        if(neres.eq.1) then
          heslo(2,14) =-54.*a4
          heslo(3,14) =-54.*a3
          heslo(4,14) =-27.*a4
          heslo(5,14) =-27.*a3
          heslo(6,14) = 27.*(a1-a3-a4)
        endif
        shape(   1) = (2.*a1-1.)*a1
     .               +(shape(11)+shape(12)+shape(14))/9.
        deriv(1, 1) = 1.-4.*a1
     .               +(deriv(1,11)+deriv(1,12)+deriv(1,14))/9.
        deriv(2, 1) = 1.-4.*a1
     .               +(deriv(2,11)+deriv(2,12)+deriv(2,14))/9.
        deriv(3, 1) = 1.-4.*a1
     .               +(deriv(3,11)+deriv(3,12)+deriv(3,14))/9.
        if(neres.eq.1) then
          do i = 1,6
            heslo(i,1) = 4.
     .                  +(heslo(i,11)+heslo(i,12)+heslo(i,14))/9.
          enddo
        end if
        shape(   2) = (2.*a2-1.)*a2
     .               +(shape(11)+shape(12)+shape(13))/9.
        deriv(1, 2) = 4.*a2-1.
     .               +(deriv(1,11)+deriv(1,12)+deriv(1,13))/9.
        deriv(2, 2) = (deriv(2,11)+deriv(2,12)+deriv(2,13))/9.
        deriv(3, 2) = (deriv(3,11)+deriv(3,12)+deriv(3,13))/9.
        if(neres.eq.1) then
          heslo(1, 2) = 4.
          do i = 1,6
            heslo(i,2) = heslo(i,2)
     .                  +(heslo(i,11)+heslo(i,12)+heslo(i,13))/9.
          enddo
        endif
        shape(   3) = (2.*a3-1.)*a3
     .               +(shape(11)+shape(13)+shape(14))/9.
        deriv(1, 3) = (deriv(1,11)+deriv(1,13)+deriv(1,14))/9.
        deriv(2, 3) =-1.+4.*a3
     .               +(deriv(2,11)+deriv(2,13)+deriv(2,14))/9.
        deriv(3, 3) = (deriv(3,11)+deriv(3,13)+deriv(3,14))/9.
        if(neres.eq.1) then
          heslo(2, 3) = 4.
          do i = 1,6
            heslo(i,3) = heslo(i,3)
     .                  +(heslo(i,11)+heslo(i,13)+heslo(i,14))/9.
          enddo
        endif
        shape(   4) = (2.*a4-1.)*a4
     .               +(shape(12)+shape(13)+shape(14))/9.
        deriv(1, 4) = (deriv(1,12)+deriv(1,13)+deriv(1,14))/9.
        deriv(2, 4) = (deriv(2,12)+deriv(2,13)+deriv(2,14))/9.
        deriv(3, 4) = 4.*a4-1.
     .               +(deriv(3,12)+deriv(3,13)+deriv(3,14))/9.
        if(neres.eq.1) then
          heslo(3, 4) = 4.
          do i = 1,6
            heslo(i,4) = heslo(i,4)
     .                  +(heslo(i,12)+heslo(i,13)+heslo(i,14))/9.
          enddo
        endif
        shape(   5) = 4.*a1*a2
     .               -(shape(11)+shape(12))*4./9.
        deriv(1, 5) = 4.*(a1-a2)
     .               -(deriv(1,11)+deriv(1,12))*4./9.
        deriv(2, 5) =-4.*a2
     .               -(deriv(2,11)+deriv(2,12))*4./9.
        deriv(3, 5) =-4.*a2
     .               -(deriv(3,11)+deriv(3,12))*4./9.
        if(neres.eq.1) then
          heslo(1, 5) =-8.
          heslo(4, 5) =-4.
          heslo(5, 5) =-4.
          do i = 1,6
            heslo(i,5) = heslo(i,5)
     .                  -(heslo(i,11)+heslo(i,12))*4./9.
          enddo
        end if
        shape(   6) = 4.*a2*a3
     .               -(shape(11)+shape(13))*4./9.
        deriv(1, 6) = 4.*a3
     .               -(deriv(1,11)+deriv(1,13))*4./9.
        deriv(2, 6) = 4.*a2
     .               -(deriv(2,11)+deriv(2,13))*4./9.
        deriv(3, 6) = 0.
     .               -(deriv(3,11)+deriv(3,13))*4./9.
        if(neres.eq.1) then
          heslo(4, 6) = 4.
          do i = 1,6
            heslo(i,6) = heslo(i,6)
     .                  -(heslo(i,11)+heslo(i,13))*4./9.
          enddo
        endif
        shape(   7) = 4.*a1*a3
     .               -(shape(11)+shape(14))*4./9.
        deriv(1, 7) =-4.*a3
     .               -(deriv(1,11)+deriv(1,14))*4./9.
        deriv(2, 7) = 4.*(a1-a3)
     .               -(deriv(2,11)+deriv(2,14))*4./9.
        deriv(3, 7) =-4.*a3
     .               -(deriv(3,11)+deriv(3,14))*4./9.
        if(neres.eq.1) then
          heslo(2, 7) =-8.
          heslo(4, 7) =-4.
          heslo(6, 7) =-4.
          do i = 1,6
            heslo(i,7) = heslo(i,7)
     .                  -(heslo(i,11)+heslo(i,14))*4./9.
          enddo
        end if
        shape(   8) = 4.*a1*a4
     .               -(shape(12)+shape(14))*4./9.
        deriv(1, 8) =-4.*a4
     .               -(deriv(1,12)+deriv(1,14))*4./9.
        deriv(2, 8) =-4.*a4
     .               -(deriv(2,12)+deriv(2,14))*4./9.
        deriv(3, 8) = 4.*(a1-a4)
     .               -(deriv(3,12)+deriv(3,14))*4./9.
        if(neres.eq.1) then
          heslo(3, 8) =-8.
          heslo(5, 8) =-4.
          heslo(6, 8) =-4.
          do i = 1,6
            heslo(i,8) = heslo(i,8)
     .                  -(heslo(i,12)+heslo(i,14))*4./9.
          enddo
        end if
        shape(   9) = 4.*a2*a4
     .               -(shape(12)+shape(13))*4./9.
        deriv(1, 9) = 4.*a4
     .               -(deriv(1,12)+deriv(1,13))*4./9.
        deriv(2, 9) = 0.
     .               -(deriv(2,12)+deriv(2,13))*4./9.
        deriv(3, 9) = 4.*a2
     .               -(deriv(3,12)+deriv(3,13))*4./9.
        if(neres.eq.1) then
          heslo(5, 9) = 4.
          do i = 1,6
            heslo(i,9) = heslo(i,9)
     .                  -(heslo(i,12)+heslo(i,13))*4./9.
          enddo
        endif
        shape(  10) = 4.*a3*a4
     .               -(shape(13)+shape(14))*4./9.
        deriv(1,10) =-(deriv(1,13)+deriv(1,14))*4./9.
        deriv(2,10) = 4.*a4
     .               -(deriv(2,13)+deriv(2,14))*4./9.
        deriv(3,10) = 4.*a3
     .               -(deriv(3,13)+deriv(3,14))*4./9.
        if(neres.eq.1) then
          heslo(6,10) = 4.
          do i = 1,6
            heslo(i,10) = heslo(i,10)
     .                  -(heslo(i,13)+heslo(i,14))*4./9.
          enddo
        endif
        shape(  15) = 256.*a1*a2*a3*a4
        deriv(1,15) = 256.*(-a2*a3*a4+a1*a3*a4)
        deriv(2,15) = 256.*(-a2*a3*a4+a1*a2*a4)
        deriv(3,15) = 256.*(-a2*a3*a4+a1*a2*a3)
        if(neres.eq.1) then
          heslo(1,15) = -512.*a3*a4
          heslo(2,15) = -512.*a2*a4
          heslo(3,15) = -512.*a2*a3
          heslo(4,15) = 256.*(a1*a4-a2*a4-a3*a4)
          heslo(5,15) = 256.*(a1*a3-a2*a3-a3*a4)
          heslo(6,15) = 256.*(a1*a2-a2*a3-a2*a4)
        end if
        do i = 1,14
          if(i.le.4) then
            shape(i) = shape(i)-shape(15)/64.
            do j = 1,3
              deriv(j,i) = deriv(j,i)-deriv(j,15)/64.
            enddo
            if(neres.eq.1) then
              do j = 1,6
                heslo(j,i) = heslo(j,i)-heslo(j,15)/64.
              enddo
            endif
          elseif(i.le.10) then
            shape(i) = shape(i)+shape(15)/8.
            do j = 1,3
              deriv(j,i) = deriv(j,i)+deriv(j,15)/8.
            enddo
            if(neres.eq.1) then
              do j = 1,6
                heslo(j,i) = heslo(j,i)+heslo(j,15)/8.
              enddo
            endif
          else
            shape(i) = shape(i)-shape(15)*27./64.
            do j = 1,3
              deriv(j,i) = deriv(j,i)-deriv(j,15)*27./64.
            enddo
            if(neres.eq.1) then
              do j = 1,6
                heslo(j,i) = heslo(j,i)-heslo(j,15)*27./64.
              enddo
            endif
          endif
        enddo
c
c*** Cubic tetrahedron
c        
      else if(nelty.eq.2.and.nnode.eq.20) then
        a=4.5
        p1=1-s-t-z
        p2=2.0/3.0-s-t-z
        p3=1.0/3.0-s-t-z
        z1=2.0/3.0-z
        z2=1.0/3.0-z
        s1=2.0/3.0-s
        s2=1.0/3.0-s
        t1=2.0/3.0-t
        t2=1.0/3.0-t

        shape(   1) = a*p1*p2*p3
        deriv(1, 1) =-a*(p1*p2+p1*p3+p2*p3)
        deriv(2, 1) =-a*(p1*p2+p1*p3+p2*p3)
        deriv(3, 1) =-a*(p1*p2+p1*p3+p2*p3)
        shape(   2) = a*z*z1*z2
        deriv(1, 2) = 0.0
        deriv(2, 2) = 0.0
        deriv(3, 2) =-a*(z*z1+z*z2-z1*z2)
        shape(   3) = a*s*s1*s2
        deriv(1, 3) =-a*(s*s1+s*s2-s1*s2)
        deriv(2, 3) = 0.0
        deriv(3, 3) = 0.0
        shape(   4) = a*t*t1*t2
        deriv(1, 4) = 0.0
        deriv(2, 4) =-a*(t*t1+t*t2-t1*t2)
        deriv(3, 4) = 0.0
        shape(   5) =3.*a*p1*p2*t
        deriv(1, 5) =3.*a*(-p2*t-p1*t)
        deriv(2, 5) =3.*a*(-p2*t-p1*t+p1*p2)
        deriv(3, 5) =3.*a*(-p2*t-p1*t)      
        shape(   6) =3.*a*p1*p2*z
        deriv(1, 6) =3.*a*(-p2*z-p1*z)      
        deriv(2, 6) =3.*a*(-p2*z-p1*z)
        deriv(3, 6) =3.*a*(-p2*z-p1*z+p1*p2)      
        shape(   7) =3.*a*p1*p2*s
        deriv(1, 7) =3.*a*(-p2*s-p1*s+p1*p2)
        deriv(2, 7) =3.*a*(-p2*s-p1*s)
        deriv(3, 7) =3.*a*(-p2*s-p1*s)
        shape(   8) =-3.*a*p1*t2*t
        deriv(1, 8) = 3.*a*(t2*t)
        deriv(2, 8) = 3.*a*(t2*t+p1*t-p1*t2)
        deriv(3, 8) = 3.*a*(t2*t)
        shape(   9) =-3.*a*p1*s2*s
        deriv(1, 9) = 3.*a*(s2*s+p1*s-p1*s2)
        deriv(2, 9) = 3.*a*(s2*s)
        deriv(3, 9) = 3.*a*(s2*s)
        shape(  10) =-3.*a*p1*z2*z
        deriv(1,10) = 3.*a*(z2*z)
        deriv(2,10) = 3.*a*(z2*z)
        deriv(3,10) = 3.*a*(z2*z+p1*z-p1*z2)
        shape(  11) =-3.*a*t2*t*z
        deriv(1,11) = 0.0
        deriv(2,11) =-3.*a*(t2*z-t*z)
        deriv(3,11) =-3.*a*t2*t
        shape(  12) =-3.*a*z2*t*z
        deriv(1,12) = 0.0
        deriv(2,12) =-3.*a*z2*z
        deriv(3,12) =-3.*a*(t*z2-t*z)
        shape(  13) =-3.*a*z2*s*z
        deriv(1,13) =-3.*a*z2*z
        deriv(2,13) = 0.0
        deriv(3,13) =-3.*a*(s*z2-s*z)
        shape(  14) =-3.*a*s2*s*z
        deriv(1,14) =-3.*a*(s2*z-s*z)
        deriv(2,14) = 0.0
        deriv(3,14) =-3.*a*s2*s
        shape(  15) =-3.*a*s2*s*t
        deriv(1,15) =-3.*a*(s2*t-s*t)
        deriv(2,15) =-3.*a*s2*s
        deriv(3,15) = 0.0
        shape(  16) =-3.*a*t2*t*s
        deriv(1,16) =-3.*a*t2*t
        deriv(2,16) =-3.*a*(t2*s-s*t)
        deriv(3,16) = 0.0
        shape(  17) =27.*p1*t*z
        deriv(1,17) =-27.*t*z
        deriv(2,17) =27.*(p1*z-t*z)
        deriv(3,17) =27.*(p1*t-t*z)
        shape(  18) =27.*p1*s*z
        deriv(1,18) =27.*(p1*z-s*z)
        deriv(2,18) =-27.*s*z
        deriv(3,18) =27.*(p1*s-s*z)
        shape(  19) =27.*p1*s*t
        deriv(1,19) =27.*(p1*t-s*t)
        deriv(2,19) =27.*(p1*s-s*t)
        deriv(3,19) =-27.*s*t
        shape(  20) =27.*s*t*z
        deriv(1,20) =27.*z*t
        deriv(2,20) =27.*s*z
        deriv(3,20) =27.*s*t
        if (neres.eq.1) then
          
          heslo(1, 1) =2.*a*(p1+p2+p3)
          heslo(2, 1) =2.*a*(p1+p2+p3)
          heslo(3, 1) =2.*a*(p1+p2+p3)
          heslo(4, 1) =2.*a*(p1+p2+p3)
          heslo(5, 1) =2.*a*(p1+p2+p3)
          heslo(6, 1) =2.*a*(p1+p2+p3)
          
          heslo(3, 2) =2.*a*(z-z1-z2)
          
          heslo(1, 3) =2.*a*(s-s1-s2)
                       
          heslo(2, 4) =2.*a*(t-t1-t2)

          heslo(1, 5) =6.*a*t
          heslo(2, 5) =6.*a*(-p1-p2+t)
          heslo(3, 5) =6.*a*t
          heslo(4, 5) =3.*a*(-p2-p1+2*t)
          heslo(5, 5) =6.*a*t
          heslo(6, 5) =3.*a*(-p2-p1+2*t)
                       
          heslo(1, 6) =6.*a*z
          heslo(2, 6) =6.*a*z
          heslo(3, 6) =6.*a*(-p1-p2+z)
          heslo(4, 6) =6.*a*z
          heslo(5, 6) =3.*a*(-p2-p1+2*z)
          heslo(6, 6) =3.*a*(-p2-p1+2*z)
                       
          heslo(1, 7) =6.*a*(-p1-p2+s)
          heslo(2, 7) =6.*a*s
          heslo(3, 7) =6.*a*s
          heslo(4, 7) =3.*a*(-p2-p1+2*s)
          heslo(5, 7) =3.*a*(-p2-p1+2*s)
          heslo(6, 7) =6.*a*s
                       
          heslo(1, 8) = 0.0
          heslo(2, 8) =6.*a*(t2-t+p1)
          heslo(3, 8) = 0.0
          heslo(4, 8) =3.*a*(t2-t)
          heslo(5, 8) = 0.0
          heslo(6, 8) =3.*a*(t2-t)
                       
          heslo(1, 9) =6.*a*(s2-s+p1)
          heslo(2, 9) = 0.0
          heslo(3, 9) = 0.0
          heslo(4, 9) =3.*a*(s2-s)
          heslo(5, 9) =3.*a*(s2-s)
          heslo(6, 9) = 0.0
                       
          heslo(1,10) = 0.0
          heslo(2,10) = 0.0
          heslo(3,10) =6.*a*(z2-z+p1)
          heslo(4,10) = 0.0
          heslo(5,10) =3.*a*(z2-z)           
          heslo(6,10) =3.*a*(z2-z)
                       
          heslo(1,11) = 0.0
          heslo(2,11) =6.*a*z
          heslo(3,11) = 0.0
          heslo(4,11) = 0.0
          heslo(5,11) = 0.0
          heslo(6,11) =3.*a*(t-t2)
                       
          heslo(1,12) = 0.0
          heslo(2,12) = 0.0
          heslo(3,12) =6.*a*t
          heslo(4,12) = 0.0
          heslo(5,12) = 0.0
          heslo(6,12) =3.*a*(z-z2)
                       
          heslo(1,13) = 0.0
          heslo(2,13) = 0.0
          heslo(3,13) =6.*a*s
          heslo(4,13) = 0.0
          heslo(5,13) =3.*a*(z-z2)
          heslo(6,13) = 0.0
                       
          heslo(1,14) =6.*a*z
          heslo(2,14) = 0.0
          heslo(3,14) = 0.0
          heslo(4,14) = 0.0
          heslo(5,14) =3.*a*(s-s2)
          heslo(6,14) = 0.0
                       
          heslo(1,15) =6.*a*t
          heslo(2,15) = 0.0
          heslo(3,15) = 0.0
          heslo(4,15) =3.*a*(s-s2)
          heslo(5,15) = 0.0
          heslo(6,15) = 0.0
                       
          heslo(1,16) = 0.0
          heslo(2,16) =6.*a*s
          heslo(3,16) = 0.0
          heslo(4,16) =3.*a*(t-t2)
          heslo(5,16) = 0.0
          heslo(6,16) = 0.0
                       
          heslo(1,17) = 0.0
          heslo(2,17) =-54.*z
          heslo(3,17) =-54.*t
          heslo(4,17) =-27.*z
          heslo(5,17) =-27.*t
          heslo(6,17) =27.*(p1-z-t)
                       
          heslo(1,18) =-54.*z
          heslo(2,18) = 0.0
          heslo(3,18) =-54.*s
          heslo(4,18) =-27.*z
          heslo(5,18) =-27.*s
          heslo(6,18) =27.*(p1-z-s)
                       
          heslo(1,19) =-54.*t
          heslo(2,19) =-54.*s
          heslo(3,19) = 0.0
          heslo(4,19) =27.*(p1-t-s)
          heslo(5,19) =-27.*t
          heslo(6,19) =-27.*s
                       
          heslo(1,20) = 0.0
          heslo(2,20) = 0.0
          heslo(3,20) = 0.0
          heslo(4,20) =27.*z
          heslo(5,20) =27.*t
          heslo(6,20) =27.*s
        end if
c
c*** Trilinear & trilinear + bubble brick 
c        
      else if(nnode.eq.8.or.nnode.eq.9) then
        sm = 0.5*(1.0-s)
        tm = 0.5*(1.0-t)
        zm = 0.5*(1.0-z)
        sp = 0.5*(1.0+s)
        tp = 0.5*(1.0+t)
        zp = 0.5*(1.0+z)
        shape(   1) = sm*tm*zm
        deriv(1, 1) =-.5*tm*zm
        deriv(2, 1) =-.5*sm*zm
        deriv(3, 1) =-.5*sm*tm
        if(neres.eq.1) then
          heslo(4, 1) = .25*zm
          heslo(5, 1) = .25*tm
          heslo(6, 1) = .25*sm
        end if
        shape(   2) = sp*tm*zm
        deriv(1, 2) = .5*tm*zm
        deriv(2, 2) =-.5*sp*zm
        deriv(3, 2) =-.5*sp*tm
        if(neres.eq.1) then
          heslo(4, 2) =-.25*zm
          heslo(5, 2) =-.25*tm
          heslo(6, 2) = .25*sp
        end if
        shape(   3) = sp*tp*zm
        deriv(1, 3) = .5*tp*zm
        deriv(2, 3) = .5*sp*zm
        deriv(3, 3) =-.5*sp*tp
        if(neres.eq.1) then
          heslo(4, 3) = .25*zm
          heslo(5, 3) =-.25*tp
          heslo(6, 3) =-.25*sp
        end if
        shape(   4) = sm*tp*zm
        deriv(1, 4) =-.5*tp*zm
        deriv(2, 4) = .5*sm*zm
        deriv(3, 4) =-.5*sm*tp
        if(neres.eq.1) then
          heslo(4, 4) =-.25*zm
          heslo(5, 4) = .25*tp
          heslo(6, 4) =-.25*sm
        end if
        shape(   5) = sm*tm*zp
        deriv(1, 5) =-.5*tm*zp
        deriv(2, 5) =-.5*sm*zp
        deriv(3, 5) = .5*sm*tm
        if(neres.eq.1) then
          heslo(4, 5) = .25*zp
          heslo(5, 5) =-.25*tm
          heslo(6, 5) =-.25*sm
        end if
        shape(   6) = sp*tm*zp 
        deriv(1, 6) = .5*tm*zp
        deriv(2, 6) =-.5*sp*zp
        deriv(3, 6) = .5*sp*tm
        if(neres.eq.1) then
          heslo(4, 6) =-.25*zp
          heslo(5, 6) = .25*tm
          heslo(6, 6) =-.25*sp
        end if
        shape(   7) = sp*tp*zp
        deriv(1, 7) = .5*tp*zp
        deriv(2, 7) = .5*sp*zp
        deriv(3, 7) = .5*sp*tp
        if(neres.eq.1) then
          heslo(4, 7) = .25*zp
          heslo(5, 7) = .25*tp
          heslo(6, 7) = .25*sp
        end if
        shape(   8) = sm*tp*zp
        deriv(1, 8) =-.5*tp*zp
        deriv(2, 8) = .5*sm*zp
        deriv(3, 8) = .5*sm*tp
        if(neres.eq.1) then
          heslo(4, 8) =-.25*zp
          heslo(5, 8) =-.25*tp
          heslo(6, 8) = .25*sm
        end if
        if(nnode.eq.9) then
          shape(   9) = (1.-s*s)*(1.-t*t)*(1.-z*z)
          deriv(1, 9) = -2.*s*(1.-t*t)*(1.-z*z)
          deriv(2, 9) = -2.*t*(1.-s*s)*(1.-z*z)
          deriv(3, 9) = -2.*z*(1.-s*s)*(1.-t*t)
          if(neres.eq.1) then
            heslo(1,9) = -2.*(1.-t*t)*(1.-z*z)
            heslo(2,9) = -2.*(1.-s*s)*(1.-z*z)
            heslo(3,9) = -2.*(1.-s*s)*(1.-t*t)
            heslo(4,9) =  4.*s*t*(1.-z*z)
            heslo(5,9) =  4.*s*z*(1.-t*t)
            heslo(6,9) =  4.*t*z*(1.-s*s)
          endif
          do i = 1,8
            shape(i) = shape(i)-.125*shape(9)
            do j = 1,3
              deriv(j,i) = deriv(j,i)-.125*deriv(j,9)
            enddo
            if(neres.eq.1) then
              do j = 1,6
                heslo(j,i) = heslo(j,i)-.125*heslo(j,9)
              enddo
            endif
          enddo
        endif
c
c*** Serendipid brick
c        
      else if(nelty.eq.1.and.nnode.eq.20) then
        sm = 0.5*(1.-s)
        tm = 0.5*(1.-t)
        zm = 0.5*(1.-z)
        sp = 0.5*(1.+s)
        tp = 0.5*(1.+t)
        zp = 0.5*(1.+z)
        ss = 0.5*(1.-s*s)
        tt = 0.5*(1.-t*t)
        zz = 0.5*(1.-z*z)
        s2 = -2.*s
        t2 = -2.*t
        z2 = -2.*z
        aa          = +sm+tm+zm-   2.5
        shape(   1) = +sm*tm*zm*(aa+aa)
        deriv(1, 1) = -   tm*zm*(aa+sm)
        deriv(2, 1) = -sm   *zm*(aa+tm)
        deriv(3, 1) = -sm*tm   *(aa+zm)
        if(neres.eq.1) then
          heslo(1, 1) = +tm*zm
          heslo(2, 1) = +sm*zm
          heslo(3, 1) = +sm*tm
          heslo(4, 1) = +zm*(aa+sm+tm)/2.
          heslo(5, 1) = +tm*(aa+sm+zm)/2.
          heslo(6, 1) = +sm*(aa+tm+zm)/2.
        endif
        aa          = +sp+tm+zm-   2.5
        shape(   2) = +sp*tm*zm*(aa+aa)
        deriv(1, 2) = +   tm*zm*(aa+sp)
        deriv(2, 2) = -sp   *zm*(aa+tm)
        deriv(3, 2) = -sp*tm   *(aa+zm)
        if(neres.eq.1) then
          heslo(1, 2) = +tm*zm
          heslo(2, 2) = +sp*zm
          heslo(3, 2) = +sp*tm
          heslo(4, 2) = -zm*(aa+sp+tm)/2.
          heslo(5, 2) = -tm*(aa+tm+zm)/2.
          heslo(6, 2) = +sp*(aa+tm+zm)/2.
        endif
        aa          = +sp+tp+zm-   2.5
        shape(   3) = +sp*tp*zm*(aa+aa)
        deriv(1, 3) = +   tp*zm*(aa+sp)
        deriv(2, 3) = +sp   *zm*(aa+tp)
        deriv(3, 3) = -sp*tp   *(aa+zm)
        if(neres.eq.1) then
          heslo(1, 3) = +tp*zm
          heslo(2, 3) = +sp*zm
          heslo(3, 3) = +sp*tp
          heslo(4, 3) = +zm*(aa+sp+tp)/2.
          heslo(5, 3) = -tp*(aa+sp+zm)/2.
          heslo(6, 3) = -sp*(aa+tp+zm)/2.
        endif
        aa          = +sm+tp+zm-   2.5
        shape(   4) = +sm*tp*zm*(aa+aa)
        deriv(1, 4) = -   tp*zm*(aa+sm)
        deriv(2, 4) = +sm   *zm*(aa+tp)
        deriv(3, 4) = -sm*tp   *(aa+zm)
        if(neres.eq.1) then
          heslo(1, 4) = +tp*zm
          heslo(2, 4) = +sm*zm
          heslo(3, 4) = +sm*tp
          heslo(4, 4) = -zm*(aa+sm+tp)/2.
          heslo(5, 4) = +tp*(aa+sm+zm)/2.
          heslo(6, 4) = -sm*(aa+tp+zm)/2.
        endif
        aa          = +sm+tm+zp-   2.5
        shape(   5) = +sm*tm*zp*(aa+aa)
        deriv(1, 5) = -   tm*zp*(aa+sm)
        deriv(2, 5) = -sm   *zp*(aa+tm)
        deriv(3, 5) = +sm*tm   *(aa+zp)
        if(neres.eq.1) then
          heslo(1, 5) = +tm*zp
          heslo(2, 5) = +sm*zp
          heslo(3, 5) = +sm*tm
          heslo(4, 5) = +zp*(aa+sm+tm)/2.
          heslo(5, 5) = -tm*(aa+sm+zp)/2.
          heslo(6, 5) = -sm*(aa+tm+zp)/2.
        endif
        aa          = +sp+tm+zp-   2.5
        shape(   6) = +sp*tm*zp*(aa+aa)
        deriv(1, 6) = +   tm*zp*(aa+sp)
        deriv(2, 6) = -sp   *zp*(aa+tm)
        deriv(3, 6) = +sp*tm   *(aa+zp)
        if(neres.eq.1) then
          heslo(1, 6) = +tm*zp
          heslo(2, 6) = +sp*zp
          heslo(3, 6) = +sp*tm
          heslo(4, 6) = -zp*(aa+sp+tm)/2.
          heslo(5, 6) = +tm*(aa+sp+zp)/2.
          heslo(6, 6) = -sp*(aa+tm+zp)/2.
        endif
        aa          = +sp+tp+zp-   2.5
        shape(   7) = +sp*tp*zp*(aa+aa)
        deriv(1, 7) = +   tp*zp*(aa+sp)
        deriv(2, 7) = +sp   *zp*(aa+tp)
        deriv(3, 7) = +sp*tp   *(aa+zp)
        if(neres.eq.1) then
          heslo(1, 7) = +tp*zp
          heslo(2, 7) = +sp*zp
          heslo(3, 7) = +sp*tp
          heslo(4, 7) = +zp*(aa+sp+tp)/2.
          heslo(5, 7) = +tp*(aa+sp+zp)/2.
          heslo(6, 7) = +sp*(aa+tp+zp)/2.
        endif
        aa          = +sm+tp+zp-   2.5
        shape(   8) = +sm*tp*zp*(aa+aa)
        deriv(1, 8) = -   tp*zp*(aa+sm)
        deriv(2, 8) = +sm   *zp*(aa+tp)
        deriv(3, 8) = +sm*tp   *(aa+zp)
        if(neres.eq.1) then
          heslo(1, 8) = +tp*zp
          heslo(2, 8) = +sm*zp
          heslo(3, 8) = +sm*tp
          heslo(4, 8) = -zp*(aa+sm+tp)/2.
          heslo(5, 8) = -tp*(aa+sm+zp)/2.
          heslo(6, 8) = +sm*(aa+tp+zp)/2.
        endif
        shape(   9) = +ss*tm*zm*2.
        deriv(1, 9) = +s2*tm*zm
        deriv(2, 9) = -ss   *zm
        deriv(3, 9) = -ss*tm
        if(neres.eq.1) then
          heslo(1, 9) = -tm*zm*2.
          heslo(4, 9) = -s2*zm/2.
          heslo(5, 9) = -s2*tm/2.
          heslo(6, 9) = +ss/2.
        endif
        shape(  10) = +sp*tt*zm*2.
        deriv(1,10) = +   tt*zm
        deriv(2,10) = +sp*t2*zm
        deriv(3,10) = -sp*tt
        if(neres.eq.1) then
          heslo(2,10) = -sp*zm*2.
          heslo(4,10) = -t*zm
          heslo(5,10) = -tt/2.
          heslo(6,10) = -sp*t2/2.
        endif
        shape(  11) = +ss*tp*zm*2.
        deriv(1,11) = +s2*tp*zm
        deriv(2,11) = +ss   *zm
        deriv(3,11) = -ss*tp
        if(neres.eq.1) then
          heslo(1,11) = -tp*zm*2.
          heslo(4,11) = +s2*zm/2.
          heslo(5,11) = -s2*tp/2.
          heslo(6,11) = -ss/2.
        endif
        shape(  12) = +sm*tt*zm*2.
        deriv(1,12) = -   tt*zm
        deriv(2,12) = +sm*t2*zm
        deriv(3,12) = -sm*tt
        if(neres.eq.1) then
          heslo(2,12) = -sm*zm*2.
          heslo(4,12) = -t*zm
          heslo(5,12) = +tt/2.
          heslo(6,12) = -sm*t2/2.
        endif
        shape(  13) = +sm*tm*zz*2.
        deriv(1,13) = -   tm*zz
        deriv(2,13) = -sm   *zz
        deriv(3,13) = +sm*tm*z2
        if(neres.eq.1) then
          heslo(3,13) = -sm*tm*2.
          heslo(4,13) = +zz/2.
          heslo(5,13) = +tm*z
          heslo(6,13) = +sm*z
        endif
        shape(  14) = +sp*tm*zz*2.
        deriv(1,14) = +   tm*zz
        deriv(2,14) = -sp   *zz
        deriv(3,14) = +sp*tm*z2
        if(neres.eq.1) then
          heslo(3,14) = -sp*tm*2.
          heslo(4,14) = -zz/2.
          heslo(5,14) = -tm*z
          heslo(6,14) = +sp*z
        endif
        shape(  15) = +sp*tp*zz*2.
        deriv(1,15) = +   tp*zz
        deriv(2,15) = +sp   *zz
        deriv(3,15) = +sp*tp*z2
        if(neres.eq.1) then
          heslo(3,15) = -sp*tp*2.
          heslo(4,15) = +zz/2.
          heslo(5,15) = -tp*z
          heslo(6,15) = -sp*z
        endif
        shape(  16) = +sm*tp*zz*2.
        deriv(1,16) = -   tp*zz
        deriv(2,16) = +sm   *zz
        deriv(3,16) = +sm*tp*z2
        if(neres.eq.1) then
          heslo(3,16) = -sm*tp*2.
          heslo(4,16) = -zz/2.
          heslo(5,16) = +tp*z
          heslo(6,16) = -sm*z
        endif
        shape(  17) = +ss*tm*zp*2.
        deriv(1,17) = +s2*tm*zp
        deriv(2,17) = -ss   *zp
        deriv(3,17) = +ss*tm
        if(neres.eq.1) then
          heslo(1,17) = -tm*zp*2.
          heslo(4,17) = -s2*zp/2.
          heslo(5,17) = +s2*tm/2.
          heslo(6,17) = -ss/2.
        endif
        shape(  18) = +sp*tt*zp*2.
        deriv(1,18) = +   tt*zp
        deriv(2,18) = +sp*t2*zp
        deriv(3,18) = +sp*tt
        if(neres.eq.1) then
          heslo(2,18) = -sp*zp*2.
          heslo(4,18) = -t*zp
          heslo(5,18) = +tt/2.
          heslo(6,18) = +sp*t2/2.
        endif
        shape(  19) = +ss*tp*zp*2.
        deriv(1,19) = +s2*tp*zp
        deriv(2,19) = +ss   *zp
        deriv(3,19) = +ss*tp
        if(neres.eq.1) then
          heslo(1,19) = -tp*zp*2.
          heslo(4,19) = +s2*zp/2.
          heslo(5,19) = +s2*tp/2.
          heslo(6,19) = +ss/2.
        endif
        shape(  20) = +sm*tt*zp*2.
        deriv(1,20) = -   tt*zp
        deriv(2,20) = +sm*t2*zp
        deriv(3,20) = +sm*tt
        if(neres.eq.1) then
          heslo(2,20) = -sm*zp*2.
          heslo(4,20) = +t*zp
          heslo(5,20) = -tt/2.
          heslo(6,20) = +sm*t2/2.
        endif
c
c*** Triquadratic brick
c        
      else if(nnode.eq.27) then
        sl=s*(s-1.)
        tl=t*(t-1.)
        zl=z*(z-1.)
        sp=s*(s+1.)
        tp=t*(t+1.)
        zp=z*(z+1.)
        s1=2.*s-1.
        t1=2.*t-1.
        z1=2.*z-1.
        s2=1.-s*s
        t2=1.-t*t
        z2=1.-z*z
        s3=1.+2.*s
        t3=1.+2.*t
        z3=1.+2.*z
        s4=-2.*s
        t4=-2.*t
        z4=-2.*z
        shape(   1) = .125*sl*tl*zl
        deriv(1, 1) = .125*s1*tl*zl
        deriv(2, 1) = .125*sl*t1*zl
        deriv(3, 1) = .125*sl*tl*z1
        if(neres.eq.1) then
          heslo(1, 1) = .25*tl*zl
          heslo(2, 1) = .25*sl*zl
          heslo(3, 1) = .25*sl*tl
          heslo(4, 1) = .125*s1*t1*zl
          heslo(5, 1) = .125*s1*tl*z1
          heslo(6, 1) = .125*sl*t1*z1
        end if
        shape(   2) = .125*sp*tl*zl
        deriv(1, 2) = .125*s3*tl*zl
        deriv(2, 2) = .125*sp*t1*zl
        deriv(3, 2) = .125*sp*tl*z1
        if(neres.eq.1) then
          heslo(1, 2) = .25*tl*zl
          heslo(2, 2) = .25*sp*zl
          heslo(3, 2) = .25*sp*tl
          heslo(4, 2) = .125*s3*t1*zl
          heslo(5, 2) = .125*s3*tl*z1
          heslo(6, 2) = .125*sp*t1*z1
        end if
        shape(   3) = .125*sp*tp*zl
        deriv(1, 3) = .125*s3*tp*zl
        deriv(2, 3) = .125*sp*t3*zl
        deriv(3, 3) = .125*sp*tp*z1
        if(neres.eq.1) then
          heslo(1, 3) = .25*tp*zl
          heslo(2, 3) = .25*sp*zl
          heslo(3, 3) = .25*sp*tp
          heslo(4, 3) = .125*s3*t3*zl
          heslo(5, 3) = .125*s3*tp*z1
          heslo(6, 3) = .125*sp*t3*z1
        end if
        shape(   4) = .125*sl*tp*zl
        deriv(1, 4) = .125*s1*tp*zl
        deriv(2, 4) = .125*sl*t3*zl
        deriv(3, 4) = .125*sl*tp*z1
        if(neres.eq.1) then
          heslo(1, 4) = .25*tp*zl
          heslo(2, 4) = .25*sl*zl
          heslo(3, 4) = .25*sl*tp
          heslo(4, 4) = .125*s1*t3*zl
          heslo(5, 4) = .125*s1*tp*z1
          heslo(6, 4) = .125*sl*t3*z1
        end if
        shape(   5) = .125*sl*tl*zp
        deriv(1, 5) = .125*s1*tl*zp
        deriv(2, 5) = .125*sl*t1*zp
        deriv(3, 5) = .125*sl*tl*z3
        if(neres.eq.1) then
          heslo(1, 5) = .25*tl*zp
          heslo(2, 5) = .25*sl*zp
          heslo(3, 5) = .25*sl*tl
          heslo(4, 5) = .125*s1*t1*zp
          heslo(5, 5) = .125*s1*tl*z3
          heslo(6, 5) = .125*sl*t1*z3
        end if
        shape(   6) = .125*sp*tl*zp
        deriv(1, 6) = .125*s3*tl*zp
        deriv(2, 6) = .125*sp*t1*zp
        deriv(3, 6) = .125*sp*tl*z3
        if(neres.eq.1) then
          heslo(1, 6) = .25*tl*zp
          heslo(2, 6) = .25*sp*zp
          heslo(3, 6) = .25*sp*tl
          heslo(4, 6) = .125*s3*t1*zp
          heslo(5, 6) = .125*s3*tl*z3
          heslo(6, 6) = .125*sp*t1*z3
        end if
        shape(   7) = .125*sp*tp*zp
        deriv(1, 7) = .125*s3*tp*zp
        deriv(2, 7) = .125*sp*t3*zp
        deriv(3, 7) = .125*sp*tp*z3
        if(neres.eq.1) then
          heslo(1, 7) = .25*tp*zp
          heslo(2, 7) = .25*sp*zp
          heslo(3, 7) = .25*sp*tp
          heslo(4, 7) = .125*s3*t3*zp
          heslo(5, 7) = .125*s3*tp*z3
          heslo(6, 7) = .125*sp*t3*z3
        end if
        shape(   8) = .125*sl*tp*zp
        deriv(1, 8) = .125*s1*tp*zp
        deriv(2, 8) = .125*sl*t3*zp
        deriv(3, 8) = .125*sl*tp*z3
        if(neres.eq.1) then
          heslo(1, 8) = .25*tp*zp
          heslo(2, 8) = .25*sl*zp
          heslo(3, 8) = .25*sl*tp
          heslo(4, 8) = .125*s1*t3*zp
          heslo(5, 8) = .125*s1*tp*z3
          heslo(6, 8) = .125*sl*t3*z3
        end if
        shape(   9) = .25*s2*tl*zl
        deriv(1, 9) = .25*s4*tl*zl
        deriv(2, 9) = .25*s2*t1*zl
        deriv(3, 9) = .25*s2*tl*z1
        if(neres.eq.1) then
          heslo(1, 9) =-.5*tl*zl
          heslo(2, 9) = .5*s2*zl
          heslo(3, 9) = .5*s2*tl
          heslo(4, 9) = .25*s4*t1*zl
          heslo(5, 9) = .25*s4*tl*z1
          heslo(6, 9) = .25*s2*t1*z1
        end if
        shape(  10) = .25*sp*t2*zl
        deriv(1,10) = .25*s3*t2*zl
        deriv(2,10) = .25*sp*t4*zl
        deriv(3,10) = .25*sp*t2*z1
        if(neres.eq.1) then
          heslo(1,10) = .5*t2*zl
          heslo(2,10) =-.5*sp*zl
          heslo(3,10) = .5*sp*t2
          heslo(4,10) = .25*s3*t4*zl
          heslo(5,10) = .25*s3*t2*z1
          heslo(6,10) = .25*sp*t4*z1
        end if
        shape(  11) = .25*s2*tp*zl
        deriv(1,11) = .25*s4*tp*zl
        deriv(2,11) = .25*s2*t3*zl
        deriv(3,11) = .25*s2*tp*z1
        if(neres.eq.1) then
          heslo(1,11) =-.5*tp*zl
          heslo(2,11) = .5*s2*zl
          heslo(3,11) = .5*s2*tp
          heslo(4,11) = .25*s4*t3*zl
          heslo(5,11) = .25*s4*tp*z1
          heslo(6,11) = .25*s2*t3*z1
        end if
        shape(  12) = .25*sl*t2*zl
        deriv(1,12) = .25*s1*t2*zl
        deriv(2,12) = .25*sl*t4*zl
        deriv(3,12) = .25*sl*t2*z1
        if(neres.eq.1) then
          heslo(1,12) = .5*t2*zl
          heslo(2,12) =-.5*sl*zl
          heslo(3,12) = .5*sl*t2
          heslo(4,12) = .25*s1*t4*zl
          heslo(5,12) = .25*s1*t2*z1
          heslo(6,12) = .25*sl*t4*z1
        end if
        shape(  13) = .25*sl*tl*z2
        deriv(1,13) = .25*s1*tl*z2
        deriv(2,13) = .25*sl*t1*z2
        deriv(3,13) = .25*sl*tl*z4
        if(neres.eq.1) then
          heslo(1,13) = .5*tl*z2
          heslo(2,13) = .5*sl*z2
          heslo(3,13) =-.5*sl*tl
          heslo(4,13) = .25*s1*t1*z2
          heslo(5,13) = .25*s1*tl*z4
          heslo(6,13) = .25*sl*t1*z4
        end if
        shape(  14) = .25*sp*tl*z2
        deriv(1,14) = .25*s3*tl*z2
        deriv(2,14) = .25*sp*t1*z2
        deriv(3,14) = .25*sp*tl*z4
        if(neres.eq.1) then
          heslo(1,14) = .5*tl*z2
          heslo(2,14) = .5*sp*z2
          heslo(3,14) =-.5*sp*tl
          heslo(4,14) = .25*s3*t1*z2
          heslo(5,14) = .25*s3*tl*z4
          heslo(6,14) = .25*sp*t1*z4
        end if
        shape(  15) = .25*sp*tp*z2
        deriv(1,15) = .25*s3*tp*z2
        deriv(2,15) = .25*sp*t3*z2
        deriv(3,15) = .25*sp*tp*z4
        if(neres.eq.1) then
          heslo(1,15) = .5*tp*z2
          heslo(2,15) = .5*sp*z2
          heslo(3,15) =-.5*sp*tp
          heslo(4,15) = .25*s3*t3*z2
          heslo(5,15) = .25*s3*tp*z4
          heslo(6,15) = .25*sp*t3*z4
        end if
        shape(  16) = .25*sl*tp*z2
        deriv(1,16) = .25*s1*tp*z2
        deriv(2,16) = .25*sl*t3*z2
        deriv(3,16) = .25*sl*tp*z4
        if(neres.eq.1) then
          heslo(1,16) = .5*tp*z2
          heslo(2,16) = .5*sl*z2
          heslo(3,16) =-.5*sl*tp
          heslo(4,16) = .25*s1*t3*z2
          heslo(5,16) = .25*s1*tp*z4
          heslo(6,16) = .25*sl*t3*z4
        end if
        shape(  17) = .25*s2*tl*zp
        deriv(1,17) = .25*s4*tl*zp
        deriv(2,17) = .25*s2*t1*zp
        deriv(3,17) = .25*s2*tl*z3
        if(neres.eq.1) then
          heslo(1,17) =-.5*tl*zp
          heslo(2,17) = .5*s2*zp
          heslo(3,17) = .5*s2*tl
          heslo(4,17) = .25*s4*t1*zp
          heslo(5,17) = .25*s4*tl*z3
          heslo(6,17) = .25*s2*t1*z3
        end if
        shape(  18) = .25*sp*t2*zp
        deriv(1,18) = .25*s3*t2*zp
        deriv(2,18) = .25*sp*t4*zp
        deriv(3,18) = .25*sp*t2*z3
        if(neres.eq.1) then
          heslo(1,18) = .5*t2*zp
          heslo(2,18) =-.5*sp*zp
          heslo(3,18) = .5*sp*t2
          heslo(4,18) = .25*s3*t4*zp
          heslo(5,18) = .25*s3*t2*z3
          heslo(6,18) = .25*sp*t4*z3
        end if
        shape(  19) = .25*s2*tp*zp
        deriv(1,19) = .25*s4*tp*zp
        deriv(2,19) = .25*s2*t3*zp
        deriv(3,19) = .25*s2*tp*z3
        if(neres.eq.1) then
          heslo(1,19) =-.5*tp*zp
          heslo(2,19) = .5*s2*zp
          heslo(3,19) = .5*s2*tp
          heslo(4,19) = .25*s4*t3*zp
          heslo(5,19) = .25*s4*tp*z3
          heslo(6,19) = .25*s2*t3*z3
        end if
        shape(  20) = .25*sl*t2*zp
        deriv(1,20) = .25*s1*t2*zp
        deriv(2,20) = .25*sl*t4*zp
        deriv(3,20) = .25*sl*t2*z3
        if(neres.eq.1) then
          heslo(1,20) = .5*t2*zp
          heslo(2,20) =-.5*sl*zp
          heslo(3,20) = .5*sl*t2
          heslo(4,20) = .25*s1*t4*zp
          heslo(5,20) = .25*s1*t2*z3
          heslo(6,20) = .25*sl*t4*z3
        end if
        shape(  21) = .5*s2*t2*zl
        deriv(1,21) = .5*s4*t2*zl
        deriv(2,21) = .5*s2*t4*zl
        deriv(3,21) = .5*s2*t2*z1
        if(neres.eq.1) then
          heslo(1,21) =-t2*zl
          heslo(2,21) =-s2*zl
          heslo(3,21) = s2*t2
          heslo(4,21) = .5*s4*t4*zl
          heslo(5,21) = .5*s4*t2*z1
          heslo(6,21) = .5*s2*t4*z1
        end if
        shape(  22) = .5*s2*tl*z2
        deriv(1,22) = .5*s4*tl*z2
        deriv(2,22) = .5*s2*t1*z2
        deriv(3,22) = .5*s2*tl*z4
        if(neres.eq.1) then
          heslo(1,22) =-tl*z2
          heslo(2,22) = s2*z2
          heslo(3,22) =-s2*tl
          heslo(4,22) = .5*s4*t1*z2
          heslo(5,22) = .5*s4*tl*z4
          heslo(6,22) = .5*s2*t1*z4
        end if
        shape(  23) = .5*sp*t2*z2
        deriv(1,23) = .5*s3*t2*z2
        deriv(2,23) = .5*sp*t4*z2
        deriv(3,23) = .5*sp*t2*z4
        if(neres.eq.1) then
          heslo(1,23) = t2*z2
          heslo(2,23) =-sp*z2
          heslo(3,23) =-sp*t2
          heslo(4,23) = .5*s3*t4*z2
          heslo(5,23) = .5*s3*t2*z4
          heslo(6,23) = .5*sp*t4*z4
        end if
        shape(  24) = .5*s2*tp*z2
        deriv(1,24) = .5*s4*tp*z2
        deriv(2,24) = .5*s2*t3*z2
        deriv(3,24) = .5*s2*tp*z4
        if(neres.eq.1) then
          heslo(1,24) =-tp*z2
          heslo(2,24) = s2*z2
          heslo(3,24) =-s2*tp
          heslo(4,24) = .5*s4*t3*z2
          heslo(5,24) = .5*s4*tp*z4
          heslo(6,24) = .5*s2*t3*z4
        end if
        shape(  25) = .5*sl*t2*z2
        deriv(1,25) = .5*s1*t2*z2
        deriv(2,25) = .5*sl*t4*z2
        deriv(3,25) = .5*sl*t2*z4
        if(neres.eq.1) then
          heslo(1,25) = t2*z2
          heslo(2,25) =-sl*z2
          heslo(3,25) =-sl*t2
          heslo(4,25) = .5*s1*t4*z2
          heslo(5,25) = .5*s1*t2*z4
          heslo(6,25) = .5*sl*t4*z4
        end if
        shape(  26) = .5*s2*t2*zp
        deriv(1,26) = .5*s4*t2*zp
        deriv(2,26) = .5*s2*t4*zp
        deriv(3,26) = .5*s2*t2*z3
        if(neres.eq.1) then
          heslo(1,26) =-t2*zp
          heslo(2,26) =-s2*zp
          heslo(3,26) = s2*t2
          heslo(4,26) = .5*s4*t4*zp
          heslo(5,26) = .5*s4*t2*z3
          heslo(6,26) = .5*s2*t4*z3
        end if
        shape(  27) = s2*t2*z2
        deriv(1,27) = s4*t2*z2
        deriv(2,27) = s2*t4*z2
        deriv(3,27) = s2*t2*z4
        if(neres.eq.1) then
          heslo(1,27) =-2.*t2*z2
          heslo(2,27) =-2.*s2*z2
          heslo(3,27) =-2.*s2*t2
          heslo(4,27) = s4*t4*z2
          heslo(5,27) = s4*t2*z4
          heslo(6,27) = s2*t4*z4
        end if
c
c*** Tricubic brick
c        
      else if (nnode.eq.64) then
        a=729./4096.
        s1=(1.0/3.0+s)*(1.0/3.0-s)*(1.-s)                           
        s2=(1.+s)*(1.0/3.0-s)*(1.-s)                              
        s3=(1.+s)*(1.0/3.0+s)*(1.-s)
        s4=(1.+s)*(1.0/3.0+s)*(1.0/3.0-s)                                
        t1=(1.0/3.0+t)*(1.0/3.0-t)*(1.-t)         
        t2=(1.+s)*(1.0/3.0-t)*(1.-t)
        t3=(1.+t)*(1.0/3.0+t)*(1.-t)                         
        t4=(1.+t)*(1.0/3.0+t)*(1.0/3.0-t)
        z1=(1.0/3.0+z)*(1.0/3.0-z)*(1.-z)
        z2=(1.+z)*(1.0/3.0-z)*(1.-z)
        z3=(1.+z)*(1.0/3.0+z)*(1.-z)
        z4=(1.+z)*(1.0/3.0+z)*(1.0/3.0-z)
        s11=(1.0/3.0-s)*(1.-s)-(1.0/3.0+s)*(1.-s)
     .       -(1.0/3.0+s)*(1.0/3.0-s)
        s21=(1.0/3.0-s)*(1.-s)-(1.+s)*(1.-s)
     .       -(1.+s)*(1.0/3.0-s)
        s31=(1.0/3.0+s)*(1.-s)+(1.+s)*(1.-s)
     .       -(1.+s)*(1.0/3.0+s)
        s41=(1.0/3.0+s)*(1.0/3.0-s)+(1.+s)*(1.0/3.0-s)
     .       -(1.+s)*(1.0/3.0+s)
        t11=(1.0/3.0-t)*(1.-t)-(1.0/3.0+t)*(1.-t)
     .       -(1.0/3.0+t)*(1.0/3.0-t)
        t21=(1.0/3.0-t)*(1.-t)-(1.+t)*(1.-t)
     .       -(1.+t)*(1.0/3.0-t)    
        t31=(1.0/3.0+t)*(1.-t)+(1.+t)*(1.-t)
     .       -(1.+t)*(1.0/3.0+t)    
        t41=(1.0/3.0+t)*(1.0/3.0-t)+(1.+t)*(1.0/3.0-t)
     .       -(1.+t)*(1.0/3.0+t)
        z11=(1.0/3.0-z)*(1.-z)-(1.0/3.0+z)*(1.-z)
     .       -(1.0/3.0+z)*(1.0/3.0-z)
        z21=(1.0/3.0-z)*(1.-z)-(1.+z)*(1.-z)
     .       -(1.+z)*(1.0/3.0-z)    
        z31=(1.0/3.0+z)*(1.-z)+(1.+z)*(1.-z)
     .       -(1.+z)*(1.0/3.0+z)    
        z41=(1.0/3.0+z)*(1.0/3.0-z)+(1.+z)*(1.0/3.0-z)
     .       -(1.+z)*(1.0/3.0+z)
        shape(   1) =   a*s1*t1*z1
        deriv(1, 1) =   a*s11*t1*z1
        deriv(2, 1) =   a*s1*t11*z1
        deriv(3, 1) =   a*s1*t1*z11
        shape(   2) =   a*s4*t1*z1
        deriv(1, 2) =   a*s41*t1*z1
        deriv(2, 2) =   a*s4*t11*z1
        deriv(3, 2) =   a*s4*t1*z11
        shape(   3) =   a*s4*t4*z1
        deriv(1, 3) =   a*s41*t4*z1
        deriv(2, 3) =   a*s4*t41*z1
        deriv(3, 3) =   a*s4*t4*z11
        shape(   4) =   a*s1*t4*z1
        deriv(1, 4) =   a*s11*t4*z1
        deriv(2, 4) =   a*s1*t41*z1
        deriv(3, 4) =   a*s1*t4*z11
        shape(   5) =   a*s1*t1*z4
        deriv(1, 5) =   a*s11*t1*z4
        deriv(2, 5) =   a*s1*t11*z4
        deriv(3, 5) =   a*s1*t1*z41
        shape(   6) =   a*s4*t1*z4
        deriv(1, 6) =   a*s41*t1*z4
        deriv(2, 6) =   a*s4*t11*z4
        deriv(3, 6) =   a*s4*t1*z41
        shape(   7) =   a*s4*t4*z4
        deriv(1, 7) =   a*s41*t4*z4
        deriv(2, 7) =   a*s4*t41*z4
        deriv(3, 7) =   a*s4*t4*z41
        shape(   8) =   a*s1*t4*z4
        deriv(1, 8) =   a*s11*t4*z4
        deriv(2, 8) =   a*s1*t41*z4
        deriv(3, 8) =   a*s1*t4*z41
        shape(   9) = 3.*a*s2*t1*z1
        deriv(1, 9) = 3.*a*s21*t1*z1
        deriv(2, 9) = 3.*a*s2*t11*z1
        deriv(3, 9) = 3.*a*s2*t1*z11
        shape(  10) = 3.*a*s3*t1*z1
        deriv(1,10) = 3.*a*s31*t1*z1
        deriv(2,10) = 3.*a*s3*t11*z1
        deriv(3,10) = 3.*a*s3*t1*z11        
        shape(  11) = 3.*a*s4*t2*z1
        deriv(1,11) = 3.*a*s41*t2*z1
        deriv(2,11) = 3.*a*s4*t21*z1
        deriv(3,11) = 3.*a*s4*t2*z11        
        shape(  12) = 3.*a*s4*t3*z1
        deriv(1,12) = 3.*a*s41*t3*z1        
        deriv(2,12) = 3.*a*s4*t31*z1        
        deriv(3,12) = 3.*a*s4*t3*z11                
        shape(  13) = 3.*a*s3*t4*z1
        deriv(1,13) = 3.*a*s31*t4*z1
        deriv(2,13) = 3.*a*s3*t41*z1
        deriv(3,13) = 3.*a*s3*t4*z11        
        shape(  14) = 3.*a*s2*t4*z1
        deriv(1,14) = 3.*a*s21*t4*z1
        deriv(2,14) = 3.*a*s2*t41*z1
        deriv(3,14) = 3.*a*s2*t4*z11        
        shape(  15) = 3.*a*s1*t3*z1
        deriv(1,15) = 3.*a*s11*t3*z1
        deriv(2,15) = 3.*a*s1*t31*z1
        deriv(3,15) = 3.*a*s1*t3*z11        
        shape(  16) = 3.*a*s1*t2*z1
        deriv(1,16) = 3.*a*s11*t2*z1
        deriv(2,16) = 3.*a*s1*t21*z1
        deriv(3,16) = 3.*a*s1*t2*z11        
        shape(  17) = 3.*a*s1*t1*z2
        deriv(1,17) = 3.*a*s11*t1*z2
        deriv(2,17) = 3.*a*s1*t11*z2
        deriv(3,17) = 3.*a*s1*t1*z21        
        shape(  18) = 3.*a*s4*t1*z2
        deriv(1,18) = 3.*a*s41*t1*z2
        deriv(2,18) = 3.*a*s4*t11*z2
        deriv(3,18) = 3.*a*s4*t1*z21        
        shape(  19) = 3.*a*s4*t4*z2
        deriv(1,19) = 3.*a*s41*t4*z2
        deriv(2,19) = 3.*a*s4*t41*z2
        deriv(3,19) = 3.*a*s4*t4*z21        
        shape(  20) = 3.*a*s1*t4*z2
        deriv(1,20) = 3.*a*s11*t4*z2
        deriv(2,20) = 3.*a*s1*t41*z2
        deriv(3,20) = 3.*a*s1*t4*z21        
        shape(  21) = 3.*a*s1*t1*z3
        deriv(1,21) = 3.*a*s11*t1*z3
        deriv(2,21) = 3.*a*s1*t11*z3
        deriv(3,21) = 3.*a*s1*t1*z31        
        shape(  22) = 3.*a*s4*t1*z3
        deriv(1,22) = 3.*a*s41*t1*z3
        deriv(2,22) = 3.*a*s4*t11*z3
        deriv(3,22) = 3.*a*s4*t1*z31
        shape(  23) = 3.*a*s4*t4*z3
        deriv(1,23) = 3.*a*s41*t4*z3
        deriv(2,23) = 3.*a*s4*t41*z3
        deriv(3,23) = 3.*a*s4*t4*z31        
        shape(  24) = 3.*a*s1*t4*z3
        deriv(1,24) = 3.*a*s11*t4*z3
        deriv(2,24) = 3.*a*s1*t41*z3
        deriv(3,24) = 3.*a*s1*t4*z31        
        shape(  25) = 3.*a*s2*t1*z4
        deriv(1,25) = 3.*a*s21*t1*z4
        deriv(2,25) = 3.*a*s2*t11*z4
        deriv(3,25) = 3.*a*s2*t1*z41        
        shape(  26) = 3.*a*s3*t1*z4
        deriv(1,21) = 3.*a*s31*t1*z4
        deriv(2,21) = 3.*a*s3*t11*z4
        deriv(3,21) = 3.*a*s3*t1*z41        
        shape(  27) = 3.*a*s4*t2*z4
        deriv(1,27) = 3.*a*s41*t2*z4
        deriv(2,27) = 3.*a*s4*t21*z4
        deriv(3,27) = 3.*a*s4*t2*z41        
        shape(  28) = 3.*a*s4*t3*z4
        deriv(1,28) = 3.*a*s41*t3*z4
        deriv(2,28) = 3.*a*s4*t31*z4
        deriv(3,28) = 3.*a*s4*t3*z41        
        shape(  29) = 3.*a*s3*t4*z4
        deriv(1,29) = 3.*a*s31*t4*z4
        deriv(2,29) = 3.*a*s3*t41*z4
        deriv(3,29) = 3.*a*s3*t4*z41        
        shape(  30) = 3.*a*s2*t4*z4
        deriv(1,30) = 3.*a*s21*t4*z4  
        deriv(2,30) = 3.*a*s2*t41*z4  
        deriv(3,30) = 3.*a*s2*t4*z41          
        shape(  31) = 3.*a*s1*t3*z4
        deriv(1,31) = 3.*a*s11*t3*z4
        deriv(2,31) = 3.*a*s1*t31*z4
        deriv(3,31) = 3.*a*s1*t3*z41        
        shape(  32) = 3.*a*s1*t2*z4
        deriv(1,32) = 3.*a*s11*t2*z4
        deriv(2,32) = 3.*a*s1*t21*z4
        deriv(3,32) = 3.*a*s1*t2*z41        
        shape(  33) =-9.*a*s2*t2*z1
        deriv(1,33) =-9.*a*s21*t2*z1
        deriv(2,33) =-9.*a*s2*t21*z1
        deriv(3,33) =-9.*a*s2*t2*z11        
        shape(  34) =-9.*a*s3*t2*z1
        deriv(1,34) =-9.*a*s31*t2*z1
        deriv(2,34) =-9.*a*s3*t21*z1
        deriv(3,34) =-9.*a*s3*t2*z11        
        shape(  35) =-9.*a*s3*t3*z1
        deriv(1,35) =-9.*a*s31*t3*z1
        deriv(2,35) =-9.*a*s3*t31*z1
        deriv(3,35) =-9.*a*s3*t3*z11        
        shape(  36) =-9.*a*s2*t3*z1
        deriv(1,36) =-9.*a*s21*t3*z1
        deriv(2,36) =-9.*a*s2*t31*z1
        deriv(3,36) =-9.*a*s2*t3*z11        
        shape(  37) =-9.*a*s2*t1*z2
        deriv(1,37) =-9.*a*s21*t1*z2
        deriv(2,37) =-9.*a*s2*t11*z2
        deriv(3,37) =-9.*a*s2*t1*z21        
        shape(  38) =-9.*a*s3*t1*z2
        deriv(1,38) =-9.*a*s31*t1*z2        
        deriv(2,38) =-9.*a*s3*t11*z2        
        deriv(3,38) =-9.*a*s3*t1*z21                
        shape(  39) =-9.*a*s4*t2*z2
        deriv(1,39) =-9.*a*s41*t2*z2        
        deriv(2,39) =-9.*a*s4*t21*z2        
        deriv(3,39) =-9.*a*s4*t2*z21                
        shape(  40) =-9.*a*s4*t3*z2
        deriv(1,40) =-9.*a*s41*t3*z2        
        deriv(2,40) =-9.*a*s4*t31*z2        
        deriv(3,40) =-9.*a*s4*t3*z21                
        shape(  41) =-9.*a*s3*t4*z2
        deriv(1,41) =-9.*a*s31*t4*z2        
        deriv(2,41) =-9.*a*s3*t41*z2        
        deriv(3,41) =-9.*a*s3*t4*z21                
        shape(  42) =-9.*a*s2*t4*z2
        deriv(1,42) =-9.*a*s21*t4*z2             
        deriv(2,42) =-9.*a*s2*t41*z2             
        deriv(3,42) =-9.*a*s2*t4*z21                     
        shape(  43) =-9.*a*s1*t3*z2
        deriv(1,43) =-9.*a*s11*t3*z2             
        deriv(2,43) =-9.*a*s1*t31*z2             
        deriv(3,43) =-9.*a*s1*t3*z21                     
        shape(  44) =-9.*a*s1*t2*z2
        deriv(1,44) =-9.*a*s11*t2*z2             
        deriv(2,44) =-9.*a*s1*t21*z2             
        deriv(3,44) =-9.*a*s1*t2*z21                     
        shape(  45) =-9.*a*s2*t1*z3
        deriv(1,45) =-9.*a*s21*t1*z3             
        deriv(2,45) =-9.*a*s2*t11*z3             
        deriv(3,45) =-9.*a*s2*t1*z31                     
        shape(  46) =-9.*a*s3*t1*z3
        deriv(1,46) =-9.*a*s31*t1*z3
        deriv(2,46) =-9.*a*s3*t11*z3
        deriv(3,46) =-9.*a*s3*t1*z31        
        shape(  47) =-9.*a*s4*t2*z3
        deriv(1,47) =-9.*a*s41*t2*z3
        deriv(2,47) =-9.*a*s4*t21*z3
        deriv(3,47) =-9.*a*s4*t2*z31
        shape(  48) =-9.*a*s4*t3*z3
        deriv(1,48) =-9.*a*s41*t3*z3
        deriv(2,48) =-9.*a*s4*t31*z3
        deriv(3,48) =-9.*a*s4*t3*z31        
        shape(  49) =-9.*a*s3*t4*z3
        deriv(1,49) =-9.*a*s31*t4*z3
        deriv(2,49) =-9.*a*s3*t41*z3
        deriv(3,49) =-9.*a*s3*t4*z31        
        shape(  50) =-9.*a*s2*t4*z3
        deriv(1,50) =-9.*a*s21*t4*z3
        deriv(2,50) =-9.*a*s2*t41*z3
        deriv(3,50) =-9.*a*s2*t4*z31        
        shape(  51) =-9.*a*s1*t3*z3
        deriv(1,51) =-9.*a*s11*t3*z3
        deriv(2,51) =-9.*a*s1*t31*z3
        deriv(3,51) =-9.*a*s1*t3*z31        
        shape(  52) =-9.*a*s1*t2*z3
        deriv(1,52) =-9.*a*s11*t2*z3
        deriv(2,52) =-9.*a*s1*t21*z3
        deriv(3,52) =-9.*a*s1*t2*z31        
        shape(  53) =-9.*a*s2*t2*z4
        deriv(1,53) =-9.*a*s21*t2*z4
        deriv(2,53) =-9.*a*s2*t21*z4
        deriv(3,53) =-9.*a*s2*t2*z41        
        shape(  54) =-9.*a*s3*t2*z4
        deriv(1,54) =-9.*a*s31*t2*z4
        deriv(2,54) =-9.*a*s3*t21*z4
        deriv(3,54) =-9.*a*s3*t2*z41        
        shape(  55) =-9.*a*s3*t3*z4
        deriv(1,55) =-9.*a*s31*t3*z4
        deriv(2,55) =-9.*a*s3*t31*z4
        deriv(3,55) =-9.*a*s3*t3*z41        
        shape(  56) =-9.*a*s2*t3*z4
        deriv(1,56) =-9.*a*s21*t3*z4
        deriv(2,56) =-9.*a*s2*t31*z4
        deriv(3,56) =-9.*a*s2*t3*z41        
        shape(  57) =27.*a*s2*t2*z2
        deriv(1,57) =27.*a*s21*t2*z2
        deriv(2,57) =27.*a*s2*t21*z2
        deriv(3,57) =27.*a*s2*t2*z21        
        shape(  58) =27.*a*s3*t2*z2
        deriv(1,58) =27.*a*s31*t2*z2
        deriv(2,58) =27.*a*s3*t21*z2
        deriv(3,58) =27.*a*s3*t2*z21        
        shape(  59) =27.*a*s3*t3*z2
        deriv(1,59) =27.*a*s31*t3*z2
        deriv(2,59) =27.*a*s3*t31*z2
        deriv(3,59) =27.*a*s3*t3*z21        
        shape(  60) =27.*a*s2*t3*z2
        deriv(1,60) =27.*a*s21*t3*z2
        deriv(2,60) =27.*a*s2*t31*z2
        deriv(3,60) =27.*a*s2*t3*z21        
        shape(  61) =27.*a*s2*t2*z3
        deriv(1,61) =27.*a*s21*t2*z3
        deriv(2,61) =27.*a*s2*t21*z3
        deriv(3,61) =27.*a*s2*t2*z31        
        shape(  62) =27.*a*s3*t2*z3
        deriv(1,62) =27.*a*s31*t2*z3
        deriv(2,62) =27.*a*s3*t21*z3
        deriv(3,62) =27.*a*s3*t2*z31        
        shape(  63) =27.*a*s3*t3*z3
        deriv(1,63) =27.*a*s31*t3*z3
        deriv(2,63) =27.*a*s3*t31*z3
        deriv(3,63) =27.*a*s3*t3*z31        
        shape(  64) =27.*a*s2*t3*z3
        deriv(1,64) =27.*a*s21*t3*z3
        deriv(2,64) =27.*a*s2*t31*z3
        deriv(3,64) =27.*a*s2*t3*z31
        if (neres.eq.1) then
          s12=-2.*((1.-s)+(1.0/3.0-s)-(1.0/3.0+s))
          s22=-2.*((1.-s)+(1.0/3.0-s)-(1.+s))
          s32=2.*((1.-s)-(1.0/3.0+s)-(1.+s))
          s42=2.*((1.0/3.0-s)-(1.0/3.0+s)-(1.+s))
          t12=-2.*((1.-t)+(1.0/3.0-t)-(1.0/3.0+t))
          t22=-2.*((1.-t)+(1.0/3.0-t)-(1.+t)) 
          t32=2.*((1.-t)-(1.0/3.0+t)-(1.+t))  
          t42=2.*((1.0/3.0-t)-(1.0/3.0+t)-(1.+t))
          z12=-2.*((1.-z)+(1.0/3.0-z)-(1.0/3.0+z))
          z22=-2.*((1.-z)+(1.0/3.0-z)-(1.+z)) 
          z32=2.*((1.-z)-(1.0/3.0+z)-(1.+z))  
          z42=2.*((1.0/3.0-z)-(1.0/3.0+z)-(1.+z))

          heslo(1, 1) =   a*s12*t1*z1
          heslo(2, 1) =   a*s1*t12*z1
          heslo(3, 1) =   a*s1*t1*z12
          heslo(4, 1) =   a*s11*t11*z1
          heslo(5, 1) =   a*s11*t1*z11
          heslo(6, 1) =   a*s1*t11*z11
                                                
          heslo(1, 2) =   a*s42*t1*z1  
          heslo(2, 2) =   a*s4*t12*z1 
          heslo(3, 2) =   a*s4*t1*z12 
          heslo(4, 2) =   a*s41*t11*z1
          heslo(5, 2) =   a*s41*t1*z11
          heslo(6, 2) =   a*s4*t11*z11          
                                      
          heslo(1, 3) =   a*s42*t4*z1           
          heslo(2, 3) =   a*s4*t42*z1  
          heslo(3, 3) =   a*s4*t4*z12      
          heslo(4, 3) =   a*s41*t41*z1
          heslo(5, 3) =   a*s41*t4*z11
          heslo(6, 3) =   a*s4*t41*z11      
                                      
          heslo(1, 4) =   a*s12*t4*z1 
          heslo(2, 4) =   a*s1*t42*z1 
          heslo(3, 4) =   a*s1*t4*z12 
          heslo(4, 4) =   a*s11*t41*z1
          heslo(5, 4) =   a*s11*t4*z11
          heslo(6, 4) =   a*s1*t41*z11
                                      
          heslo(1, 5) =   a*s12*t1*z4 
          heslo(2, 5) =   a*s1*t12*z4 
          heslo(3, 5) =   a*s1*t1*z42 
          heslo(4, 5) =   a*s11*t11*z4
          heslo(5, 5) =   a*s11*t1*z41
          heslo(6, 5) =   a*s1*t11*z41
                                                
          heslo(1, 6) =   a*s42*t4*z4 
          heslo(2, 6) =   a*s4*t42*z4 
          heslo(3, 6) =   a*s4*t4*z42 
          heslo(4, 6) =   a*s41*t41*z4
          heslo(5, 6) =   a*s41*t4*z41
          heslo(6, 6) =   a*s4*t41*z41
                                                
          heslo(1, 7) =   a*s42*t4*z4 
          heslo(2, 7) =   a*s4*t42*z4 
          heslo(3, 7) =   a*s4*t4*z42 
          heslo(4, 7) =   a*s41*t41*z4
          heslo(5, 7) =   a*s41*t4*z41
          heslo(6, 7) =   a*s4*t41*z41
                                      
          heslo(1, 8) =   a*s12*t4*z4 
          heslo(2, 8) =   a*s1*t42*z4 
          heslo(3, 8) =   a*s1*t4*z42 
          heslo(4, 8) =   a*s11*t41*z4
          heslo(5, 8) =   a*s11*t4*z41
          heslo(6, 8) =   a*s1*t41*z41
                                                                            
          heslo(1, 9) = 3.*a*s22*t1*z1 
          heslo(2, 9) = 3.*a*s2*t12*z1 
          heslo(3, 9) = 3.*a*s2*t1*z12 
          heslo(4, 9) = 3.*a*s21*t11*z1
          heslo(5, 9) = 3.*a*s21*t1*z11
          heslo(6, 9) = 3.*a*s2*t11*z11
                                      
          heslo(1,10) = 3.*a*s32*t1*z1 
          heslo(2,10) = 3.*a*s3*t12*z1 
          heslo(3,10) = 3.*a*s3*t1*z12 
          heslo(4,10) = 3.*a*s31*t11*z1
          heslo(5,10) = 3.*a*s31*t1*z11
          heslo(6,10) = 3.*a*s3*t11*z11
                                                                         
          heslo(1,11) = 3.*a*s42*t2*z1 
          heslo(2,11) = 3.*a*s4*t22*z1 
          heslo(3,11) = 3.*a*s4*t2*z12 
          heslo(4,11) = 3.*a*s41*t21*z1
          heslo(5,11) = 3.*a*s41*t2*z11
          heslo(6,11) = 3.*a*s4*t21*z11
                                      
          heslo(1,12) = 3.*a*s42*t3*z1 
          heslo(2,12) = 3.*a*s4*t32*z1 
          heslo(3,12) = 3.*a*s4*t3*z12 
          heslo(4,12) = 3.*a*s41*t31*z1
          heslo(5,12) = 3.*a*s41*t3*z11
          heslo(6,12) = 3.*a*s4*t31*z11
                                                                         
          heslo(1,13) = 3.*a*s32*t4*z1 
          heslo(2,13) = 3.*a*s3*t42*z1 
          heslo(3,13) = 3.*a*s3*t4*z12 
          heslo(4,13) = 3.*a*s31*t41*z1
          heslo(5,13) = 3.*a*s31*t4*z11
          heslo(6,13) = 3.*a*s3*t41*z11
                                      
          heslo(1,14) = 3.*a*s22*t4*z1 
          heslo(2,14) = 3.*a*s2*t42*z1 
          heslo(3,14) = 3.*a*s2*t4*z12 
          heslo(4,14) = 3.*a*s21*t41*z1
          heslo(5,14) = 3.*a*s21*t4*z11
          heslo(6,14) = 3.*a*s2*t41*z11
                                                                            
          heslo(1,15) = 3.*a*s12*t3*z1 
          heslo(2,15) = 3.*a*s1*t32*z1 
          heslo(3,15) = 3.*a*s1*t3*z12 
          heslo(4,15) = 3.*a*s11*t31*z1
          heslo(5,15) = 3.*a*s11*t3*z11
          heslo(6,15) = 3.*a*s1*t31*z11
                                      
          heslo(1,16) = 3.*a*s12*t2*z1 
          heslo(2,16) = 3.*a*s1*t22*z1 
          heslo(3,16) = 3.*a*s1*t2*z12 
          heslo(4,16) = 3.*a*s11*t21*z1
          heslo(5,16) = 3.*a*s11*t2*z11
          heslo(6,16) = 3.*a*s1*t21*z11
                                   
          heslo(1,17) = 3.*a*s12*t1*z2 
          heslo(2,17) = 3.*a*s1*t12*z2 
          heslo(3,17) = 3.*a*s1*t1*z22 
          heslo(4,17) = 3.*a*s11*t11*z2
          heslo(5,17) = 3.*a*s11*t1*z21
          heslo(6,17) = 3.*a*s1*t11*z21
                                      
          heslo(1,18) = 3.*a*s42*t1*z2 
          heslo(2,18) = 3.*a*s4*t12*z2 
          heslo(3,18) = 3.*a*s4*t1*z22 
          heslo(4,18) = 3.*a*s41*t11*z2
          heslo(5,18) = 3.*a*s41*t1*z21
          heslo(6,18) = 3.*a*s4*t11*z21
                                      
          heslo(1,19) = 3.*a*s42*t4*z2 
          heslo(2,19) = 3.*a*s4*t42*z2 
          heslo(3,19) = 3.*a*s4*t4*z22 
          heslo(4,19) = 3.*a*s41*t41*z2
          heslo(5,19) = 3.*a*s41*t4*z21
          heslo(6,19) = 3.*a*s4*t41*z21
                                                                            
          heslo(1,20) = 3.*a*s12*t4*z2 
          heslo(2,20) = 3.*a*s1*t42*z2 
          heslo(3,20) = 3.*a*s1*t4*z22 
          heslo(4,20) = 3.*a*s11*t41*z2
          heslo(5,20) = 3.*a*s11*t4*z21
          heslo(6,20) = 3.*a*s1*t41*z21
                                      
          heslo(1,21) = 3.*a*s12*t1*z3 
          heslo(2,21) = 3.*a*s1*t12*z3 
          heslo(3,21) = 3.*a*s1*t1*z32 
          heslo(4,21) = 3.*a*s11*t11*z3
          heslo(5,21) = 3.*a*s11*t1*z31
          heslo(6,21) = 3.*a*s1*t11*z31
                                      
          heslo(1,22) = 3.*a*s42*t1*z3 
          heslo(2,22) = 3.*a*s4*t12*z3 
          heslo(3,22) = 3.*a*s4*t1*z32 
          heslo(4,22) = 3.*a*s41*t11*z3
          heslo(5,22) = 3.*a*s41*t1*z31
          heslo(6,22) = 3.*a*s4*t11*z31
                                      
          heslo(1,23) = 3.*a*s42*t4*z3 
          heslo(2,23) = 3.*a*s4*t42*z3 
          heslo(3,23) = 3.*a*s4*t4*z32 
          heslo(4,23) = 3.*a*s41*t41*z3
          heslo(5,23) = 3.*a*s41*t4*z31
          heslo(6,23) = 3.*a*s4*t41*z31
                                      
          heslo(1,24) = 3.*a*s12*t4*z3 
          heslo(2,24) = 3.*a*s1*t42*z3 
          heslo(3,24) = 3.*a*s1*t4*z32 
          heslo(4,24) = 3.*a*s11*t41*z3
          heslo(5,24) = 3.*a*s11*t4*z31
          heslo(6,24) = 3.*a*s1*t41*z31
                                                                         
          heslo(1,25) = 3.*a*s22*t4*z4 
          heslo(2,25) = 3.*a*s2*t42*z4 
          heslo(3,25) = 3.*a*s2*t4*z42 
          heslo(4,25) = 3.*a*s21*t41*z4
          heslo(5,25) = 3.*a*s21*t4*z41
          heslo(6,25) = 3.*a*s2*t41*z41
                                      
          heslo(1,26) = 3.*a*s32*t1*z4 
          heslo(2,26) = 3.*a*s3*t12*z4 
          heslo(3,26) = 3.*a*s3*t1*z42 
          heslo(4,26) = 3.*a*s31*t11*z4
          heslo(5,26) = 3.*a*s31*t1*z41
          heslo(6,26) = 3.*a*s3*t11*z41
                                      
          heslo(1,27) = 3.*a*s42*t2*z4 
          heslo(2,27) = 3.*a*s4*t22*z4 
          heslo(3,27) = 3.*a*s4*t2*z42 
          heslo(4,27) = 3.*a*s41*t21*z4
          heslo(5,27) = 3.*a*s41*t2*z41
          heslo(6,27) = 3.*a*s4*t21*z41
                                      
          heslo(1,28) = 3.*a*s42*t3*z4 
          heslo(2,28) = 3.*a*s4*t32*z4 
          heslo(3,28) = 3.*a*s4*t3*z42 
          heslo(4,28) = 3.*a*s41*t31*z4
          heslo(5,28) = 3.*a*s41*t3*z41
          heslo(6,28) = 3.*a*s4*t31*z41
                                      
          heslo(1,29) = 3.*a*s32*t4*z4 
          heslo(2,29) = 3.*a*s3*t42*z4 
          heslo(3,29) = 3.*a*s3*t4*z42 
          heslo(4,29) = 3.*a*s31*t41*z4
          heslo(5,29) = 3.*a*s31*t4*z41
          heslo(6,29) = 3.*a*s3*t41*z41
                                                                            
          heslo(1,30) = 3.*a*s22*t4*z4 
          heslo(2,30) = 3.*a*s2*t42*z4 
          heslo(3,30) = 3.*a*s2*t4*z42 
          heslo(4,30) = 3.*a*s21*t41*z4
          heslo(5,30) = 3.*a*s21*t4*z41
          heslo(6,30) = 3.*a*s2*t41*z31
                                      
          heslo(1,31) = 3.*a*s12*t3*z4 
          heslo(2,31) = 3.*a*s1*t32*z4 
          heslo(3,31) = 3.*a*s1*t3*z42 
          heslo(4,31) = 3.*a*s11*t31*z4
          heslo(5,31) = 3.*a*s11*t3*z41
          heslo(6,31) = 3.*a*s1*t31*z41
                                      
          heslo(1,32) = 3.*a*s12*t2*z4 
          heslo(2,32) = 3.*a*s1*t22*z4 
          heslo(3,32) = 3.*a*s1*t2*z42 
          heslo(4,32) = 3.*a*s11*t21*z4
          heslo(5,32) = 3.*a*s11*t2*z41
          heslo(6,32) = 3.*a*s1*t21*z41
                                   
          heslo(1,33) =-9.*a*s22*t2*z1 
          heslo(2,33) =-9.*a*s2*t22*z1 
          heslo(3,33) =-9.*a*s2*t2*z12 
          heslo(4,33) =-9.*a*s21*t21*z1
          heslo(5,33) =-9.*a*s21*t2*z11
          heslo(6,33) =-9.*a*s2*t21*z11
                                   
          heslo(1,34) =-9.*a*s32*t2*z1 
          heslo(2,34) =-9.*a*s3*t22*z1 
          heslo(3,34) =-9.*a*s3*t2*z12 
          heslo(4,34) =-9.*a*s31*t21*z1
          heslo(5,34) =-9.*a*s31*t2*z11
          heslo(6,34) =-9.*a*s3*t21*z11
                                   
          heslo(1,35) =-9.*a*s32*t3*z1 
          heslo(2,35) =-9.*a*s3*t32*z1 
          heslo(3,35) =-9.*a*s3*t3*z12 
          heslo(4,35) =-9.*a*s31*t31*z1
          heslo(5,35) =-9.*a*s31*t3*z11
          heslo(6,35) =-9.*a*s3*t31*z11
                                   
          heslo(1,36) =-9.*a*s22*t3*z1 
          heslo(2,36) =-9.*a*s2*t32*z1 
          heslo(3,36) =-9.*a*s2*t3*z12 
          heslo(4,36) =-9.*a*s21*t31*z1
          heslo(5,36) =-9.*a*s21*t3*z11
          heslo(6,36) =-9.*a*s2*t31*z11
                                   
          heslo(1,37) =-9.*a*s22*t1*z2 
          heslo(2,37) =-9.*a*s2*t12*z2 
          heslo(3,37) =-9.*a*s2*t1*z22 
          heslo(4,37) =-9.*a*s21*t11*z2
          heslo(5,37) =-9.*a*s21*t1*z21
          heslo(6,37) =-9.*a*s2*t11*z21
                                      
          heslo(1,38) =-9.*a*s32*t1*z2 
          heslo(2,38) =-9.*a*s3*t12*z2 
          heslo(3,38) =-9.*a*s3*t1*z22 
          heslo(4,38) =-9.*a*s31*t11*z2
          heslo(5,38) =-9.*a*s31*t1*z21
          heslo(6,38) =-9.*a*s3*t11*z21
                                      
          heslo(1,39) =-9.*a*s42*t2*z2 
          heslo(2,39) =-9.*a*s4*t22*z2 
          heslo(3,39) =-9.*a*s4*t2*z22 
          heslo(4,39) =-9.*a*s41*t21*z2
          heslo(5,39) =-9.*a*s41*t2*z21
          heslo(6,39) =-9.*a*s4*t21*z21
                                                                            
          heslo(1,40) =-9.*a*s42*t3*z2 
          heslo(2,40) =-9.*a*s4*t32*z2 
          heslo(3,40) =-9.*a*s4*t3*z22 
          heslo(4,40) =-9.*a*s41*t31*z2
          heslo(5,40) =-9.*a*s41*t3*z21
          heslo(6,40) =-9.*a*s4*t31*z21
                                   
          heslo(1,41) =-9.*a*s32*t4*z2 
          heslo(2,41) =-9.*a*s3*t42*z2 
          heslo(3,41) =-9.*a*s3*t4*z22 
          heslo(4,41) =-9.*a*s31*t41*z2
          heslo(5,41) =-9.*a*s31*t4*z21
          heslo(6,41) =-9.*a*s3*t41*z21
                                      
          heslo(1,42) =-9.*a*s22*t4*z2 
          heslo(2,42) =-9.*a*s2*t42*z2 
          heslo(3,42) =-9.*a*s2*t4*z22 
          heslo(4,42) =-9.*a*s21*t41*z2
          heslo(5,42) =-9.*a*s21*t4*z21
          heslo(6,42) =-9.*a*s2*t41*z21
                                      
          heslo(1,43) =-9.*a*s12*t3*z2 
          heslo(2,43) =-9.*a*s1*t32*z2 
          heslo(3,43) =-9.*a*s1*t3*z22 
          heslo(4,43) =-9.*a*s11*t31*z2
          heslo(5,43) =-9.*a*s11*t3*z21
          heslo(6,43) =-9.*a*s1*t31*z21
                                      
          heslo(1,44) =-9.*a*s12*t2*z2 
          heslo(2,44) =-9.*a*s1*t22*z2 
          heslo(3,44) =-9.*a*s1*t2*z22 
          heslo(4,44) =-9.*a*s11*t21*z2
          heslo(5,44) =-9.*a*s11*t2*z21
          heslo(6,44) =-9.*a*s1*t21*z21
                                   
          heslo(1,45) =-9.*a*s22*t1*z3 
          heslo(2,45) =-9.*a*s2*t12*z3 
          heslo(3,45) =-9.*a*s2*t1*z32 
          heslo(4,45) =-9.*a*s21*t11*z3
          heslo(5,45) =-9.*a*s21*t1*z31
          heslo(6,45) =-9.*a*s2*t11*z31
                                      
          heslo(1,46) =-9.*a*s32*t1*z3 
          heslo(2,46) =-9.*a*s3*t12*z3 
          heslo(3,46) =-9.*a*s3*t1*z32 
          heslo(4,46) =-9.*a*s31*t11*z3
          heslo(5,46) =-9.*a*s31*t1*z31
          heslo(6,46) =-9.*a*s3*t11*z31
                                      
          heslo(1,47) =-9.*a*s42*t2*z3 
          heslo(2,47) =-9.*a*s4*t22*z3 
          heslo(3,47) =-9.*a*s4*t2*z32 
          heslo(4,47) =-9.*a*s41*t21*z3
          heslo(5,47) =-9.*a*s41*t2*z31
          heslo(6,47) =-9.*a*s4*t21*z31
                                      
          heslo(1,48) =-9.*a*s42*t3*z3 
          heslo(2,48) =-9.*a*s4*t32*z3 
          heslo(3,48) =-9.*a*s4*t3*z32 
          heslo(4,48) =-9.*a*s41*t31*z3
          heslo(5,48) =-9.*a*s41*t3*z31
          heslo(6,48) =-9.*a*s4*t31*z31
                                   
          heslo(1,49) =-9.*a*s32*t4*z3 
          heslo(2,49) =-9.*a*s3*t42*z3 
          heslo(3,49) =-9.*a*s3*t4*z32 
          heslo(4,49) =-9.*a*s31*t41*z3
          heslo(5,49) =-9.*a*s31*t4*z31
          heslo(6,49) =-9.*a*s3*t41*z31
                                                                            
          heslo(1,50) =-9.*a*s22*t4*z3 
          heslo(2,50) =-9.*a*s2*t42*z3 
          heslo(3,50) =-9.*a*s2*t4*z32 
          heslo(4,50) =-9.*a*s21*t41*z3
          heslo(5,50) =-9.*a*s21*t4*z31
          heslo(6,50) =-9.*a*s2*t41*z31
                                      
          heslo(1,51) =-9.*a*s12*t3*z3 
          heslo(2,51) =-9.*a*s1*t32*z3 
          heslo(3,51) =-9.*a*s1*t3*z32 
          heslo(4,51) =-9.*a*s11*t31*z3
          heslo(5,51) =-9.*a*s11*t3*z31
          heslo(6,51) =-9.*a*s1*t31*z31
                                      
          heslo(1,52) =-9.*a*s12*t2*z3 
          heslo(2,52) =-9.*a*s1*t22*z3 
          heslo(3,52) =-9.*a*s1*t2*z32 
          heslo(4,52) =-9.*a*s11*t21*z3
          heslo(5,52) =-9.*a*s11*t2*z31
          heslo(6,52) =-9.*a*s1*t21*z31
                                      
          heslo(1,53) =-9.*a*s22*t2*z4 
          heslo(2,53) =-9.*a*s2*t22*z4 
          heslo(3,53) =-9.*a*s2*t2*z42 
          heslo(4,53) =-9.*a*s21*t21*z4
          heslo(5,53) =-9.*a*s21*t2*z41
          heslo(6,53) =-9.*a*s2*t21*z41
                                   
          heslo(1,54) =-9.*a*s32*t2*z4 
          heslo(2,54) =-9.*a*s3*t22*z4 
          heslo(3,54) =-9.*a*s3*t2*z42 
          heslo(4,54) =-9.*a*s31*t21*z4
          heslo(5,54) =-9.*a*s31*t2*z41
          heslo(6,54) =-9.*a*s3*t21*z41
                                      
          heslo(1,55) =-9.*a*s32*t3*z4 
          heslo(2,55) =-9.*a*s3*t32*z4 
          heslo(3,55) =-9.*a*s3*t3*z42 
          heslo(4,55) =-9.*a*s31*t31*z4
          heslo(5,55) =-9.*a*s31*t3*z41
          heslo(6,55) =-9.*a*s3*t31*z41
                                      
          heslo(1,56) =-9.*a*s22*t3*z4 
          heslo(2,56) =-9.*a*s2*t32*z4 
          heslo(3,56) =-9.*a*s2*t3*z42 
          heslo(4,56) =-9.*a*s21*t31*z4
          heslo(5,56) =-9.*a*s21*t3*z41
          heslo(6,56) =-9.*a*s2*t31*z41
                                      
          heslo(1,57) =27.*a*s22*t2*z2 
          heslo(2,57) =27.*a*s2*t22*z2 
          heslo(3,57) =27.*a*s2*t2*z22 
          heslo(4,57) =27.*a*s21*t21*z2
          heslo(5,57) =27.*a*s21*t2*z21
          heslo(6,57) =27.*a*s2*t21*z21
                                   
          heslo(1,58) =27.*a*s32*t2*z2 
          heslo(2,58) =27.*a*s3*t22*z2 
          heslo(3,58) =27.*a*s3*t2*z22 
          heslo(4,58) =27.*a*s31*t21*z2
          heslo(5,58) =27.*a*s31*t2*z21
          heslo(6,58) =27.*a*s3*t21*z21
                                   
          heslo(1,59) =27.*a*s32*t3*z2 
          heslo(2,59) =27.*a*s3*t32*z2 
          heslo(3,59) =27.*a*s3*t3*z22 
          heslo(4,59) =27.*a*s31*t31*z2
          heslo(5,59) =27.*a*s31*t3*z21
          heslo(6,59) =27.*a*s3*t31*z21
                                                                            
          heslo(1,60) =27.*a*s22*t3*z2 
          heslo(2,60) =27.*a*s2*t32*z2 
          heslo(3,60) =27.*a*s2*t3*z22 
          heslo(4,60) =27.*a*s21*t31*z2
          heslo(5,60) =27.*a*s21*t3*z21
          heslo(6,60) =27.*a*s2*t31*z21
                                      
          heslo(1,61) =27.*a*s22*t2*z3 
          heslo(2,61) =27.*a*s2*t22*z3 
          heslo(3,61) =27.*a*s2*t2*z32 
          heslo(4,61) =27.*a*s21*t21*z3
          heslo(5,61) =27.*a*s21*t2*z31
          heslo(6,61) =27.*a*s2*t21*z31
                                      
          heslo(1,62) =27.*a*s32*t2*z3 
          heslo(2,62) =27.*a*s3*t22*z3 
          heslo(3,62) =27.*a*s3*t2*z32 
          heslo(4,62) =27.*a*s31*t21*z3
          heslo(5,62) =27.*a*s31*t2*z31
          heslo(6,62) =27.*a*s3*t21*z31
                                                
          heslo(1,63) =27.*a*s32*t3*z3 
          heslo(2,63) =27.*a*s3*t32*z3 
          heslo(3,63) =27.*a*s3*t3*z32 
          heslo(4,63) =27.*a*s31*t31*z3
          heslo(5,63) =27.*a*s31*t3*z31
          heslo(6,63) =27.*a*s3*t31*z31
                                      
          heslo(1,64) =27.*a*s22*t3*z3 
          heslo(2,64) =27.*a*s2*t32*z3 
          heslo(3,64) =27.*a*s2*t3*z32 
          heslo(4,64) =27.*a*s21*t31*z3
          heslo(5,64) =27.*a*s21*t3*z31
          heslo(6,64) =27.*a*s2*t31*z31

        end if
      else
        istop=1
      end if
c
c*** Errors
c      
      if(istop.eq.1)
     .  call runend('SHCOS3: NOT AVAILABLE ELEMENT INERPOLATION')
      
      end
