      subroutine shdis2(    s,    t,nelty,nnode,
     .                  shape,deriv)
c***********************************************************************
c
c**** This routine evaluates shape functions and their first derivates 
c**** for 2-d discontinuos standar interpolation elements.
c
c                TRIANGLES:        1  3  &  6        nodes
c                QUADRILATERALS:   1  3  4  6  &  9  nodes
c
c***********************************************************************
      implicit real*8(a-h,o-z)
      real*8 deriv(2,nnode), shape(nnode)
      integer   istop
      istop=0
c
c*** One node
c
      if(nnode.eq.1) then         
        shape(  1)= 1.0                                
        deriv(1,1)= 0.0
        deriv(2,1)= 0.0
c
c*** Triangles
c      
      else if(nelty.eq.2) then
        if(nnode.eq.3) then
          a1=1.0-s-t
          a2=s
          a3=t
          shape(  1)=(7.0/4.0)*(a1-1.0/7.0) 
          shape(  2)=(7.0/4.0)*(a2-1.0/7.0)   
          shape(  3)=(7.0/4.0)*(a3-1.0/7.0)   
          deriv(1,1)=-7.0/4.0
          deriv(2,1)=-7.0/4.0
          deriv(1,2)= 7.0/4.0
          deriv(2,2)= 0.0
          deriv(1,3)= 0.0
          deriv(2,3)= 7.0/4.0
        else if(nnode.eq.6) then
          c=49.0/8.0
          a1=6.0/7.0-s-t
          a4=4.0/7.0-s-t                                    !  3       
          shape(1)=c*a4*a1                                  !
          shape(2)=c*(t-3.0/7.0)*(t-1.0/7.0)                !
          shape(3)=c*(s-3.0/7.0)*(s-1.0/7.0)                !  6      5     
          shape(4)=c*a1*(t-1.0/7.0)                         !
          shape(5)=c*(s-1.0/7.0)*(t-1.0/7.0)                !               
          shape(6)=c*a1*(s-1.0/7.0)                         !  1     4     2
          deriv(1,1)=-c*(a1+a4)
          deriv(1,2)=0.0
          deriv(1,3)=c*(2.0*s-4.0/7.0)
          deriv(1,4)=c*(1.0/7.0-t)
          deriv(1,5)=c*(t-1.0/7.0)
          deriv(1,6)=c*(1.0/7.0-s+a1)
          deriv(2,1)=-c*(a1+a4)
          deriv(2,2)=c*(2.0*t-4.0/7.0)
          deriv(2,3)=0.0
          deriv(2,4)=c*(1.0/7.0-t+a1)
          deriv(2,5)=c*(s-1.0/7.0)
          deriv(2,6)=c*(1.0/7.0-s)
        else
          istop=1
        end if
c         
c*** Quadrilaterals
c         
      else if(nelty.eq.1) then
        if(nnode.eq.3) then
          sq23=sqrt(2.0)/sqrt(3.0)
          sq16=     1.0 /sqrt(6.0)
          co30=sqrt(3.0)/     2.0
          shape( 1)=sq23*(t+sq16)     
          shape( 2)=sq23*(-s*co30-t*0.5+sq16)
          shape( 3)=sq23*( s*co30-t*0.5+sq16)
          deriv(1,1)= 0.0
          deriv(1,2)=-sq23*co30
          deriv(1,3)= sq23*co30
          deriv(2,1)= sq23
          deriv(2,2)=-sq16
          deriv(2,3)=-sq16
        else if(nnode.eq.4) then
          a = -0.577350269189626
          b =  0.25/(a*a)
          shape(  1)= b*( a-s)*( a-t)
          shape(  2)= b*(-a-s)*( a-t)
          shape(  3)= b*(-a-s)*(-a-t)
          shape(  4)= b*( a-s)*(-a-t)
          deriv(1,1)=-b*       ( a-t)
          deriv(1,2)=-b*       ( a-t)
          deriv(1,3)=-b*       (-a-t)
          deriv(1,4)=-b*       (-a-t)
          deriv(2,1)=-b*( a-s)
          deriv(2,2)=-b*(-a-s)
          deriv(2,3)=-b*(-a-s)
          deriv(2,4)=-b*( a-s)
        else if(nnode.eq.6) then
          a=1/sqrt(6.0)
          b=sqrt(3.0)
          c=1.0/3.0
          shape(1)=2.0*c*  (2.0*t-a)      *     (t+a)        
          shape(2)=   c *(-b*s-t+2.0*a)   *   (-b*s-t-a)
          shape(3)=   c *(b*s-t+2.0*a)    *   (b*s-t-a)
          shape(4)=8.0*c*(0.5*b*s-0.5*t+a)*(-0.5*b*s-0.5*t+a)
          shape(5)=8.0*c*    (t+a)        *(0.5*b*s-0.5*t+a)
          shape(6)=8.0*c*    (t+a)        *(-0.5*b*s-0.5*t+a)
	  deriv(1,1)=0.0                                    !         1   
          deriv(1,2)=c*(-b)*(-2.0*b*s-2.0*t+a)              !          
          deriv(1,3)=c*b*(2.0*b*s-2.0*t+a)                  !             
          deriv(1,4)=-4.0*s                                 !       
          deriv(1,5)=8.0*c*0.5*b*(t+a)                      !      6       5   
          deriv(1,6)=8.0*c*(-0.5)*b*(t+a)                   !     
          deriv(2,1)=2.0*c*(4*t+a)                          ! 
          deriv(2,2)=-c*(-2.0*b*s-2.0*t+a)                  !
          deriv(2,3)=-c*(2.0*b*s-2.0*t+a)                   !  2       4      3
          deriv(2,4)=-4.0*c*(-t+2.0*a)
          deriv(2,5)=8.0*c*(0.5*b*s-t+a/2.0)
          deriv(2,6)=8.0*c*(-0.5*b*s-t+a/2.0)
        else if(nnode.eq.9) then
          a = sqrt(15.0)/5.0
          b = 25.0/36.0
          shape(  1) =  b *  s  *  t  *(s-a)*(t-a)
          shape(  2) =  b *  s  *  t  *(s+a)*(t-a)          !
          shape(  3) =  b *  s  *  t  *(s-a)*(t+a)          ! 4    7    3
          shape(  4) =  b *  s  *  t  *(s+a)*(t+a)          !
          shape(  5) =-2.0*b*(s+a)*(s-a)*  t  *(t-a)        !
          shape(  6) =-2.0*b*(s+a)*  s  *(t+a)*(t-a)        ! 8    9    6
          shape(  7) =-2.0*b*(s+a)*(s-a)*(t+a)*  t          !
          shape(  8) =-2.0*b*  s  *(s-a)*(t+a)*(t-a)        !
          shape(  9) = 4.0*b*(s+a)*(s-a)*(t+a)*(t-a)        ! 1    5    2
          deriv(1,1) =  b *(2.*s-a)*  t  *(t-a)
          deriv(1,2) =  b *(2.*s+a)*  t  *(t-a)
          deriv(1,3) =  b *(2.*s-a)*  t  *(t+a)
          deriv(1,4) =  b *(2.*s+a)*  t  *(t+a)
          deriv(1,5) =-2.*b*  2.*s  *  t  *(t-a)
          deriv(1,6) =-2.*b*(2.*s+a)*(t+a)*(t-a)
          deriv(1,7) =-2.*b*  2.*s  *  t  *(t+a)
          deriv(1,8) =-2.*b*(2.*s-a)*(t+a)*(t-a)
          deriv(1,9) = 4.*b*  2.*s  *(t+a)*(t-a)
          deriv(2,1) =  b *  s  *(s-a)*(2.*t-a)
          deriv(2,2) =  b *  s  *(s+a)*(2.*t-a)
          deriv(2,3) =  b *  s  *(s-a)*(2.*t+a)
          deriv(2,4) =  b *  s  *(s+a)*(2.*t+a)
          deriv(2,5) =-2.*b*(s+a)*(s-a)*(2.*t-a)
          deriv(2,6) =-2.*b*  s  *(s+a)*  2.*t
          deriv(2,7) =-2.*b*(s+a)*(s-a)*(2.*t+a)
          deriv(2,8) =-2.*b*  s  *(s-a)*  2.*t
          deriv(2,9) = 4.*b*(s+a)*(s-a)*  2.*t
        else
          istop=1
        end if
      end if
c
c*** Errors have been detected
c
      if(istop.eq.1) call runend('SHDIS2: NOT AVAILABLE ELEMENT')
      
      end



