      subroutine shdis3(    s,    t,     z,nelty,nnode,
     .                  shape,deriv)
c***********************************************************************
c
c**** This routine evaluates shape functions and their first derivates 
c**** for 3-d discontinuos standar interpolation elements:
c
c                TETRAHEDRA:   1     &  4   nodes
c                HEXAHEDRA:    1  4  &  8   nodes
c
c***********************************************************************
      implicit none
      integer   nelty,nnode,istop
      real*8    s,t,z,a1,a2,a3,a4,sl,tl,zl,sp,tp,zp,a,p1,p2,p3,p4,
     .          p5,p6,p7,p8
      real*8    shape(nnode), deriv(3,nnode)
      istop=0
c
c*** One node
c      
      if (nnode.eq.1) then
        shape(  1) = 1.0
        deriv(1,1) = 0.0
        deriv(2,1) = 0.0
        deriv(3,1) = 0.0
c
c*** Tetrahedra
c
      else if (nelty.eq.2) then
        if(nnode.eq.4) then
          a1=1.-s-t-z
          a2=s
          a3=t
          a4=z
          shape(  1) = 7./3.*(a1-1./7.)
          deriv(1,1) =-7./3.
          deriv(2,1) =-7./3.
          deriv(3,1) =-7./3.
          shape(  2) = 7./3.*(a2-1./7.)
          deriv(1,2) = 7./3.
          deriv(2,2) = 0.
          deriv(3,2) = 0.
          shape(  3) = 7./3.*(a3-1./7.)
          deriv(1,3) = 0.
          deriv(2,3) = 7./3.
          deriv(3,3) = 0.
          shape(  4) = 7./3.*(a4-1./7.)
          deriv(1,4) = 0.
          deriv(2,4) = 0.
          deriv(3,4) = 7./3.
        else if(nnode.eq.10) then
          a=98.0/9.0
          p1=9.0/14.0-s-t-z
          p2=6.0/7.0-s-t-z
          p3=z-5.0/14.0
          p4=z-1.0/7.0
          p5=t-5.0/14.0
          p6=t-1.0/7.0
          p7=s-5.0/14.0
          p8=s-1.0/7.0
          shape(   1) =  a*p1*p2  
          shape(   2) =  a*p3*p4  
          shape(   3) =  a*p5*p6  
          shape(   4) =  a*p7*p8  
          shape(   5) =2.0*a*p8*p2  
          shape(   6) =2.0*a*p6*p8  
          shape(   7) =2.0*a*p6*p2  
          shape(   8) =2.0*a*p4*p6  
          shape(   9) =2.0*a*p4*p8  
          shape(  10) =2.0*a*p6*p4  
          deriv(1, 1) =-a*(p1+p2)
          deriv(2, 1) =-a*(p1+p2)
          deriv(3, 1) =-a*(p1+p2)
          deriv(1, 2) = 0.0
          deriv(2, 2) = 0.0
          deriv(3, 2) = a*(p3+p4)
          deriv(1, 3) = 0.0
          deriv(2, 3) = a*(p5+p6)
          deriv(3, 3) = 0.0
          deriv(1, 4) = a*(p7+p8)
          deriv(2, 4) = 0.0
          deriv(3, 4) = 0.0
          deriv(1, 5) =-2.*a*(p8-p2)
          deriv(2, 5) =-2.*a*p8
          deriv(3, 5) =-2.*a*p8
          deriv(1, 6) = 2.*a*p8
          deriv(2, 6) = 2.*a*p6
          deriv(3, 6) = 0.0
          deriv(1, 7) =-2.*a*p6
          deriv(2, 7) =-2.*a*(p6-p2)
          deriv(3, 7) =-2.*a*p6
          deriv(1, 8) =-2.*a*p4
          deriv(2, 8) =-2.*a*p4
          deriv(3, 8) =-2.*a*(p4-p2)
          deriv(1, 9) = 2.*a*p4
          deriv(2, 9) = 0.0
          deriv(3, 9) = 2.*a*p8
          deriv(1,10) = 0.0
          deriv(2,10) = 2.*a*p4
          deriv(3,10) = 2.*a*p6
        else
          istop=1
        end if
c
c*** Hexahedra
c        
      else if (nelty.eq.1) then
        if(nnode.eq.4) then
          shape(  1) = 7./24.*(6./7.-s-t+z)
          deriv(1,1) =-7./24.
          deriv(2,1) =-7./24.
          deriv(3,1) = 7./24.
          shape(  2) = 7./24.*(6./7.+s+t+z)
          deriv(1,2) = 7./24.
          deriv(2,2) = 7./24.
          deriv(3,2) = 7./24.
          shape(  3) = 7./24.*(6./7.+s-t-z)
          deriv(1,3) = 7./24.
          deriv(2,3) =-7./24.
          deriv(3,3) =-7./24.
          shape(  4) = 7./24.*(6./7.-s+t-z)
          deriv(1,4) =-7./24.
          deriv(2,4) = 7./24.
          deriv(3,4) =-7./24.
        else if (nnode.eq.8) then
          sl = s-5./6.
          tl = t-5./6.
          zl = z-5./6.
          sp = s+5./6.
          tp = t+5./6.
          zp = z+5./6.
          shape(  1) =-27./125.*sl*tl*zl
          deriv(1,1) =-27./125.*tl*zl
          deriv(2,1) =-27./125.*sl*zl
          deriv(3,1) =-27./125.*sl*tl
          deriv(1,2) = 27./125.*tl*zl
          deriv(2,2) = 27./125.*sp*zl
          deriv(3,2) = 27./125.*sp*tl
          shape(  3) =-27./125.*sp*tp*zl
          deriv(1,3) =-27./125.*tp*zl
          deriv(2,3) =-27./125.*sp*zl
          deriv(3,3) =-27./125.*sp*tp
          shape(  4) = 27./125.*sl*tp*zl
          deriv(1,4) = 27./125.*tp*zl
          deriv(2,4) = 27./125.*sl*zl
          deriv(3,4) = 27./125.*sl*tp
          shape(  5) = 27./125.*sl*tl*zp
          deriv(1,5) = 27./125.*tl*zp
          deriv(2,5) = 27./125.*sl*zp
          deriv(3,5) = 27./125.*sl*tl
          shape(  6) =-27./125.*sp*tl*zp
          deriv(1,6) =-27./125.*tl*zp
          deriv(2,6) =-27./125.*sp*zp
          deriv(3,6) =-27./125.*sp*tl
          shape(  7) = 27./125.*sp*tp*zp
          deriv(1,7) = 27./125.*tp*zp
          deriv(2,7) = 27./125.*sp*zp
          deriv(3,7) = 27./125.*sp*tp
          shape(  8) =-27./125.*sl*tp*zp
          deriv(1,8) =-27./125.*tp*zp
          deriv(2,8) =-27./125.*sl*zp
          deriv(3,8) =-27./125.*sl*tp
        else if(nnode.eq.10) then
          a=49.0/288.0
          p1=-6.0/7.0-s-t+z
          p2=6.0/7.0-s-t+z
          p3=-6.0/7.0+s+t+z
          p4=6.0/7.0+s+t+z
          p5=-6.0/7.0+s-t-z
          p6=6.0/7.0+s-t-z
          p7=-6.0/7.0-s+t-z
          p8=6.0/7.0-s+t-z
          shape(   1) = a*p1*p2
          shape(   2) = a*p3*p4
          shape(   3) = a*p5*p6
          shape(   4) = a*p7*p8
          shape(   5) = a*p6*p2
          shape(   6) = a*p4*p2
          shape(   7) = a*p8*p2
          shape(   8) = a*p4*p6
          shape(   9) = a*p4*p8
          shape(  10) = a*p6*p8
          deriv(1, 1) =-a*(p1+p2)
          deriv(2, 1) =-a*(p1+p2)
          deriv(3, 1) = a*(p1+p2)
          deriv(1, 2) = a*(p3+p4)
          deriv(2, 2) = a*(p3+p4)
          deriv(3, 2) = a*(p3+p4)
          deriv(1, 3) = a*(p5+p6)
          deriv(2, 3) =-a*(p5+p6)
          deriv(3, 3) =-a*(p5+p6)
          deriv(1, 4) =-a*(p7+p8)
          deriv(2, 4) = a*(p7+p8)
          deriv(3, 4) =-a*(p7+p8)
          deriv(1, 5) = a*(p6-p2)
          deriv(2, 5) =-a*(p6+p2)
          deriv(3, 5) = a*(p6-p2)        
          deriv(1, 6) = a*(p4-p2)
          deriv(2, 6) = a*(p4-p2)
          deriv(3, 6) = a*(p4+p2)        
          deriv(1, 7) =-a*(p8+p2)
          deriv(2, 7) =-a*(p8-p2)
          deriv(3, 7) = a*(p8-p2)
          deriv(1, 8) = a*(p4+p6)
          deriv(2, 8) =-a*(p4-p6)
          deriv(3, 8) =-a*(p4-p6)        
          deriv(1, 9) =-a*(p4-p8)
          deriv(2, 9) = a*(p4+p8)
          deriv(3, 9) =-a*(p4-p8)        
          deriv(1,10) =-a*(p6-p8)
          deriv(2,10) = a*(p6-p8)
          deriv(3,10) =-a*(p6+p8)
        else
          istop=1
        end if
      end if
      
      end
