      subroutine trahes(heslo,hessi,ndime,nnode,ntens,xjaci,wmat1,
     .                  wmat2,d2sdx,deriv,elcod)
c*************************************************************************
c                                                       
c**** This routine computes the Hessian matrix in the Cartesian system
c**** of coordinates according to the rule
c
c     d^2 N / d x_i d x_j
c        = (d^2 N / d s_k d s_l) (d s_k/ d x_i)(d s_l/ d x_j)
c        + (d N / d s_k) (d^2 s_k / d x_i d x_j) 
c
c*************************************************************************
      implicit none
      integer    ndime,nnode,ntens,inode,idime,jdime,kdime,ldime
      real*8     heslo(ntens,*),       hessi(ntens,*),
     .           xjaci(ndime,*),       wmat1(ndime,ndime,*),
     .           wmat2(ndime,ndime,*), d2sdx(ndime,ndime,*),
     .           deriv(ndime,*),       elcod(ndime,*)
c
c*** Transforms the array HESLO to a symmetric matrix WMAT1
c
      do inode=1,nnode
        call vetoma(heslo(1,inode),wmat1(1,1,inode),ndime,ntens)
      end do
c
c*** Computes (d^2 N / d s_k d s_l) (d s_k/ d x_i)(d s_l/ d x_j) for
c*** each node
c
      do inode=1,nnode
        call btdbma(wmat2(1,1,inode),wmat1(1,1,inode),xjaci,ndime,ndime)
      end do
c
c*** Obtains (d^2 s_k / d x_i d x_j) as the solution of the system
c*** (d x_l / d s_k) (d^2 s_k / d x_i d x_j) 
c***     = - (d^2 x_l / d s_k d s_l) (d s_k/ d x_i)(d s_l/ d x_j), 
c*** for l,i,j = 1,...,NDIME
c
      do kdime=1,ndime
        do idime=1,ndime
          do jdime=1,ndime
            d2sdx(kdime,idime,jdime)=0.0
            do ldime=1,ndime
              do inode=1,nnode
                d2sdx(kdime,idime,jdime)=d2sdx(kdime,idime,jdime)
     .            -xjaci(kdime,ldime)*wmat2(idime,jdime,inode)
     .            *elcod(ldime,inode)
              end do
            end do
          end do
        end do
      end do
c
c*** Computes the second Cartesian derivatives of the shape functions
c
      do inode=1,nnode
        do idime=1,ndime
          do jdime=1,ndime
            do kdime=1,ndime
              wmat2(idime,jdime,inode)=wmat2(idime,jdime,inode)
     .          +deriv(kdime,inode)*d2sdx(kdime,idime,jdime)
            end do
          end do
        end do
      end do
c
c*** Writes the Hessian matrix as an array
c
      do inode=1,nnode
        call matove(wmat2(1,1,inode),hessi(1,inode),ndime,ntens)
      end do

      end
