      subroutine exacso(
     .  exvel,exveg,expre,exprg,force,
     .  gpcod,tactu,itask)
c**************************************************************************
c
c**** This routine computes the exact solution at the integration points.
c**** For ITASK = 1 it gives u, grad(u), p & grad(p), and for ITASK = 2
c**** the force vector. Each particular case must be coded. 
c
c**************************************************************************
      implicit none
      include 'com_prob.f'
      integer  itask
      real*8
     .  exvel(ndime), exveg(ndime,ndime), expre,
     .  exprg(ndime), force(ndime),       
     .  gpcod(ndime), tactu
      real*8
     .  x,y,z,a,f0,df0,d2f0,d3f0,f,df,d2f,d3f,g,dg,d2g,d3g,
     .  u,v,dudx,dudy,dvdx,dvdy,p,dpdx,dpdy,d2udx,d2udy,d2vdx,
     .  d2vdy,h,dhdz,d2hdz,d2tdx,d2tdy,pi,convx,convy

      goto (1, 2, 3, 4) oufer
      
    1 x   = gpcod(1)
      y   = gpcod(2)
      a   = fpara(1)
      f0  = x*x*(1.0d0-x)*(1.0d0-x)
      df0 = 2.0d0*x-6.0d0*x*x+4.0d0*x*x*x
      d2f0= 2.0d0-12.0d0*x+12.0d0*x*x
      d3f0= -12.0d0+24.0d0*x
      f   = f0*exp(a*x)
      df  = df0*exp(a*x)+a*f
      d2f = d2f0*exp(a*x)+a*df0*exp(a*x)+a*df
      d3f = d3f0*exp(a*x)+a*d2f0*exp(a*x)+a*d2f0*exp(a*x)
     .      +a*a*df0*exp(a*x)+a*d2f
      g   = y*y*(1.0d0-y)*(1.0d0-y)
      dg  = 2.0d0*y-6.0d0*y*y+4.0d0*y*y*y
      d2g = 2.0d0-12.0d0*y+12.0d0*y*y
      d3g = -12.0d0+24.0d0*y
      
      u =  f*dg
      v = -df*g
      dudx =  df*dg
      dudy =  f*d2g
      dvdx = -d2f*g
      dvdy = -df*dg
      p = 0.0d0
      dpdx = 0.0d0 
      dpdy = 0.0d0 
      h     = 1.0d0
      dhdz  = 0.0d0
      d2hdz = 0.0d0
      if(ndime.eq.3) then
        z     = gpcod(3)
        h = z - z*z
        dhdz = 1.0d0 - 2.0d0*z
        d2hdz = -2.0d0
      end if
      if(itask.eq.1) then
        exvel(1)   = u*h
        exvel(2)   = v*h
        exveg(1,1) = dudx*h
        exveg(1,2) = dvdx*h
        exveg(2,1) = dudy*h
        exveg(2,2) = dvdy*h
        expre      = p
        exprg(1)   = dpdx
        exprg(2)   = dpdy
        if(ndime.eq.3) then
          exvel(3)   = 0.0d0
          exveg(1,3) = 0.0d0
          exveg(2,3) = 0.0d0
          exveg(3,3) = 0.0d0
          exveg(3,1) = u*dhdz
          exveg(3,2) = v*dhdz
          exprg(3)   = 0.0d0
        end if
      else if(itask.eq.2) then
        if(iecns.gt.0) then
          convx = h*h*u*dudx + h*h*v*dudy
          convy = h*h*u*dvdx + h*h*v*dvdy
        else
          convx = 0.0d0
          convy = 0.0d0
        end if
        d2udx = d2f*dg
        d2udy = f*d3g
        d2vdx =-d3f*g
        d2vdy =-df*d2g
        force(1) = convx
     .    - visco(1)*(h*d2udx + h*d2udy + u*d2hdz) + dpdx
     .    - 2.0d0*frano(1,2)*ofram(3,2)*v*h + poros(1)*u*h
        force(2) = convy
     .    - visco(1)*(h*d2vdx + h*d2vdy + v*d2hdz) + dpdy
     .    + 2.0d0*frano(1,2)*ofram(3,2)*u*h + poros(1)*v*h
        if(ndime.eq.3) force(3) = 2.0d0*frano(1,2)*
     .    (ofram(1,2)*v*h-ofram(2,2)*u*h)
      end if
      return
      
    2 x = gpcod(1)
      y = gpcod(2)
      if(itask.eq.1) then
        exvel(1) =  x
        exvel(2) = -y
        exveg(1,1) = 1.0d0
        exveg(1,2) = 0.0d0
        exveg(2,1) = 0.0d0
        exveg(2,2) =-1.0d0
        expre      = 0.0d0
        exprg(1)   = 0.0d0
        exprg(2)   = 0.0d0
      else if(itask.eq.2) then
        force(1) = 0.0d0
        force(2) = 0.0d0
      end if
      return
      
    3 x = gpcod(1)
      y = gpcod(2)
      if(itask.eq.1) then
        exvel(1) =  x-0.5d0
        exvel(2) = -y+0.5d0
        exveg(1,1) = 1.0d0
        exveg(1,2) = 0.0d0
        exveg(2,1) = 0.0d0
        exveg(2,2) =-1.0d0
        expre      = 0.0d0
        exprg(1)   = 0.0d0
        exprg(2)   = 0.0d0
      else if(itask.eq.2) then
        force(1) = -x+0.5d0
        force(2) = -y+0.5d0
      end if
      
      return
c
c*** Temperature equation; In (0,1)^2, u,v=0: -\Delta T = L(T*)
c    with the exact solution T*:=sin(pi*x)sin(pi*y)exp(xy)
c
    4 x = gpcod(1)
      y = gpcod(2)
      if(itask.eq.1) then
        exvel(1)   = 0.0d0
        exvel(2)   = 0.0d0
        exveg(1,1) = 0.0d0
        exveg(1,2) = 0.0d0
        exveg(2,1) = 0.0d0
        exveg(2,2) = 0.0d0
        expre      = 0.0d0
        exprg(1)   = 0.0d0
        exprg(2)   = 0.0d0
      else if(itask.eq.2) then
        pi    = 2.0d0*atan2(1.0d0,0.0d0)
        d2tdx = exp(x*y)*sin(pi*y)*
     .          ( 2.0d0*y*pi*cos(pi*x) +
     .          (y*y-pi*pi)*dsin(pi*x) )
        d2tdy = exp(x*y)*sin(pi*x)*
     .          ( 2.0d0*x*pi*cos(pi*y) +
     .          (x*x-pi*pi)*sin(pi*y) )
        force(1) = -d2tdx-d2tdy
      end if
      return
      
      end
