      subroutine nvsshc(
     .  vegau,grave,hessi,cartd,shape,elvel,elpre,
     .  elext,corif,ofram,wmatr,ndime,nnode,nnods,
     .  ntens,nhess,nevab,nnodt,nevat,kscns,frano,
     .  hleng,timom,shcns,denac,vigau,pogau,dtins,
     .  ietns,iecns,ievns,zeroc)
c***********************************************************************
c
c**** This routine computes the contribution from the shock capturing
c**** term to the Navier-Stokes equations      
c
c***********************************************************************
      implicit none
      integer
     .  idime,ndime,inode,nnode,ievab,inods,nnods,
     .  ntens,nhess,nevab,nnodt,nevat,jdime,jnode,
     .  kdime,ldime,jevab,kscns,ietns,iecns,ievns
      real*8
     .  vigau,pogau,conve,velap,gradp,scvis,slvis,
     .  denac,reloc,xmuit,venor,grnor,vepar,reyno,
     .  addit,frano(4,2),hleng,timom,shcns,duvdt,dtins,
     .  zeroc,vecave,vecdot
      real*8
     .  vegau(ndime),          grave(ndime,ndime),
     .  hessi(ntens,nhess),    cartd(ndime,nnodt), 
     .  shape(nnode),          elvel(nevab,*),
     .  elpre(nnods),          elext(ndime,2),  
     .  corif(3),              ofram(3,2), 
     .  wmatr(nevat,nevat)
c
c*** Compute the velocity gradient, if it has not been calculated in RECONS
c
      if(iecns.eq.0) then
        do idime=1,ndime
          do jdime=1,ndime  
            grave(idime,jdime)=0.0d0
            jevab=jdime-ndime
            do inode=1,nnode
              jevab=jevab+ndime
              grave(idime,jdime)=grave(idime,jdime)+
     .          cartd(idime,inode)*elvel(jevab,1)
            end do
          end do
        end do
      end if
c
c*** Compute the velocity norm at the current integration point and
c*** the Coriolis term      
c
      venor=0.0d0
      do jdime=1,ndime
        venor=venor+vegau(jdime)*vegau(jdime)
        corif(jdime)=0.0d0
      end do
      if(frano(1,2).gt.zeroc) then
        xmuit=2.0d0*denac
        corif(1)=xmuit*vegau(1)
        corif(2)=xmuit*vegau(2)
        corif(3)=0.0d0
        if(ndime.eq.3) corif(3)=xmuit*vegau(3)
        call vecpro(ofram(1,2),corif,corif)
      else
        call veczer(3,corif)
      end if
c
c*** Loop over the space dimension (number of momentum equations)
c
      do idime=1,ndime
c
c*** Compute the shock-capturing viscosity for each scalar equation.
c*** The temporal derivative is included at the moment of computing
c*** the element residual. It is approximated by a one step difference.
c*** Also, the convective term is treated implicitly, no matter the type 
c*** of integration scheme employed
c
        scvis=0.0d0
        conve=0.0d0
        velap=0.0d0
        gradp=0.0d0
        duvdt=0.0d0
        if(ietns.eq.1) then
          ievab=idime
          do inode=1,nnode
            duvdt=duvdt+shape(inode)
     .        *(elvel(ievab,1)-elvel(ievab,2))
            ievab=ievab+ndime
          end do
          duvdt=duvdt/dtins
        end if
        if(iecns.eq.1)
     .    conve=vecdot(vegau,grave(1,idime),ndime)
        if(ievns.eq.1.and.nhess.ne.0) then
          ievab=idime
          do inode=1,nnode
            velap=velap
     .        +vecave(hessi(1,inode),ndime)*elvel(ievab,1)
            ievab=ievab+ndime
          end do
        end if
        do inods=1,nnods
          gradp=gradp+cartd(idime,nnode+inods)*elpre(inods)
        end do
        call vecnor(grave(1,idime),ndime,grnor,2)
        reloc=denac*conve-vigau*velap+gradp-elext(idime,1)
     .    -elext(idime,2)+corif(idime)+pogau*vegau(idime)
        if(reloc.lt.0.0) reloc=-reloc
        if((reloc.gt.zeroc).and.(grnor.gt.zeroc)) then
          vepar=reloc/grnor
          reyno=denac*vepar*hleng*0.5d0/vigau
          scvis=0.5d0*(shcns-1.0d0/reyno)*hleng*reloc/grnor
          if(scvis.lt.0.0) scvis=0.0d0
c
c*** Compute the viscosity to be introduced along the streamlines
c
          slvis=scvis                                       ! isotropic SC
          if(kscns.eq.2) then                               ! anisotropic SC
            slvis=scvis-timom*denac*venor
            if(slvis.lt.0.0) slvis=0.0d0
          end if
c
c*** Compute the contribution to the element stiffness matrix
c
          do inode=1,nnode
            ievab=(inode-1)*ndime+idime
            do jnode=1,nnode
              jevab=(jnode-1)*ndime+idime
              addit=0.0d0
              do kdime=1,ndime
                addit=addit
     .            +scvis*cartd(kdime,inode)*cartd(kdime,jnode)
                if(venor.gt.zeroc) then
                  do ldime=1,ndime
                    addit=addit+(slvis-scvis)
     .                *cartd(kdime,inode)
     .                *vegau(kdime)*vegau(ldime)
     .                *cartd(ldime,jnode)/venor
                  end do
                end if
              end do
              wmatr(ievab,jevab)=wmatr(ievab,jevab)+addit
            end do
          end do
        end if
      end do

      end

