      subroutine nvssto(
     .  wmatr,wrhsi,shape,cartd,elvel,elpre,waxis,
     .  vegau,gpcod,ndime,nnode,nnods,nnodt,nevab,
     .  nevat,naxis,ievns,ietns,itwns,inpes,fvins,
     .  penal,denac,visac,dtins,kfans,zeroc,twost)
c***********************************************************************
c
c**** This routine computes the Stokes problem contribution to the 
c**** complete Navier-Stokes equations
c
c***********************************************************************
      implicit none
      integer  ndime,nnode,nnods,nnodt,nevab,nevat,naxis,ievns,
     .         ietns,itwns,inpes,kfans,twost,idime,ievab,inode,
     .         jnode,jevab,jdime,inods,jnods,kdime,ieva1,jeva1,
     .         jeva2,knods
      real*8   gpcod,fvins,penal,denac,visac,dtins,xmui1,
     .         xmui2,addit,xmuit,prepr,zeroc,bulkv
      real*8   wmatr(nevat,nevat),         wrhsi(nevat), 
     .         shape(nnodt),               cartd(ndime,nnode),     
     .         elvel(nevab,*),             elpre(nnods,*)
      real*8   waxis(nnode*naxis),         vegau(ndime,*)
c
c*** Initializations
c
      if(fvins.gt.zeroc) bulkv = visac*(1.0d0-(fvins-1.0d0)*2.0d0/3.0d0) 
c
c*** Contribution from the temporal derivative using one-step differences
c
      if((ietns.eq.1).and.(twost.ne.1)) then
        do idime=1,ndime  
          vegau(idime,2)=0.0d0
          ievab=idime
          do inode=1,nnode
            vegau(idime,2)=vegau(idime,2)+shape(inode)*elvel(ievab,2)
            ievab=ievab+ndime
          end do
        end do
        if(itwns.eq.0) then
          xmui1=denac/dtins
          do inode=1,nnode
            xmui2=xmui1*shape(inode)
            do idime=1,ndime
              ievab=(inode-1)*ndime+idime
              wrhsi(ievab)=wrhsi(ievab)+xmui2*vegau(idime,2)
            end do
            if(kfans.eq.1) then
              do jnode=1,nnode
                addit=shape(jnode)*xmui2
                do idime=1,ndime
                  ievab=(inode-1)*ndime+idime
                  jevab=(jnode-1)*ndime+idime
                  wmatr(ievab,jevab)=wmatr(ievab,jevab)+addit
                end do
              end do
            end if
          end do
        end if
      end if
c
c*** Contribution from the temporal derivative using two-step differences
c
      if(twost.eq.1) then
        do idime=1,ndime  
          vegau(idime,2)=0.0d0
          vegau(idime,3)=0.0d0
          ievab=idime
          do inode=1,nnode
            vegau(idime,2)=vegau(idime,2)+shape(inode)*elvel(ievab,2)
            vegau(idime,3)=vegau(idime,3)+shape(inode)*elvel(ievab,3)
            ievab=ievab+ndime
          end do
        end do
        if(itwns.eq.0) then
          xmui1=0.5d0*denac/dtins
          do inode=1,nnode
            xmui2=xmui1*shape(inode)
            do idime=1,ndime
              ievab=(inode-1)*ndime+idime
              wrhsi(ievab)=wrhsi(ievab)
     .          +xmui2*(4.0d0*vegau(idime,2)-vegau(idime,3))
            end do
            if(kfans.eq.1) then
              xmui2=xmui2*3.0d0
              do jnode=1,nnode
                addit=shape(jnode)*xmui2
                do idime=1,ndime
                  ievab=(inode-1)*ndime+idime
                  jevab=(jnode-1)*ndime+idime
                  wmatr(ievab,jevab)=wmatr(ievab,jevab)+addit
                end do
              end do
            end if
          end do
        end if
      end if
c
c*** Steady state contribution of the pressure term
c
      if(kfans.eq.1) then
        do inods=1,nnods            
          do idime=1,ndime          
            ievab=idime-ndime
            xmuit=shape(nnode+inods)
            do inode=1,nnode
              ievab=ievab+ndime
              addit=cartd(idime,inode)*xmuit
              wmatr(ievab,nevab+inods)=
     .          wmatr(ievab,nevab+inods)-addit
              wmatr(nevab+inods,ievab)=
     .          wmatr(nevab+inods,ievab)-addit
            end do
          end do
        end do
        if (naxis.eq.1) then              
          if (abs(gpcod).ge.zeroc) then 
            do inode=1,nnode
              waxis(inode)=shape(inode)/gpcod
            end do
          else
            do inode=1,nnode
              waxis(inode)=cartd(1,inode)
            end do
          end if
          do inods=1,nnods
            ievab=-1
            xmuit=shape(nnode+inods)
            do inode=1,nnode
              ievab=ievab+2
              addit=waxis(inode)*xmuit
              wmatr(ievab,nevab+inods)=
     .          wmatr(ievab,nevab+inods)-addit
              wmatr(nevab+inods,ievab)=
     .          wmatr(nevab+inods,ievab)-addit
            end do
          end do
        end if
      end if
c
c*** Contribution from the penalization
c
      if(inpes.ge.1) then
        prepr=0.0d0
        if(inpes.ge.2) then                     ! Previous pressure : p(n,i-1) 
          do knods=1,nnods                      ! (iter) or p(n-1) (art. comp.)
            prepr=prepr+shape(nnode+knods)
     .      *elpre(knods,inpes-1)
          end do
        end if
        do inods=1,nnods                        ! Term eps -p(n,i) q
          xmuit=penal*shape(nnode+inods) 
          if(kfans.eq.1) then
            do jnods=1,nnods
              wmatr(nevab+inods,nevab+jnods)=
     .          wmatr(nevab+inods,nevab+jnods)-
     .          xmuit*shape(nnode+jnods)
            end do
          end if
          wrhsi(nevab+inods)=wrhsi(nevab+inods) ! Term eps -p(n,i-1) q or
     .      -xmuit*prepr                        ! eps -p(n-1) q
        end do
      end if
c
c*** Contribution of the viscous term  
c
      if(ievns.eq.1) then
        if(kfans.eq.1) then
          ieva1=0       
          do inode=1,nnode 
            do idime=1,ndime
              ieva1=ieva1+1
              jeva1=ieva1
              do jnode=inode+1,nnode
                jeva1=jeva1+ndime
                do kdime=1,ndime
                  addit=cartd(kdime,inode)*cartd(kdime,jnode)*visac
                  wmatr(ieva1,jeva1)=wmatr(ieva1,jeva1)+addit
                  wmatr(jeva1,ieva1)=wmatr(jeva1,ieva1)+addit
                end do
              end do
              do kdime=1,ndime
                wmatr(ieva1,ieva1)=wmatr(ieva1,ieva1)+
     .            cartd(kdime,inode)*cartd(kdime,inode)*visac
              end do
              if(fvins.gt.zeroc) then
                jeva2=ieva1-idime+ndime
                do jnode=inode+1,nnode
                  xmuit=cartd(idime,jnode)*bulkv
                  do jdime=1,ndime
                    jeva2=jeva2+1
                    addit=cartd(jdime,inode)*xmuit
                    wmatr(ieva1,jeva2)=wmatr(ieva1,jeva2)+addit
                    wmatr(jeva2,ieva1)=wmatr(jeva2,ieva1)+addit
                  end do
                end do
                jeva2=ieva1-idime
                xmuit=cartd(idime,inode)*visac
                do jdime=1,ndime
                  jeva2=jeva2+1
                  addit=cartd(jdime,inode)*xmuit
                  wmatr(ieva1,jeva2)=wmatr(ieva1,jeva2)+addit
                end do
              end if
            end do
          end do
          if (naxis.eq.1) then
            ievab=-1
            do inode=1,nnode
              ievab=ievab+2
              xmuit=2.0d0*bulkv*waxis(inode)
              jevab=-1
              do jnode=1,nnode
                jevab=jevab+2
                wmatr(ievab,jevab)=wmatr(ievab,jevab)+
     .            xmuit*waxis(jnode)  
              end do
            end do
          end if
        end if
      end if
      
      end
