      subroutine nvsvis(
     .  shape,cartd,elpro,turbu,elvel,elkin,elene,ellon,grave,
     .  nnode,ndime,nproe,nprod,kturb,ietns,hleng,visac,vista,
     .  ackin,aceps,olkin,oleps,tkcom,tecom,elvit,zeroc)
c**************************************************************************
c
c**** This routine computes the kinematic energy, the rate of energy 
c**** dissipation, the kinematic viscosity and the turbulent viscosity
c***  at the integration points.
c
c**************************************************************************
      implicit none
      integer   nnode,ndime,kdime,nproe,nprod,kturb,ietns,i,j,inode
      real*8    visac,vista,ackin,aceps,olkin,oleps,
     .          seci4,tkcom,tecom,xmile,zeroc,vecdot
      real*8    shape(nnode),               cartd(ndime,nnode),    
     .          elpro(nprod),               turbu(2),             
     .          elvel(ndime,nnode,1+ietns), elkin(nnode,1+ietns), 
     .          elene(nnode,1+ietns),       ellon(nnode),
     .          elvit(nnode),               grave(ndime,ndime),
     .          hleng(ndime)
c
c*** Physical kinematic viscosity (only used by the turbulence routines)
c      
      visac=elpro(2)/elpro(1)
c
c*** Now compute the turbulent viscosity, first for laminar flows
c
      if(kturb.eq.0) then
        vista=0.0d0
c
c*** Turbulent viscosity for mixing length models. The mixing length
c*** is TURBU(2), unless this value is zero, in which case the smallest
c*** element length is used. This corresponds to the Smagorinski model.
c*** Negative values of TURBU(2) correspond to models coded in TURFIX.
c        
      else if (kturb.eq.1) then
        if(turbu(2).gt.zeroc) then                          ! This could be 
          xmile=turbu(2)                                    ! omitted
        else if(turbu(2).gt.-zeroc) then                    ! This case is 
          xmile=hleng(1)                                    ! not in TURFIX
        else                                                ! This is the 
          xmile=vecdot(shape,ellon,nnode)                   ! general case
        end if                                              ! (but the slowest)
        do i=1,ndime
          do j=1,ndime
            grave(i,j)=0.0
            do inode=1,nnode
              grave(i,j)=grave(i,j)+cartd(i,inode)*
     .          elvel(j,inode,1)
            end do
          end do
        end do
        seci4 = 0.0
        do i = 1,ndime
          do j = 1,ndime
            seci4 = seci4+grave(i,j)*(grave(i,j)+grave(j,i))
          end do
        end do
        vista=turbu(1)*xmile*xmile
        vista=vista*sqrt(seci4)
c
c*** One or two equation turbulence models that need the calculation of
c*** the turbulent kinetic energy      
c
      else if (kturb.ge.2) then
        ackin=vecdot(shape,elkin,nnode) 
        ackin=max(tkcom,ackin)
        olkin=0.0d0
        if(ietns.eq.1) then
          olkin=vecdot(shape,elkin(1,2),nnode)
          olkin=max(tkcom,olkin)
        else
          olkin=0.0d0
        end if
        if (kturb.eq.3.or.kturb.eq.4.or.kturb.eq.5) then
          aceps=vecdot(elene,shape,nnode)
          aceps=max(tecom,aceps)
          oleps=0.0d0
          if(ietns.eq.1) then
            oleps=vecdot(elene(1,2),shape,nnode)
            oleps=max(tecom,oleps)
          else
            oleps=0.0d0
          end if
        else if(kturb.ge.6) then
          call runend('NVSVIS: DEFINE HOW TO COMPUTE DISSIPATION')
        end if
c
c*** Compute the turbulent kinematic viscosity
c
        vista=vecdot(shape,elvit,nnode)
      end if

      end
