      subroutine nvsvsg(shape,cartd,denac,visac,porac,hleng,
     .                  elvel,chale,chave,chavg,ndime,nnode,
     .                  nnodt,ldimo,iecns,sfnsc,sfnsp,sfnsr,
     .                  sfnss,sfnsd,sfdmo,frano,timom,tidiv,
     .                  timdm,thcon,penal,zeroc)
c***********************************************************************
c
c**** This routine computes the characteristic velocity and the 
c**** stability parameters for the Navier-Stokes equations
c
c***********************************************************************
      implicit none
      integer   ndime,nnode,nnodt,ldimo,
     .          idime,ievab,inode,jdime,iecns,ivepo
      real*8    sfnsc,sfnsp,sfnsr,sfnss,sfnsd,sfdmo,frano(4,2),
     .          timom,tidiv,timdm,tinsc,elno1,reynk,chale,
     .          denac,visac,porac,thcon,penal,zeroc,freto,
     .          freq1,freq2,freq3,freq4
      real*8    shape(nnode),   cartd(ndime,nnodt),
     .          hleng(ndime),   elvel(ndime*nnode,*),
     .          chave(ndime),   chavg(ndime,ndime)
c
c***  Decide whether to consider the current velocity or the previous one
c
      ivepo=1
      if(thcon.lt.zeroc) ivepo=2
c
c***  Characteristic velocity
c
      call veczer(ndime,chave)
      do idime=1,ndime
        ievab=idime-ndime
        do inode=1,nnode
          ievab=ievab+ndime
          chave(idime)=chave(idime)+shape(inode)*elvel(ievab,ivepo)
        end do
      end do
      call vecnor(chave,ndime,elno1,2)
c
c***  Compute the stability frequencies and stability parameters. Recall that
c***  sfnsp >= sfnsc when convection is also stabilized
c
      freq1 = sfnsp*4.0d0*visac/(chale*chale)               !Stokes term
      freq2 = sfnsc*2.0d0*denac*elno1/chale                 !Convective term
      freq3 = sfnsr*denac*frano(1,2)                        !Coriolis force
      freq4 = sfnss*abs(porac)                              !Porosity term

      timom = 0.0d0
      tidiv = 0.0d0
      freto = freq1+freq2+freq3
      if(freto.gt.zeroc) then
        timom = 1.0d0/(freto + freq4)
        freto = 1.0d0/(freto*chale*chale)
        tidiv = sfnsd/(freto + penal)
      end if
c
c***  Compute the stability time affecting the divergence of the momentum
c***  and the characteristic velocity gradient
c
      timdm=0.0d0
      if (ldimo.eq.1) then
        timdm=sfdmo*hleng(1)*hleng(1)/(frano(1,2)*denac)
        if (iecns.eq.1) then
          do idime=1,ndime
            do jdime=1,ndime
              chavg(idime,jdime)=0.0d0
              ievab=jdime
              do inode=1,nnode
                chavg(idime,jdime)=chavg(idime,jdime)+
     .            cartd(idime,inode)*elvel(ievab,ivepo)
                ievab=ievab+ndime
              end do
            end do
          end do
        end if
      end if
      
      end
