      subroutine recon1(shape,cartd,grave,vegau,elvel,rmom1,bmome,
     .                  waxis,ndime,nnode,nevab,naxis,jlins,fcons,
     .                  denac,gpcod,kfans,zeroc)
c***********************************************************************
c
c**** This routine computes the residual of the Navier-Stokes equations
c**** due to the convective term 
c
c***********************************************************************
      implicit none
      integer  ndime,nnode,nevab,naxis,jlins,kfans,
     .         idime,jdime,ievab,jevab,inode
      real*8   fcons,denac,gpcod,dilas,zeroc,addit,addi1,vecdot
      real*8   shape(nnode),               cartd(ndime,nnode),
     .         grave(ndime,ndime),         vegau(ndime,*),
     .         elvel(nevab,*),             rmom1(ndime,nevab),         
     .         bmome(ndime),               waxis(nnode*naxis)
c
c*** Divergence, velocity and velocity gradient ( G_ij = d_i u_j) 
c
      dilas=0.0d0
      do idime=1,ndime
        do jdime=1,ndime  
          grave(idime,jdime)=0.0d0
          jevab=jdime-ndime
          do inode=1,nnode
            jevab=jevab+ndime
            grave(idime,jdime)=grave(idime,jdime)+
     .        cartd(idime,inode)*elvel(jevab,1)
          end do
        end do
        dilas=dilas+grave(idime,idime)
        vegau(idime,1)=0.0d0
        ievab=idime-ndime
        do inode=1,nnode
          ievab=ievab+ndime
          vegau(idime,1)=vegau(idime,1)+shape(inode)*elvel(ievab,1)
        end do
      end do
      if (naxis.eq.1) then             
        if (abs(gpcod).gt.zeroc) then 
          dilas=dilas+vegau(1,1)/gpcod
        else
          dilas=dilas+grave(1,1)
        end if
      end if
c
c*** Contribution of the convective term to the LHS
c
      if(kfans.eq.1) then
        if(jlins.ge.1) then                                 ! Picard
          ievab=0
          do inode=1,nnode
            addit=denac*(vecdot(vegau(1,1),cartd(1,inode),ndime)
     .        +fcons*dilas*shape(inode))
            do idime=1,ndime
              ievab=ievab+1
              rmom1(idime,ievab)=rmom1(idime,ievab)+addit
            end do
          end do
          if (jlins.eq.2) then                              ! Newton Raphson
            ievab=0
            do inode=1,nnode
              do idime=1,ndime
                ievab=ievab+1
                do jdime=1,ndime
                  rmom1(jdime,ievab)=rmom1(jdime,ievab)+denac*(
     .              grave(idime,jdime)*shape(inode)+
     .              fcons*vegau(jdime,1)*cartd(idime,inode))
                end do
              end do
            end do
            if ((naxis.eq.1).and.(fcons.gt.zeroc)) then
              ievab=-1
              do inode=1,nnode
                ievab=ievab+2
                do jdime=1,ndime
                  rmom1(jdime,ievab)=rmom1(jdime,ievab)+denac*
     .              vegau(jdime,1)*waxis(inode)
                end do
              end do
            end if
          end if
        end if
      end if
c
c*** Contribution of the convective term to the RHS
c
      if(jlins.eq.0) then
        do idime=1,ndime
          bmome(idime)=bmome(idime)-denac*(
     .      vecdot(vegau(1,1),grave(1,idime),ndime)
     .      +fcons*dilas*vegau(idime,1))
        end do
      else if(jlins.eq.2) then
        do idime=1,ndime
          bmome(idime)=bmome(idime)+denac*(
     .      vecdot(vegau(1,1),grave(1,idime),ndime)
     .      +fcons*dilas*vegau(idime,1))
        end do
      end if

      end
