      subroutine vislaw(visce,kvila,visco,visca,tvelo,grave,vpres,
     .                  vtemp,vfree,ndime,zeroc)
c***********************************************************************
c
c**** This routine computes the viscosity as a dependent variable
c
c***********************************************************************
      implicit none
      integer kvila,ndime,i,j
      real*8  visce,visca,vpres,vtemp,vfree,zeroc,
     .        totem,seci4,secib,zetap,sigma
      real*8  visco(10)
      real*8  tvelo(ndime),    grave(ndime,ndime)
c
c*** Viscosity in the "air"-filled region
c
      if(vfree.lt.1.0e-2) then
        visce=visca
        return
      end if
c
c*** Constant viscosity
c
      if(kvila.eq.0) then
        visce=visco(1)
c
c*** Power law: mu = mu0 [4 I2]^((n-1)/2) exp (b/T), where:
c*** VISCO = [ mu0 , n , b , cut-off ]
c
      else if(kvila.eq.1) then 
        seci4 = 0.0
        do i = 1,ndime
          do j = 1,ndime
            seci4 = seci4+grave(i,j)*(grave(i,j)+grave(j,i))
          end do
        end do
        visce=visco(4)
        if(seci4.gt.zeroc) then
          totem=0.0d0
          if(visco(3).gt.zeroc) totem=visco(3)/(273.0d0+vtemp)
          visce=visco(1)*(seci4**(visco(2)*0.5d0-0.5d0))*exp(totem)
        end if
c
c*** Carreau model: mu = mu0 [1+ 4 l^2 I2]^((n-1)/2) exp (b/T), where:
c*** VISCO = [ mu0 , l , n , b , cut-off ]
c
      else if(kvila.eq.2) then 
        seci4 = 0.0d0
        do i = 1,ndime
          do j = 1,ndime
            seci4 = seci4+grave(i,j)*(grave(i,j)+grave(j,i))
          end do
        end do
        visce=visco(5)
        if(seci4.gt.zeroc) then
          totem=0.0d0
          if(visco(4).gt.zeroc) totem=visco(4)/(273.0d0+vtemp)
          visce=visco(1)*(1.0d0+visco(2)*visco(2)*
     .         seci4**(visco(3)*0.5d0-0.5d0))*exp(totem)
        end if
c
c*** Perzyna's model with a Sellars-Tegart yield stress variation:
c*** mu = {sigma + [g1*sqrt(4 I2/3)]^(1/m)}/sqrt(3.0 * 4 I2), with
c*** sigma = 1/p1 * log { (Z/p2)^(p3) + sqrt [(Z/p2)^(2 p3) + 1]} and
c*** Z = sqrt(4 I2 / 3) * exp(Q/RT), where
c*** VISCO = [ p1 , p2 , p3 , Q , R , g1 , m , cut-off ]
c
      else if(kvila.eq.3) then
        seci4 = 0.0d0
        do i = 1,ndime
          do j = 1,ndime
            seci4 = seci4 +grave(i,j)*(grave(i,j)+grave(j,i))
          end do
        end do
        secib = sqrt(seci4/3.0d0)
        if(secib.gt.zeroc) then
          visce=visco(8)
        else
          totem=0.0d0
          if(visco(4).gt.zeroc) totem=visco(4)/(273.0d0+vtemp)
          zetap = secib*exp(totem/visco(5))
          sigma = 1.0d0/visco(1)*log((zetap/visco(2))**visco(3)+
     .            sqrt((zetap/visco(2))**(2.0d0*visco(3))+1.0d0))
          visce = (sigma + (visco(6)*secib)**(1.0d0/visco(7)))
     .            /(3.0d0*secib)
          if(visce.gt.visco(8)) visce=visco(8)
        end if
      end if
c
c*** Average viscosity
c
      visce=vfree*visce+(1.0-vfree)*visca

      end
