      subroutine elmgat(lnods,coord,veloc,presu,tempe,fresu,tkine,
     .                  tener,cletu,visct,elcod,elvel,elpre,eltem,
     .                  elfrs,elkin,elene,ellon,elvit,itask) 
c*************************************************************************
c
c**** This routine performs the gather operations previous to call the
c**** element library. The gather is done according to the tasks and their
c**** needs:
c
c     itask =  1   -->  Compute element properties:
c                       u(n),p(n),T(n),f(n)
c     itask =  2   -->  Compute the element matrix & RHS for the 
c                       Navier-Stokes equations
c                       u(n),u(n-1),p(n),p(n-1),T(n),T(n-1),L
c     itask =  3   -->  Updates the pressure
c                       u(n),p(k)
c     itask =  4   -->  Compute the element matrix & RHS for the 
c                       heat equation
c                       u(n),u(n-1),T(n),T(n-1)
c     itask =  5   -->  Compute the element matrix & RHS for the 
c                       free-surface tracking (FST) problem
c                       u(n),u(n-1),f(n),f(n-1)
c     itask =  6   -->  Compute the element matrix & RHS for the
c                       variables to be smoothed
c                       u(n),p(n),x
c     itask =  7   -->  Compute the element matrix & RHS for the 
c                       turbulent kinetic energy
c                       u(n),k(n),k(n-1)
c     itask =  8   -->  Compute the element matrix & RHS for the 
c                       rate of turbulent energy dissipation
c                       u(n),k(n),e(n),e(n-1)
c     itask =  9   -->  Same as ITASK = 2 but from the boundary 
c                       contributions
c     itask = 10   -->  Compute the element matrix & RHS for the 
c                       rate of turbulent energy vorticity
c                       u(n),k(n),w(n),w(n-1)
c     itask = 11   -->  Compute the element matrix & RHS for the 
c                       rate of turbulent integral time scale (tau)
c                       u(n),k(n),tau(n),tau(n-1)
c     itask = 12   -->  Gather of ELCOD to put atmosferic stress on
c                       required boundaries
c     itask = 13   -->  Calculation of errors      
c
c*************************************************************************
      implicit none
      include 'com_prob.f'
      include 'com_auxi.f'
      integer   itask,iposi
      integer   lnods(nnodt) 
      real*8
     .  coord(ndime,npoin), veloc(ntotv,ncove),
     .  presu(ntotp,ncopr), tempe(npoin,ncote),
     .  fresu(npoin,ncofs), tkine(npoin,ncove),
     .  tener(npoin,ncove), cletu(npoin), visct(npoin)
      real*8
     .  elcod(ndime,nnode), elvel(nevab,ncove-iepns),
     .  elpre(nnods,ncopr-iepns), eltem(nnode,ncote-iephe),
     .  elfrs(nnode,ncofs-iepfs), elkin(nnode,ncove-iepns),
     .  elene(nnode,ncove-iepns), ellon(nnode),
     .  elvit(nnode)
c
c*** Gather of coordinates
c      
      if(itask.eq.6 .or. itask.eq.12 .or.
     .  (itask.eq.2 .and. ip1p1.eq.1) ) then
        call gatvec(lnods,coord,elcod,ndime,nnode,npoin)
      end if
c
c*** Gather of velocities
c
      if(iepns.eq.1) then
        call gatvec(lnods,veloc(1,1),elvel(1,1),ndime,nnode,npoin)
        if(ietns.eq.1) then
          if((itask.eq. 2).or.
     .      ( itask.eq. 4).or.
     .      ( itask.eq. 5).or.
     .      ( itask.eq.13)    ) 
     .      call gatvec(lnods,veloc(1,3),elvel(1,2),ndime,nnode,npoin)
          if((itask.eq.2).and.(twost.ne.0))
     .      call gatvec(lnods,veloc(1,4),elvel(1,3),ndime,nnode,npoin)
        end if
      else if(iupns.eq.1) then
        call gatvec(lnods,veloc(1,1),elvel(1,1),ndime,nnode,npoin)
      end if
c
c*** Gather of pressures
c
      if((itask.eq. 1).or.
     .  ( itask.eq. 2).or.
     .  ( itask.eq. 3).or.
     .  ( itask.eq. 6).or.
     .  ( itask.eq. 9).or.
     .  ( itask.eq.13)    ) then
        if(iepns.eq.1) then
          iposi=1
          if((itask.eq.3).and.(inpes.ge.2)) iposi=inpes
          call rlnods(lnods,nnodt,nnode, 1,-npoin)
          call gatvec(lnods(nnode+1),presu(1,iposi),elpre(1,1),1,
     .      nnods,ntotp)
          if((itask.eq.2)) then
            if(ietns.eq.1) 
     .        call gatvec(lnods(nnode+1),presu(1,3),elpre(1,2),1,
     .        nnods,ntotp)
          end if
          call rlnods(lnods,nnodt,nnode, 1,npoin)
        end if
      end if
c
c*** Gather of temperatures
c
      if((itask.eq.1                    ).or.
     .  ( itask.eq.2.and.bouex.gt.1.0e-4).or.
     .  ( itask.eq.4                    )    ) then
        if(iephe.eq.1) then
          call gatvec(lnods,tempe(1,1),eltem(1,1),1,nnode,npoin)
          if((itask.eq.2.and.bouex.gt.1.0e-4).or.
     .      ( itask.eq.4)                   ) then
            if(iethe.eq.1)
     .        call gatvec(lnods,tempe(1,3),eltem(1,2),1,nnode,npoin)
          end if
        else if(iuphe.eq.1) then 
          call gatvec(lnods,tempe(1,1),eltem(1,1),1,nnode,npoin)
        end if
      end if
c
c*** Gather of free-surface values
c      
      if((itask.eq.1).or.
     .  ( itask.eq.2).or.
     .  ( itask.eq.5).or.
     .  ( itask.eq.9)    ) then
        if(iepfs.eq.1) then
          call gatvec(lnods,fresu(1,1),elfrs(1,1),1,nnode,npoin)
          if(itask.eq.5) then
            if(ietfs.eq.1) 
     .        call gatvec(lnods,fresu(1,3),elfrs(1,2),1,nnode,npoin)
          end if
        else if(iupfs.eq.1) then
          call gatvec(lnods,fresu(1,1),elfrs(1,1),1,nnode,npoin)
        end if
      end if
c
c*** Gather of mixing length
c
      if((itask.eq.2.and.kturb.eq.1)) then
        call gatvec(lnods,cletu,ellon,1,nnode,npoin)
      end if
c
c*** Gather of turbulent viscosity
c
      if((itask.eq. 2.and.kturb.ge.2).or.
     .  ( itask.eq. 7)               .or.
     .  ( itask.eq. 8)               .or.
     .  ( itask.eq.10)               .or.
     .  ( itask.eq.11)                   ) then
        call gatvec(lnods,visct,elvit,1,nnode,npoin)
      end if
c
c*** Gather of turbulent kinetic energy
c
      if((itask.eq. 2.and.kturb.ge.2).or.
     .  ( itask.eq. 7)               .or.
     .  ( itask.eq. 8)               .or.
     .  ( itask.eq.10)               .or.
     .  ( itask.eq.11)                   ) then
        call gatvec(lnods,tkine(1,1),elkin(1,1),1,nnode,npoin)
        if(ietns.eq.1) then
          if((itask.eq.2.and.kturb.ge.2).or.
     .      ( itask.eq.7)                   ) then
            call gatvec(lnods,tkine(1,3),elkin(1,2),1,nnode,npoin)
          end if
        end if
      end if
c
c*** Gather of turbulent rate of energy dissipation
c      
      if((itask.eq.2.and.kturb.ge.3).or.
     .  ( itask.eq.8)              ) then
        call gatvec(lnods,tener(1,1),elene(1,1),1,nnode,npoin)
        if(ietns.eq.1) then
          if((itask.eq.2.and.kturb.ge.3).or.
     .      ( itask.eq.8)                   ) then
            call gatvec(lnods,tener(1,3),elene(1,2),1,nnode,npoin)
          end if
        end if
      end if
c
c*** Gather of turbulent rate of energy vorticity
c
      if((itask.eq.2.and.kturb.ge.3).or.
     .  ( itask.eq.10)              ) then
        call gatvec(lnods,tener(1,1),elene(1,1),1,nnode,npoin)
        if(ietns.eq.1) then
          if((itask.eq.2.and.kturb.ge.3).or.
     .      ( itask.eq.10)                   ) then
            call gatvec(lnods,tener(1,3),elene(1,2),1,nnode,npoin)
          end if
        end if
      end if
c
c*** Gather of turbulent rate of integral time scale (tau)
c
      if((itask.eq.2.and.kturb.ge.3).or.
     .  ( itask.eq.11)              ) then
        call gatvec(lnods,tener(1,1),elene(1,1),1,nnode,npoin)
        if(ietns.eq.1) then
          if((itask.eq.2.and.kturb.ge.3).or.
     .      ( itask.eq.11)                   ) then
            call gatvec(lnods,tener(1,3),elene(1,2),1,nnode,npoin)
          end if
        end if
      end if

      end
