      subroutine elmlib(objec,ileob,iffix,lnods,shape,shaga,
     .                  elpro,elcod,elvel,elpre,eltem,elfrs,
     .                  elmat,elrhs,elkin,elene,ellon,elvit,
     .                  work1,itask)
c************************************************************************
c
c**** This routine performs computations at the element level according
c**** to the following tasks:
c
c     itask =  1   -->  Compute element properties
c     itask =  2   -->  Compute the element matrix & RHS for the 
c                       Navier-Stokes equations
c     itask =  3   -->  Update the pressure
c     itask =  4   -->  Compute the element matrix & RHS for the 
c                       heat equation
c     itask =  5   -->  Compute the element matrix & RHS for the 
c                       free-surface tracking (FST) problem
c     itask =  6   -->  Compute the element matrix & RHS for the
c                       variables to be smoothed
c     itask =  7   -->  Compute the element matrix & RHS for the 
c                       turbulent kinetic energy
c     itask =  8   -->  Compute the element matrix & RHS for the 
c                       rate of turbulent energy dissipation
c     itask =  9   -->  Same as itask = 2 for boundary contributions
c                       (See routines BOUACC and BOULIB)
c     itask = 10   -->  Compute the element matrix & RHS for the 
c                       rate of turbulent energy vorticity 
c     itask = 11   -->  Compute the element matrix & RHS for the 
c                       turbulent integral time scale (tau)
c     itask = 12   -->  Task for BOULIB (prescription of atmosferic
c                       stress)
c     itask = 13   -->  Calculation of errors with respect to an
c                       exact solution      
c
c**** The object (now elements) are organized as follows:
c
c     objec(ileob(fcart)) : Pointer to Cartesian derivatives      CARTD
c     objec(ileob(fdvol)) : Pointer to volume element             DVOLU
c     objec(ileob(fgpco)) : Pointer to integration points coords. GPCOD
c     objec(ileob(fhess)) : Pointer to Hessian matrix             HESSI
c     objec(ileob(ftrag)) : Pointer to transformation matrix      TRAGL
c     objec(ileob(fhlen)) : Pointer to element lengths            HLENG
c
c************************************************************************
      implicit none
      include 'com_prob.f'
      include 'com_auxi.f'
      integer   itask
      real*8    dummy
      integer   ileob(*)
      integer   iffix(*), lnods(*)
      real*8
     .  objec(*), shape(*), shaga(*), elpro(*), elcod(*),
     .  elvel(*), elpre(*), eltem(*), elfrs(*), elmat(*),
     .  elrhs(*), elkin(*), elene(*), ellon(*), elvit(*),
     .  work1(*)
c
c*** Direct to appropriate processor
c
      go to ( 10 , 20 , 30 , 40 , 50 , 60 , 70 , 80 , 90 , 100,
     .       110 ,120 ,130 ) , itask
c
c*** Compute element properties
c
   10 call proelm(shape,elpro,elvel,elpre,eltem,elfrs,
     .            objec(ileob(fcart)),objec(ileob(fgpco)),
     .            objec(ileob(fhlen)),work1(iproe(    1)),
     .            work1(iproe( 2)))
      return
c
c*** Compute element matrix and RHS for the Navier-Stokes equations
c
   20 call nvselm(shape,elpro,elmat,elrhs,elvel,elpre,eltem,
     .            elfrs,elkin,elene,ellon,elvit,
     .            objec(ileob(fcart)),objec(ileob(fdvol)),
     .            objec(ileob(fgpco)),objec(ileob(fhess)),
     .            objec(ileob(ftrag)),objec(ileob(fhlen)),
     .            work1(invse(    1)),work1(invse(    2)),
     .            work1(invse(    3)),work1(invse(    4)),
     .            work1(invse(    5)),work1(invse(    6)),
     .            work1(invse(    7)),work1(invse(    8)),
     .            work1(invse(    9)),work1(invse(   10)),
     .            work1(invse(   11)),work1(invse(   12)),
     .            work1(invse(   13)),work1(invse(   14)),
     .            work1(invse(   15)),work1(invse(   16)),
     .            work1(invse(   17)),work1(invse(   18)),
     .            work1(invse(   19)),work1(invse(   20)),
     .            work1(invse(   21)),work1(invse(   22)),
     .            work1(invse(   23)),work1(invse(   24)),
     .            %val(produ_p))
      call chefil(iffix,lnods,shape,elfrs,elvel,
     .            objec(ileob(fcart)),objec(ileob(fdvol)))
      call nvsmod(%val(lpoty_p),%val(lroty_p),%val(presv_p),
     .            %val(exnor_p),%val(skcos_p),
     .            iffix,lnods,elmat,elrhs,elfrs,elvel,work1,0)
      return
c
c*** Updates the pressure (when it has been eliminated at the element level)
c
   30 call preelm(shape,elvel,elpre,
     .            objec(ileob(fcart)),objec(ileob(fdvol)),
     .            objec(ileob(fgpco)),
     .            work1(  ipree(    1)),work1(  ipree(    2)),
     .            work1(  ipree(    3)),work1(  ipree(    4)))
      return
c
c*** Compute element matrix and RHS for the heat equation
c
   40 call temelm(shape,elpro,elmat,elrhs,elvel,eltem,
     .            objec(ileob(fcart)),objec(ileob(fdvol)),
     .            objec(ileob(fgpco)),objec(ileob(fhess)),
     .            objec(ileob(ftrag)),objec(ileob(fhlen)),
     .            work1(iteme(    1)),work1(iteme(    2)),
     .            work1(iteme(    3)),work1(iteme(    4)),
     .            work1(iteme(    5)),work1(iteme(    6)),
     .            work1(iteme(    7)),work1(iteme(    8)),
     .            work1(iteme(    9)),work1(iteme(   10)))
      return
c
c*** Compute element matrix and RHS for the FST equation
c
   50 call pscelm(shape,elmat,elrhs,elvel,elfrs,
     .            %val(tanve_p),                               ! Thinflow
     .            objec(ileob(fcart)),objec(ileob(fdvol)),
     .            objec(ileob(ftrag)),objec(ileob(fhlen)),
     .            work1(ipsce(    1)),work1(ipsce(    2)),
     .            work1(ipsce(    3)),work1(ipsce(    4)))
      call pschol(%val(lpoty_p),%val(lroty_p),%val(exnor_p),
     .            iffix,lnods,elfrs,elrhs,elvel,elmat)
      return
c
c*** Compute smoothed variables for postprocess
c
   60 call smoelm(shape,elpro,elmat,elcod,elvel,elpre,elrhs,
     .            objec(ileob(fcart)),objec(ileob(fdvol)),
     .            work1(ismoe(    3)),work1(ismoe(    4)),
     .            work1(ismoe(    5)),work1(ismoe(    6)),
     .            work1(ismoe(    7)),work1(ismoe(    8)),
     .            work1(ismoe(    9)),work1(ismoe(   10)))
      return
c
c*** Compute the turbulent kinetic energy (K)
c
   70 call kinelm(shape,elpro,elmat,elrhs,elvel,elkin,elvit,
     .            objec(ileob(fcart)),objec(ileob(fdvol)),
     .            objec(ileob(fgpco)),objec(ileob(fhess)),
     .            objec(ileob(ftrag)),objec(ileob(fhlen)),
     .            work1(ikine(    1)),work1(ikine(    2)),
     .            work1(ikine(    3)),work1(ikine(    4)),
     .            work1(ikine(    5)),work1(ikine(    6)),
     .            %val(produ_p))
      call turmod(iffix,lnods,elmat,elrhs,elkin,dummy,1,1,
     .            %val(lpoty_p),%val(lroty_p))
      return
c
c*** Compute the turbulent energy dissipation (E), alone or together with K
c
   80 if(tucou.eq.0) then                                   ! Full coupling
        call kepelm(shape,elpro,elmat,elrhs,elvel,elkin,elene,
     .              elvit,
     .              objec(ileob(fcart)),objec(ileob(fdvol)),
     .              objec(ileob(fgpco)),objec(ileob(fhess)),
     .              objec(ileob(ftrag)),objec(ileob(fhlen)),
     .              work1(ikine(    1)),work1(ikine(    2)),
     .              work1(ikine(    3)),work1(ikine(    4)),
     .              work1(ikine(    5)),work1(ikine(    6)),
     .              %val(produ_p))
        call turmod(iffix,lnods,elmat,elrhs,elkin,elene,2,2,
     .              %val(lpoty_p),%val(lroty_p))
      else if(tucou.eq.1) then                              ! Iterat. coupling
        call epselm(shape,elpro,elmat,elrhs,elvel,elkin,elene,
     .              elvit,
     .              objec(ileob(fcart)),objec(ileob(fdvol)),
     .              objec(ileob(fgpco)),objec(ileob(fhess)),
     .              objec(ileob(ftrag)),objec(ileob(fhlen)),
     .              work1(ikine(    1)),work1(ikine(    2)),
     .              work1(ikine(    3)),work1(ikine(    4)),
     .              work1(ikine(    5)),work1(ikine(    6)),
     .              %val(produ_p))
        call turmod(iffix,lnods,elmat,elrhs,elkin,elene,2,1,
     .              %val(lpoty_p),%val(lroty_p))
      end if
      return
c
c*** Task used in BOULIB (Boundary contribution for Navier Stokes)
c
   90 continue
      return
c
c*** Compute the turbulent energy vorticity (W)
c
  100 call worelm(shape,elpro,elmat,elrhs,elvel,elkin,elene,
     .            elvit,
     .            objec(ileob(fcart)),objec(ileob(fdvol)),
     .            objec(ileob(fgpco)),objec(ileob(fhess)),
     .            objec(ileob(ftrag)),objec(ileob(fhlen)),
     .            work1(ikine(    1)),work1(ikine(    2)),
     .            work1(ikine(    3)),work1(ikine(    4)),
     .            work1(ikine(    5)),work1(ikine(    6)),
     .            %val(produ_p))
      call turmod(iffix,lnods,elmat,elrhs,elkin,elene,2,1,
     .            %val(lpoty_p),%val(lroty_p))
      return
c
c*** Compute the turbulent integral time scale (tau)
c
  110 call tauelm(shape,elpro,elmat,elrhs,elvel,elkin,elene,
     .            elvit,
     .            objec(ileob(fcart)),objec(ileob(fdvol)),
     .            objec(ileob(fgpco)),objec(ileob(fhess)),
     .            objec(ileob(ftrag)),objec(ileob(fhlen)),
     .            work1(ikine(    1)),work1(ikine(    2)),
     .            work1(ikine(    3)),work1(ikine(    4)),
     .            work1(ikine(    5)),work1(ikine(    6)),
     .            work1(ikine(    7)),%val(produ_p))
      call turmod(iffix,lnods,elmat,elrhs,elkin,elene,2,1,
     .            %val(lpoty_p),%val(lroty_p))
      return      
c
c*** Task used in BOULIB (Prescription of atmosferic stress)
c
  120 continue
      return
c
c*** Calculation of errors with respect to an exact solution
c
  130 call errelm(shape,elvel,elpre,
     .            objec(ileob(fcart)),objec(ileob(fdvol)),
     .            objec(ileob(fgpco)),
     .            work1(invse(    1)),work1(invse(    2)),
     .            work1(invse(    3)),work1(invse(    4)),
     .            work1(invse(    5)),work1(invse(    6)))
      return

      end
