      subroutine nvsmod(lpoty,lroty,presv,exnor,skcos,iffix,lnods,
     .                  elmat,elrhs,elfrs,elvel,worma,iclas)
c************************************************************************
c   
c**** This routine modifies the element stiffness matrix of the Navier-
c**** Stokes equations to impose the correct boundary conditions for
c**** temporary free nodes and for local boundary conditions (in skew-
c**** systems). In this case the rotation of the nodal matrices if this
c**** system is the tangent is understood to be  
c
c      { NORMAL , TANGENT 1 , TANGENT 2 }
c
c**** The tangent vectors have been computed according to a good conditioning
c**** of the calculations. Therefore, only the possibility of prescribing
c**** the normal component must be considered when this option is used.
c**** Also, if the flow is confined a pressure value is prescribed.
c
c************************************************************************
      implicit none
      include 'com_prob.f'
      include 'com_auxi.f'
      integer
     .  inode,idime,itott,ievav,kevav,ipoin,ibopo,iroty,
     .  ifaco,itotp,inods,ievap,ieva0,iclas,ifact,jdime,
     .  jnode,jevav
      integer   iffix(nvfix),             lnods(nnodt),
     .          lpoty(npoin),             lroty(nbopo)
      real*8    elmat(nevat,nevat),       elrhs(nevat),
     .          elfrs(nnode),             elvel(nevab),
     .          exnor(ndime,ndime,nbopo), skcos(ndime,ndime,nfaco),
     .          worma(ndime,ndime,2),     presv(ntotv)
c
c*** Initializations
c
      ifact=0
      if((kfans.eq.1).or.(kpeel.eq.1)) ifact=1
c
c*** Prescribe one pressure degree of freedom if the flow is confined
c
      if((iconf.eq.1).and.(inpes.eq.0)) then
        do inods=1,nnods
          itotp=ntotv-npoin+lnods(nnode+inods)
          if (itotp.eq.ntotv+1) then
            ievap=nevab+inods
            if(ifact.eq.1) then
              do kevav=1,nevat
                elmat(kevav,ievap)=0.0
                elmat(ievap,kevav)=0.0
              end do
              elmat(ievap,ievap)=1.0
            end if
            elrhs(ievap)=0.0
          end if
        end do
      end if
c
c*** Rotate the nodal matrices, the element nodal velocities and RHS to 
c*** prescribe boundary conditions in a skew-system, either the tangent
c*** one or another prescribed by the user. Also, periodical boundary 
c*** conditions are accounted for. In the case in which the B.C.'s are
c*** NOT prescribed by MatMan, ELVEL must be rotated as indicated in the
c*** lines commented out with C-S.
c
      if((kerot.eq.1).or.(keima.gt.0)) then      
        do inode=1,nnode
          ipoin=lnods(inode)
          ibopo=lpoty(ipoin)
          if(ibopo.gt.0) then
            iroty=lroty(ibopo)
            if(iroty.eq.1) then                             !Tangent system
              ieva0=(inode-1)*ndime+1
              call mbvab2(worma,exnor(1,1,ibopo),elvel(ieva0),  
     .          ndime,ndime)
              call vecasi(ndime,worma,elvel(ieva0))
              itott=(ipoin-1)*ndime
              ieva0=ieva0-1
              do idime=1,ndime
                if (iffix(itott+idime).eq.1)
     .            elvel(ieva0+idime)=presv(itott+idime)
              end do
              call rotnod(inode,nnode,ndime,nevat,elmat,elrhs,
     .                    exnor(1,1,ibopo),worma,kpeel,ifact)
            else if(iroty.gt.1) then                        !Given system
              ifaco=iroty-1
              ieva0=(inode-1)*ndime+1
              call mbvab2(worma,skcos(1,1,ifaco),elvel(ieva0),
     .          ndime,ndime)
              call vecasi(ndime,worma,elvel(ieva0))
              itott=(ipoin-1)*ndime
              ieva0=ieva0-1
              do idime=1,ndime
                if (iffix(itott+idime).eq.1)
     .            elvel(ieva0+idime)=presv(itott+idime)
              end do
              call rotnod(inode,nnode,ndime,nevat,elmat,elrhs,
     .                    skcos(1,1,ifaco),worma,kpeel,ifact)
            else if(iroty.lt.0) then                        !IPOIN is a slave
              ifaco=lroty(lpoty(-iroty))-1
              if (ifaco.lt.1) then
                call rotnod(inode,nnode,ndime,nevat,elmat,  !Rotate the 
     .                      elrhs,exnor(1,1,ibopo),         !stiffness matrix
     .                      worma(1,1,2),kpeel,ifact)               
              else
                call mbmab0(worma,exnor(1,1,ibopo),         ! Master node has
     .            skcos(1,1,ifaco),ndime,ndime,ndime)       ! a given system
                ieva0=(inode-1)*ndime+1
                call mbvab2(worma(1,1,2),worma,elvel(ieva0),
     .            ndime,ndime)
                call vecasi(ndime,worma(1,1,2),elvel(ieva0))
                itott=(-iroty-1)*ndime
                ieva0=ieva0-1
                do idime=1,ndime
                  if (iffix(itott+idime).eq.1)
     .              elvel(ieva0+idime)=presv(itott+idime)
                end do
                call rotnod(inode,nnode,ndime,nevat,elmat,  !Rotate the 
     .                      elrhs,worma,worma(1,1,2),kpeel, !stiffness matrix
     .                      ifact)
              end if
            end if
          end if
        end do
      end if
c
c*** Prescribe the velocity degrees of freedom of temporary holes after
c*** rotations (that is, the prescription is given in the tangent system)
c      
      if(iepfs.eq.1) then
        do inode=1,nnode
          if(elfrs(inode).ge.cripc) then
            ipoin=lnods(inode)
            ievav=(inode-1)*ndime
            itott=(ipoin-1)*ndime
            do idime=1,ndime
              ievav=ievav+1
              itott=itott+1
              if(iffix(itott).eq.2) then
                if(ifact.eq.1) then
                  do kevav=1,nevat
                    elmat(kevav,ievav)=0.0d0
                    elmat(ievav,kevav)=0.0d0
                  end do
                  elmat(ievav,ievav)=1.0d0
                end if
                elrhs(ievav)=0.0d0
                elvel(ievav)=presv(itott)
              end if
            end do
          end if
        end do
      end if
c
c*** Eliminate bubble d.o.f at elemental level if it is required, and save
c*** the matrices to recover them (recall that matrix factorization is needed
c*** in this case, that is,  KFANS = 1).
c
      if (nscon)
     .  call conde0(ndime,nnode,nnods*(1-kpeel),nevat,nelem,
     .              ielem,lmatb_p,lenpo,worma,worma(1,1,2),
     .              elmat,elrhs,iclas)
      
      end
