      subroutine preelm(shape,elvel,elpre,cartd,dvolu,gpcod,
     .                  premm,premi,gtuve,princ)
c*************************************************************************
c
c**** This routine computes some norms of the divergence of the velocity
c**** field for each element for future use in the routine CONVER. The 
c**** element contributions are added to those stored in COM_AUXI
c
c**** This routine also computes the pressure from the velocity when a 
c**** penalty method has been employed and the pressure is discontinuous,
c**** so that it has been eliminated at the element level. The basic
c**** equation is
c
c     eps M P(i) + G^T U(i) = eps M P(k),   k = i-1, n-1 or 0
c
c**** Residual terms that might have been added to the incompressibility
c**** equation are not taken into account. The value of the P(k) that
c**** reaches this routine is already the correct one.
c
c*************************************************************************
      implicit real*8(a-h,o-z)
      include 'com_prob.f'
      include 'com_auxi.f'
      real*8  shape(nnodt,ngaus),   cartd(ndime,nnodt,ngaus),
     .        dvolu(ngaus),         gpcod(ndime,ngaus*ngpco)
      real*8  elvel(nevab),         elpre(nnods)
      real*8  premm(nnods,nnods),   premi(nnods,nnods),
     .        gtuve(nnods),         princ(nnods)

      call veczer(nnods*nnods,premm)         ! Initializes some arrays
      call veczer(nnods      ,gtuve)
c
c*** Main integration loop
c
      do igaus=1,ngaus
c
c*** Computes the divergence of the velocity
c
        diveu=0.0
        do idime=1,ndime
          ievab=idime-ndime
          do inode=1,nnode
            ievab=ievab+ndime
            diveu=diveu+cartd(idime,inode,igaus)*elvel(ievab)
          end do
        end do
        if(naxis.eq.1) then
          if(abs(gpcod(1,igaus)).gt.1.0e-8) then
            do inode=1,nnode
              ieva1=ndime*(inode-1)+1
              diveu=diveu+shape(inode,igaus)
     .          *elvel(ieva1)/gpcod(1,igaus)
            end do
          else
            do inode=1,nnode
              ieva1=ndime*(inode-1)+1
              diveu=diveu+cartd(1,inode,igaus)*elvel(ieva1)
            end do
          end if
        end if
c
c*** Computes the vector G^T U and the pressure mass matrix, if needed
c
        do inods=1,nnods
          gtuve(inods)=gtuve(inods)
     .      +shape(nnode+inods,igaus)*diveu*dvolu(igaus)
          if((kpeel.eq.1).or.(inpco.eq.0)) then
            do jnods=1,nnods
              premm(inods,jnods)=premm(inods,jnods)+
     .          shape(nnode+inods,igaus)*shape(nnode+jnods,igaus)
     .          *dvolu(igaus)
            end do
          end if
        end do
c
c*** Computes some norms of div(u)
c
        gtuln(1,1)=gtuln(1,1)+abs(diveu)*dvolu(igaus)       ! L1 norm
        gtuln(2,1)=gtuln(2,1)+diveu*diveu*dvolu(igaus)      ! L2 norm
        if(gtuln(3,1).lt.abs(diveu)) gtuln(3,1)=abs(diveu)  ! L-inf norm
      end do       ! igaus=1,ngaus

      if (inpco.eq.0) then
        do inods=1,nnods
          gtuln(1,2)=gtuln(1,2)+abs(gtuve(inods))
          gtuln(2,2)=gtuln(2,2)+gtuve(inods)*gtuve(inods)
          if(gtuln(3,2).lt.abs(gtuve(inods)))
     .      gtuln(3,2)=abs(gtuve(inods))
        end do
      end if
c
c*** Updates the pressure, if needed
c
      if(kpeel.eq.1) then
        call invmtx(premm,premi,dummy,nnods)
        call mbvab0(princ,premi,gtuve,nnods,nnods)
        coeff=1.0
        if(inpes.eq.1) coeff=0.0
        do inods=1,nnods
          elpre(inods)=coeff*elpre(inods)-princ(inods)/penal
        end do
      end if

      end
