      subroutine turmod(itufi,lnods,elmat,elrhs,elkin,elsec,ivari,ncomp,
     .                  lpoty,lroty)
c****************************************************************************
c   
c**** This routine modifies the element stiffness matrix of a turbulence
c**** parameter to account for the boundary conditions at the nodes with a
c**** wall law. The possibilities are as follows:
c
c     If NCOMP = 1
c       IVARI = 1: Modification for K
c       IVARI = 2: Modification for the second turbulence parameter
c      
c     If NCOMP = 2
c       Modification for both K and the second turbulence parameter       
c
c****************************************************************************
      implicit none
      include 'com_prob.f'
      include 'com_auxi.f'
      integer   inode,jnode,ivari,ncomp,iposi,ieva1,
     .          ieva2,jeva1,jeva2,ipoin,ibopo,iroty
      real*8    presc,adiag
      integer   itufi(npoin*2),                  lnods(nnode),
     .          lpoty(npoin),                    lroty(nbopo)
      real*8    elmat(ncomp*nnode,ncomp*nnode),  elkin(nnode),
     .          elrhs(ncomp*nnode),              elsec(nnode)
c
c*** Modification of one parameter. ITUFI is stored per blocks.
c
      if(ncomp.eq.1) then
        do inode=1,nnode
          ipoin=lnods(inode)
          ibopo=lpoty(ipoin)
          if (ibopo.gt.0) then
            iroty=-lroty(ibopo)
            if (iroty.gt.0) ipoin=iroty
          end if
          iposi=(ivari-1)*npoin+ipoin
          if(itufi(iposi).eq.2) then
            if(ivari.eq.1) then
              presc=elkin(inode)
            else if(ivari.eq.2) then
              presc=elsec(inode)
            end if
            adiag=elmat(inode,inode)
            do jnode=1,nnode
              elrhs(jnode)=elrhs(jnode)-elmat(jnode,inode)*presc
              elmat(inode,jnode)=0.0
              elmat(jnode,inode)=0.0
            end do
            elmat(inode,inode)=adiag
            elrhs(inode)=presc*adiag
          endif
        end do
c
c*** Modification of two parameters. ITUFI is stored per nodes.
c        
      else if(ncomp.eq.2) then
        ieva1=-1
        do inode=1,nnode
          ipoin=lnods(inode)
          ibopo=lpoty(ipoin)
          if (ibopo.gt.0) then
            iroty=-lroty(ibopo)
            if (iroty.gt.0) ipoin=iroty
          end if
          ieva1=ieva1+2
          iposi=2*(ipoin-1)+1                               ! Deals with K
          if(itufi(iposi).eq.2) then
            presc=elkin(inode)
            adiag=elmat(ieva1,ieva1)
            do jnode=1,2*nnode
              elrhs(ieva1)=elrhs(ieva1)-elmat(jnode,ieva1)*presc
              elmat(ieva1,jnode)=0.0d0
              elmat(jnode,ieva1)=0.0d0
            end do
            elmat(ieva1,ieva1)=adiag
            elrhs(ieva1)=presc*adiag
          endif

          ieva2=ieva1+1
          iposi=iposi+1                                     ! Deals with ELSEC
          if(itufi(iposi).eq.2) then
            presc=elsec(inode)
            adiag=elmat(ieva2,ieva2)
            do jnode=1,2*nnode
              elrhs(ieva2)=elrhs(ieva2)-elmat(jnode,ieva2)*presc
              elmat(ieva2,jnode)=0.0d0
              elmat(jnode,ieva2)=0.0d0
            end do
            elmat(ieva2,ieva2)=adiag
            elrhs(ieva2)=presc*adiag
          endif
        end do
      end if

      end
