      subroutine kepgal(
     .  shape,cartd,nnode,ndime,ietns,tstep,visac,
     .  vista,ackin,aceps,olkin,oleps,wmatr,wrhsi,
     .  elkin,elene,elvel,turbu,vegau,grkea,grave,
     .  epsil,produ)
c***********************************************************************
c
c**** This routine computes the Galerkin contribution to the K-E system
c**** of equations. The Newton-Raphson linearization is assumed. It is
c**** understood at the moment of reaching this routine that JTURB = 2.      
c
c***********************************************************************
      implicit none
      integer
     .  nnode,ndime,ietns,kdime,ldime,inode,jnode,
     .  ieva1,ieva2,jeva1,jeva2
      real*8
     .  tstep,visac,vista,ackin,aceps,olkin,oleps,
     .  epsil,ack05,fact1,fact2,fact3,addi1,addi2,
     .  addi3,conve,fac31,fac11,akoae,xmass
      real*8
     .  wmatr(2*nnode,2*nnode), wrhsi(2*nnode), shape(nnode),
     .  cartd(ndime,nnode)    , elkin(nnode)  , elene(nnode),
     .  elvel(ndime,nnode)    , turbu(6)      , vegau(ndime),
     .  grkea(ndime,2)        , grave(ndime,ndime), produ(2)  
c
c*** Velocity, gradient of K, gradient of E and production term at 
c*** the current integration point
c
      do kdime=1,ndime  
        vegau(kdime)=0.0d0                                  ! u
        grkea(kdime,1)=0.0d0                                ! grad(k)
        grkea(kdime,2)=0.0d0                                ! grad(e)        
        do jnode=1,nnode
          vegau(kdime)=vegau(kdime)
     .      +shape(jnode)*elvel(kdime,jnode)
          grkea(kdime,1)=grkea(kdime,1)
     .      +cartd(kdime,jnode)*elkin(jnode)
          grkea(kdime,2)=grkea(kdime,2)
     .      +cartd(kdime,jnode)*elene(jnode)
        end do
        do ldime=1,ndime
          grave(kdime,ldime)=0.0d0
          do jnode=1,nnode
            grave(kdime,ldime)=grave(kdime,ldime)
     .        +cartd(kdime,jnode)*elvel(ldime,jnode)
          end do
        end do
      end do
      epsil=0.0d0                                           ! 2 E(u):E(u)
      do kdime=1,ndime
        do ldime=1,ndime                         
          epsil=epsil+grave(kdime,ldime)
     .      *(grave(kdime,ldime)+grave(ldime,kdime))
        end do
      end do
c
c*** Steady contribution to the RHS
c
c     vista=turbu(1)*ackin*ackin/aceps
      akoae= ackin/aceps
      fact2= turbu(1)*akoae
      fact3= fact2/turbu(3)                                 ! C_u*k/sig_k*e
      fac31= fact2/turbu(4)                                 ! C_u*k/sig_e*e   
      fact2=-epsil*fact2                                    ! -2*I^2*C_u*k/e
      fact1= fact3*akoae                                    ! C_u*k^2/sig_k*e^2
      fac11= fac31*akoae                                    ! C_u*k^2/sig_e*e^2
      ieva1=-1
      do inode=1,nnode
        ieva1=ieva1+2                                       ! k-position
        ieva2=ieva1+1                                       ! e-position
        addi1=0.0d0                                         ! grad(N).grad(k)
        addi2=0.0d0                                         ! grad(N).grad(e)
        do kdime=1,ndime
          addi1=addi1+cartd(kdime,inode)*grkea(kdime,1)
          addi2=addi2+cartd(kdime,inode)*grkea(kdime,2)
        end do
        addi1=0.0d0
        addi2=0.0d0
        wrhsi(ieva1)=wrhsi(ieva1)+addi1*vista/turbu(3)
     .    +produ(1)*shape(inode)-shape(inode)*aceps*0.0d0
        wrhsi(ieva2)=wrhsi(ieva2)+addi2*vista/turbu(4)
     .    +produ(2)*shape(inode)
     .    -shape(inode)*turbu(6)*aceps*aceps/ackin*0.0d0
c
c*** Transient contribution to the RHS
c
        if(ietns.eq.1) then
          wrhsi(ieva1)=wrhsi(ieva1)+shape(inode)*olkin/tstep
          wrhsi(ieva2)=wrhsi(ieva2)+shape(inode)*oleps/tstep          
        end if
c
c*** Steady contribution to the element matrix
c        
        jeva1=-1
        do jnode=1,nnode
          jeva1=jeva1+2
          jeva2=jeva1+1
          addi3=0.0d0                                       ! grad(N).grad(N)
          conve=0.0d0                                       ! N [u.grad(N)]
          xmass=shape(inode)*shape(jnode)                   ! N.N
          do kdime=1,ndime
            addi3=addi3+cartd(kdime,inode)*cartd(kdime,jnode)
            conve=conve+vegau(kdime)*cartd(kdime,jnode)
          end do
          conve=shape(inode)*conve
          wmatr(ieva1,jeva1)=wmatr(ieva1,jeva1)
     .      +conve+addi3*(visac+vista/turbu(3)) 
     .      +2.0d0*fact3*addi1*shape(jnode)
     .      -2.0d0*turbu(1)*akoae*epsil*xmass*0.0d0
          wmatr(ieva1,jeva2)=wmatr(ieva1,jeva2)
     .      -fact1*addi1*shape(jnode)
     .      +turbu(1)*akoae*akoae*epsil*xmass*0.0d0+xmass
          wmatr(ieva2,jeva1)=wmatr(ieva2,jeva1)
     .      +2.0d0*fac31*addi2*shape(jnode)
     .      -turbu(5)*turbu(1)*epsil*xmass*0.0d0
     .      -turbu(6)*xmass/(akoae*akoae)*0.0d0
          wmatr(ieva2,jeva2)=wmatr(ieva2,jeva2)
     .      +conve+addi3*(visac+vista/turbu(4))
     .      -fac11*addi2*shape(jnode)
     .      +1.0d0*turbu(6)*xmass/akoae
c
c*** Transient contribution to the element matrix
c          
          if(ietns.eq.1) then
            addi3=xmass/tstep
            wmatr(ieva1,jeva1)=wmatr(ieva1,jeva1)+addi3
            wmatr(ieva2,jeva2)=wmatr(ieva2,jeva2)+addi3            
          end if
        end do
      end do

      
      end
