      subroutine kepres(
     .  shape,cartd,hessi,nnode,ndime,ntens,ietns,itwtu,
     .  ksctu,neres,visac,vista,sftuc,shctu,zeroc,epsil,
     .  ackin,aceps,olkin,oleps,tstep,hleng,chale,wmatr,
     .  wrhsi,elkin,elene,elvit,turbu,vegau,grkea,vgran,
     .  produ)
c**************************************************************************
c
c**** This routine computes the SUPG & DC contributions to the K-E system
c**** of equations. All the terms except the convective one are moved to
c**** the RHS.
c
c**************************************************************************
      implicit none
      integer
     .  nnode,ndime,ntens,ietns,itwtu,ksctu,kdime,ldime,
     .  inode,jnode,ieva1,ieva2,jeva1,jeva2,neres
      real*8
     .  visac,vista,sftuc,shctu,zeroc,epsil,ackin,aceps,
     .  olkin,oleps,tstep,hleng,chale,chav1,chav2,grgr1,
     .  grgr2,grnut,vinod,xlap1,xlap2,ress1,ress2,tikin,
     .  tieps,elno1,pecl3,elno2,conv1,conv2,grno1,grno2,
     .  resd1,resd2,vepar,scdi1,scdi2,sldi1,sldi2,addit,
     .  addi1,addi2,prodv,pecle,vecave
      real*8
     .  wmatr(2*nnode,2*nnode), wrhsi(2*nnode),     shape(nnode),
     .  cartd(ndime,nnode),     hessi(ntens,nnode), elkin(nnode),
     .  elene(nnode),           turbu(6),           vegau(ndime), 
     .  grkea(ndime,2),         vgran(nnode),       elvit(nnode),
     .  produ(2)
c
c*** Set some values
c
      chav1=visac+vista/turbu(3)
      chav2=visac+vista/turbu(4)
c
c*** Compute the residual without the convective term (RESS1 and RESS2)
c
      if(sftuc+shctu.gt.zeroc) then
        grgr1=0.0                                           ! grad(nut).grad(k)
        grgr2=0.0                                           ! grad(nut).grad(e)
        do kdime=1,ndime
          grnut=0.0                                         ! grad(nut)
          do jnode=1,nnode
            vinod=elvit(jnode)
            grnut=grnut+cartd(kdime,jnode)*vinod
          end do
          grgr1=grgr1+grkea(kdime,1)*grnut
          grgr2=grgr2+grkea(kdime,2)*grnut          
        end do
        xlap1=0.0                                           ! div(grad(k))
        xlap2=0.0                                           ! div(grad(e))
        if(neres.ne.0) then
          do jnode=1,nnode
            xlap1=xlap1+vecave(hessi(1,jnode),ndime)*elkin(jnode)
            xlap2=xlap2+vecave(hessi(1,jnode),ndime)*elene(jnode)          
          end do
        end if
        ress1=-chav1*xlap1-grgr1/turbu(3)+aceps-produ(1)
        ress2=-chav2*xlap2-grgr2/turbu(4)
     .    -produ(2)+turbu(6)*aceps*aceps/ackin
        if((ietns.eq.1).and.(itwtu.eq.1)) then
          ress1=ress1+(ackin-olkin)/tstep
          ress2=ress2+(aceps-oleps)/tstep          
        end if
        call vecnor(vegau,ndime,elno1,2)                    ! |u|
      end if
c
c*** Compute the SUPG perturbation to the element matrix and RHS
c
      tikin=0.0
      if((sftuc.gt.zeroc).and.(elno1.gt.zeroc)) then
        do jnode=1,nnode                                    ! v.grad(N)
          vgran(jnode)=0.0
          do kdime=1,ndime
            vgran(jnode)=vgran(jnode)+vegau(kdime)*cartd(kdime,jnode)
          end do
        end do
        if(chav1.gt.zeroc) then                             ! Compute tau for k
          pecl3=0.5*elno1*chale/(chav1*3.0)
          tikin=0.5*sftuc*(min(pecl3,1.0d0))*chale/elno1
        else
          tikin=0.5*sftuc*chale/elno1
        end if
        if(chav2.gt.zeroc) then                             ! Compute tau for e
          pecl3=0.5*elno1*chale/(chav2*3.0)
          tieps=0.5*sftuc*(min(pecl3,1.0d0))*chale/elno1
        else
          tieps=0.5*sftuc*chale/elno1
        end if
        ieva1=-1
        do inode=1,nnode
          ieva1=ieva1+2
          ieva2=ieva1+1
          wrhsi(ieva1)=wrhsi(ieva1)-tikin*vgran(inode)*ress1
          wrhsi(ieva2)=wrhsi(ieva2)-tieps*vgran(inode)*ress2
          jeva1=-1
          do jnode=1,nnode
            jeva1=jeva1+2
            jeva2=jeva1+1
            addit=vgran(inode)*vgran(jnode)
            wmatr(ieva1,jeva1)=wmatr(ieva1,jeva1)+tikin*addit
            wmatr(ieva2,jeva2)=wmatr(ieva2,jeva2)+tieps*addit            
          end do
        end do
      end if
c
c*** Compute the discontinuity-capturing contribution to the
c*** element matrix
c
      if(shctu.gt.zeroc) then
        elno2=elno1*elno1                                   ! |u|^2
        conv1=0.0                                           ! u.grad(k)
        conv2=0.0                                           ! u.grad(e)        
        grno1=0.0                                           ! |grad(k)|
        grno2=0.0                                           ! |grad(e)|
        do kdime=1,ndime
          conv1=conv1+vegau(kdime)*grkea(kdime,1)
          conv2=conv2+vegau(kdime)*grkea(kdime,2)
          grno1=grno1+grkea(kdime,1)*grkea(kdime,1)
          grno2=grno2+grkea(kdime,2)*grkea(kdime,2)          
        end do
        grno1=sqrt(grno1)
        grno2=sqrt(grno2)        
        resd1=abs(ress1+conv1)                              ! Total res. for k
        resd2=abs(ress2+conv2)                              ! Total res. for e
        scdi1=0.0
        sldi1=0.0
        scdi2=0.0
        sldi2=0.0
        if((resd1.gt.zeroc).and.(grno1.gt.zeroc)) then
          vepar=resd1/grno1
          if (chav1.ge.zeroc) then
            pecle=vepar*hleng*0.5/chav1
            scdi1=0.5*(shctu-1.0/pecle)*hleng*vepar
            if(scdi1.lt.0.0) scdi1=0.0
          else
            scdi1=0.5*shctu*hleng*vepar
          end if
          sldi1=scdi1                                       ! Isotropic SC (k)
          if(ksctu.eq.2) then                               ! Anisot. SC (k)
            sldi1=scdi1-tikin*elno2
            if(sldi1.lt.0.0) sldi1=0.0
          end if
        end if
        if((resd2.gt.zeroc).and.(grno2.gt.zeroc)) then        
          vepar=resd2/grno2
          if (chav2.ge.zeroc) then
            pecle=vepar*hleng*0.5/chav2
            scdi2=0.5*(shctu-1.0/pecle)*hleng*vepar
            if(scdi2.lt.0.0) scdi2=0.0
          else
            scdi2=0.5*shctu*hleng*vepar
          end if
          sldi2=scdi2                                       ! Isotropic SC (e)
          if(ksctu.eq.2) then                               ! Anisot. SC (e)
            sldi2=scdi2-tieps*elno2
            if(sldi2.lt.0.0) sldi2=0.0
          end if
        end if
        if((scdi1+scdi2).gt.zeroc) then
          ieva1=-1
          do inode=1,nnode
            ieva1=ieva1+2
            ieva2=ieva1+1
            jeva1=-1
            do jnode=1,nnode
              jeva1=jeva1+2
              jeva2=jeva1+1
              addi1=0.0
              addi2=0.0              
              do kdime=1,ndime
                prodv=cartd(kdime,inode)*cartd(kdime,jnode)
                addi1=addi1+scdi1*prodv
                addi2=addi2+scdi2*prodv                
                if(elno2.gt.zeroc) then
                  do ldime=1,ndime
                    prodv=cartd(kdime,inode)*vegau(kdime)
     .                *vegau(ldime)*cartd(ldime,jnode)/elno2
                    addi1=addi1+(sldi1-scdi1)*prodv
                    addi2=addi2+(sldi2-scdi2)*prodv 
                  end do
                end if
              end do
              wmatr(ieva1,jeva1)=wmatr(ieva1,jeva1)+addi1
              wmatr(ieva2,jeva2)=wmatr(ieva2,jeva2)+addi2              
            end do
          end do
        end if
      end if
      
      end
