      subroutine kingal(
     .  shape,cartd,nnode,ndime,ietns,jturb,jlitu,
     .  tstep,visac,vista,ackin,aceps,olkin,wmatr,
     .  wrhsi,elkin,elvel,grave,turbu,vegau,grkac,
     .  qprod,naxis,gpcor,zeroc)
c*************************************************************************
c
c**** This routine computes the Galerkin contribution to the K-equation.
c**** For the K model (1 equation) the production term is written as
c**** QPROD = Cu sqrt(K) L Q, where Q = 2 E(u):E(u) and the term sqrt(K) 
c**** is linearized. The turbulent kinematic viscosity is also expressed
c**** in terms of K and L. For 2 equation models, QPROD and the turbulent
c**** kinematic viscosity are frozen and computed previously in UPPROD
c**** and UPDVIS, respectively.      
c
c*************************************************************************
      implicit none
      integer   nnode,ndime,ietns,jturb,jlitu,kdime,ldime,
     .          inode,jnode,i,j,naxis
      real*8    tstep,visac,vista,ackin,olkin,ack05,fact1,
     .          fact2,fact3,addi1,addi2,conve,xnewt,qprod,
     .          aceps,xmile,chavi,gpcor,zeroc
      real*8    wmatr(nnode,nnode),    wrhsi(nnode),
     .          shape(nnode),          cartd(ndime,nnode),
     .          elkin(nnode),          elvel(ndime,nnode), 
     .          grave(ndime,ndime),    turbu(6),
     .          vegau(ndime),          grkac(ndime)

      chavi=visac+vista/turbu(3)
c
c*** Velocity, gradient of K and production term at the current
c*** integration point
c
      do kdime=1,ndime  
        vegau(kdime)=0.0d0                                  ! v
        grkac(kdime)=0.0d0                                  ! grad(K)
        do jnode=1,nnode
          vegau(kdime)=vegau(kdime)+shape(jnode)*elvel(kdime,jnode)
          grkac(kdime)=grkac(kdime)+cartd(kdime,jnode)*elkin(jnode)
        end do
      end do
c
c*** K-model: The production term QPROD is now computed.     
c
      aceps= turbu(1)*ackin*ackin/vista      
      if(jturb.eq.2) then
        do i=1,ndime
          do j=1,ndime
            grave(i,j)=0.0d0
            do inode=1,nnode
              grave(i,j)=grave(i,j)+cartd(i,inode)*
     .          elvel(j,inode)
            end do
          end do
        end do
        qprod = 0.0d0
        do i = 1,ndime
          do j = 1,ndime
            qprod= qprod+grave(i,j)*(grave(i,j)+grave(j,i))
          end do
        end do
        if (naxis.eq.1) then                                ! Axisymetric
          if (abs(gpcor).ge.zeroc) then                     ! contribution
            qprod=qprod+2.d0*vegau(1)*vegau(1)/(gpcor*gpcor)
          else
            qprod=qprod+2.d0*grave(1,1)*grave(1,1)
          end if
        end if
        qprod=qprod*vista
      end if
c
c*** 2-Equations models: linearization with nu_t fixed. The production
c*** term QPROD is now used.        
c
      if (jlitu.eq.0) then                                  ! RHS
        fact1=0.0d0
      else if (jlitu.eq.1) then                             ! Picard
        fact1=turbu(1)*ackin/vista
      else if (jlitu.eq.2) then                             ! Newton_Raphson
        fact1=2.0d0*turbu(1)*ackin/vista
      end if
      fact2=turbu(1)*ackin*ackin/vista
      do inode=1,nnode                                      ! Steady RHS
        wrhsi(inode)=wrhsi(inode)+qprod*shape(inode)
        if(jlitu.eq.0) then
          wrhsi(inode)=wrhsi(inode)-fact2*shape(inode)      ! C_u*K^2/E
        else if(jlitu.eq.2) then  
          wrhsi(inode)=wrhsi(inode)+fact2*shape(inode)
        end if
        if(ietns.eq.1) then                                 ! Transient RHS 
          wrhsi(inode)=wrhsi(inode)
     .      +shape(inode)*olkin/tstep
        end if
        do jnode=1,nnode
          addi2=0.0d0                                       ! Steady matrix
          conve=0.0d0
          do kdime=1,ndime
            addi2=addi2+cartd(kdime,inode)*cartd(kdime,jnode)
            conve=conve+vegau(kdime)*cartd(kdime,jnode)
          end do
          wmatr(inode,jnode)=wmatr(inode,jnode)
     .      +addi2*chavi
     .      +fact1*shape(inode)*shape(jnode)
     .      +shape(inode)*conve
          if(ietns.eq.1) then                               ! Transient matrix
            wmatr(inode,jnode)=wmatr(inode,jnode)
     .        +shape(inode)*shape(jnode)/tstep
          end if
        end do
      end do
      
      end
