      subroutine iterat(
     .  lnods,lnodb,lboel,iffix,ivabo,coord,presc,
     .  tload,veloc,presu,tempe,fresu,shape,shaga,
     .  shapb,elpro,rhsid,work1)
c**************************************************************************
c      
c**** This routine controls the the order in which the different problems
c**** are solved (Navier-Stokes equations, heat equation and free-surface
c**** tracking). It also  checks convergence of  the global iterative pro-
c**** cesses and updates the unknowns.
c
c**************************************************************************
      implicit none
      include 'com_prob.f'
      include 'com_auxi.f'
      logical*1 wellf
      integer   inspa,ihepa,ifspa,korde,icupr,jetns,jecns,
     .          jsnsc,lturb
      integer   pdone(3)
      real*8    cpucu
      data inspa,ihepa,ifspa/0,0,0/
      save inspa,ihepa,ifspa
      integer   lnods(*),lnodb(*),lboel(*),iffix(*),
     .          ivabo(*)
      real*8    coord(*),presc(*),tload(*),veloc(*),
     .          presu(*),tempe(*),fresu(*),shape(*),
     .          shaga(*),shapb(*),elpro(*),rhsid(*),
     .          work1(*)
c
c*** Write to a file to be used for a re-start run if the CPU time limit
c*** has been reached
c
      call cputim(cpucu)
      cpucu=cpucu-cpuin
      if (tlimt.le.cpucu .or. preli) then
        wellf=.true.
        if (tlimt.le.cpucu) wellf=.false.
        call outrst(veloc,presu,tempe,fresu,
     .    %val(tkine_p),%val(tener_p),%val(cletu_p),%val(visct_p),
     .    elpro,wellf)
      end if
c
c*** Loop on iterations
c
      iiter=iiter+1
      call lvecze(3,pdone)
      if(iiter.eq.1) then
        inspa=0
        ihepa=0
        ifspa=0
      else if (rstar) then
        inspa=iiter                           ! The run is a RESTART
        ihepa=iiter
        ifspa=iiter
      end if
      korde=0
    1 korde=korde+1
      icupr=iorde(korde)
      if(korde.le.3) then                     ! Direct to appropriate module
        go to ( 10 , 20 , 30 ) , icupr
      else
        call conupd(veloc,presu,tempe,fresu,
     .    pdone)                              ! Check convergence and
        return                                ! update unknowns
      end if
c
c*** Deal with the incompressible Navier-Stokes equations
c
   10 if(iepns.eq.0)                       go to 1    ! NS eqs. not solved
      if((istag(1).eq.1).and.inspa.gt.0) then         ! NS eqs. are staggered
        pdone(1) = 1
        go to 1
      end if
      if(iiter.gt.micns)                   go to 1    ! NS eqs. not converged
      jlins=ilins
      if((ilins.eq.2).and.(iiter.le.npica)) jlins=1
      if (istep+iiter.ne.2)                           ! Updates turbulence
     .  call turpar(iffix,lnods,coord,veloc,          ! parameters (the first 
     .              presu,tempe,fresu,rhsid,          ! iteration is laminar)
     .              elpro,shape,shaga,work1)
      jetns=ietns                                     ! Save the real values
      jecns=iecns                                    
      jsnsc=lsnsc
      lturb=kturb
      if (istep+iiter.eq.2) then
        if (inist) then
          kturb=0                                     ! Solve the Stokes
          ietns=0                                     ! problem at the first 
          iecns=0                                     ! iteration of the 
          lsnsc=0                                     ! first time step
        end if
      end if
      call navsto(lnods,lnodb,lboel,iffix,
     .            ivabo,coord,tload,veloc,
     .            presu,tempe,fresu,shape,
     .            shaga,shapb,elpro,rhsid,
     .            work1)
      pdone(1)=1                                      ! Iteration done
      ietns=jetns                                     ! Restore the real
      iecns=jecns                                     ! values
      lsnsc=jsnsc
      kturb=lturb
      inspa=inspa+1                                   ! Num. of passes
      if ((istep+iiter.eq.2).and.(inist)
     .  .and.(ietns.eq.1)) then
        call vecasi(ntotv,veloc,veloc(1+2*ntotv))
        call vecasi(ntotp,presu,presu(1+2*ntotp))
      end if
      go to 1
c
c*** Deal with the heat equation
c
   20 if(iephe.eq.0)                       go to 1    ! Heat eq. not solved
      if((istag(2).eq.1).and.ihepa.gt.0) then         ! Heat eq. is staggered
        pdone(2) = 1
        go to 1
      end if
      if((istag(2).eq.1).and.istep.eq.1) then         ! Heat solv at end of step
        pdone(2) = 1
        go to 1
      end if
      if(iiter.gt.miche)                   go to 1    ! Heat eq. not converged
      call temper(lnods,lnodb,lboel,iffix,
     .            ivabo,coord,tload,veloc,
     .            presu,tempe,fresu,shape,
     .            shaga,shapb,elpro,rhsid,
     .            work1)
      pdone(2)=1                                      ! Iteration done
      ihepa=ihepa+1                                   ! Num. of passes
      go to 1
c
c*** Tracking of the free surface (FST) 
c
   30 if(iepfs.eq.0)                       go to 1    ! FST not solved
      if((istag(3).eq.1).and.ifspa.gt.0) then         ! FST is staggered
        pdone(3) = 1
        go to 1
      end if
      if((istag(3).eq.1).and.istep.eq.1) then         ! FST solv at end of step 
        pdone(3) = 1
        go to 1
      end if
      if(iiter.gt.micpc)                   go to 1    ! FST not converged
      call frsutr(lnods,lnodb,lboel,iffix,
     .            ivabo,coord,tload,veloc,
     .            presu,tempe,fresu,shape,
     .            shapb,elpro,rhsid,work1)
      pdone(3)=1                                      ! Iteration done
      ifspa=ifspa+1                                   ! Num. of passes
      go to 1
      
      end
