      subroutine nvsvef(delta,frico,tveno,vikin,velfr,vonka,zeroc)
c*****************************************************************************
c
c*** Computes the friction velocity by the Newton-Raphson method. This friction
c*** velocity is computed from  U_p / U_* = (1/X) log (U_* D/nu) + C, where
c*** X = 0.41 (VONKA) is the von Karman constant, U_p is the tangent wall ve-
c*** locity at the current integration point, C is a constant which value is
c*** 5.5 and D = DELTA is a user defined paramater (TURBU(8)), its physical 
c*** meaning being the boundary layer width. The result is VELFR.
c
c*****************************************************************************
      implicit none
      integer   itera,j
      real*8    delta,tveno,vikin,velfr,xmuit,fdvfr,devfr,
     .          frico,vkinv,diffd,parco,vonka,zeroc
c
c*** Case with non-zero boundary layer width
c
      if (delta.gt.zeroc) then
        velfr=sqrt(tveno*vikin/delta)
        if (velfr*delta/vikin .gt. 30.0d0) then
          vkinv=1.0d0/vonka
          xmuit=delta/vikin                                 ! D/nu
          itera=0                                           ! i=0
          velfr=tveno                                       ! U_*^0
          parco=1.0d0
          do while((parco.ge.1.0e-6).and.itera.lt.100)
            itera=itera+1                                   ! i=i+1
            if (velfr.lt.zeroc) velfr=zeroc
            fdvfr=velfr*(vkinv*log(xmuit*velfr)+5.5d0)
     .        -tveno                                        ! F(U_*^i)
            diffd=vkinv*(log(xmuit*velfr)+1.0d0)+5.5d0      ! DF(U_*^i)
            if(abs(diffd).lt.zeroc) diffd=zeroc
            devfr=-fdvfr/diffd                              ! delta(U_*^0)
            parco=abs(devfr/tveno)                          ! U_*^i=U_*^i-1
            velfr=velfr+devfr                               ! +delta(U_*)
          end do
        end if
c
c*** If DELTA = 0, either a friction coefficient is prescribed (TURBU(9) in
c*** the data file) or the friction velocity is set to zero.
c
      else if (frico.gt.zeroc) then  
        velfr=tveno/frico
      else
        velfr=0.0d0
      end if

      end
