      subroutine turpar(iffix,lnods,coord,veloc,presu,tempe,fresu,
     .                  rhsid,elpro,shape,shaga,work1)
c***********************************************************************
c        
c**** This routine computes the turbulent parameters according to the
c**** turbulence model that has been chosen
c      
c***********************************************************************
      implicit none
      include 'com_prob.f'
      include 'com_auxi.f'
      include 'def_logi.f'
      character*14 wopos
      integer   tuite,ipase,i
      integer   iffix(nvfix), lnods(nnodt,nelem)
      real*8    erro1,erro2,time1,time2,cucpu
      real*8    coord(ndime,npoin),       veloc(ntotv,ncove),
     .          presu(ntotp,ncopr),       tempe(npoin,ncote),
     .          fresu(npoin,ncofs),       rhsid(npoin),
     .          elpro(nprod,ngaus,nelem), shape(nnodt,ngaus),
     .          shaga(ngaus,nnode),       work1(*)  
c
c*** Print headings
c
      call cputim(time1)
      cucpu=time1-cpuin
      if(kturb.ge.2) then
        write(lulog,100)
      end if
c
c*** Compute the kinematic viscosity at the boundary nodes to prescribe
c*** the wall law there
c
      if(kturb.ge.1) 
     .  call visbou(%val(itufi_p),lnods,%val(vikin_p),
     .              fresu,shaga,elpro,work1)
c
c*** Compute boundary conditions according to the following strategy:
c*** KWALL = 1 --> The turbulent stress is prescribed. The iterative scheme
c                  is: guess U, compute U_*, compute and fix T (stress).      
c*** KWALL = 2 --> The velocity is precribed. The iterative scheme is:
c                  guess T, compute U_*, compute and fix U.
c*** The boundary values of the turbulence parameters are also computed.
c      
      if(kwall.eq.1 .and. kturb.ge.2) then                                 
        call turwal(%val(tkine_p),%val(tener_p),           
     .              %val(itufi_p),%val(vikin_p),                     
     .              veloc,%val(lpoty_p),%val(lroty_p))
      else if (kwall.ne.1 .and. kturb.ge.1) then
        call turwa1(lnods,%val(itufi_p),shaga,veloc,   
     .              %val(tkine_p),%val(tener_p),                 
     .              %val(vikin_p),%val(lpoty_p),
     .              %val(lroty_p),%val(exnor_p),iffix,work1,        
     .              work1(1+2*npoin),work1(1+3*npoin)) 
      end if
c
c*** Initializations
c
      tuite=0
      ipase=0
      erro1=1.0+toctu
      erro2=0.0
      tuerr=1.0+toctu
c
c*** Updates production terms for K and E
c
      if (kreys.eq.1) then
        if (iiter.gt.int(turbu(16)) .and. kturb.ge.2) jreys=1
      end if
      if (jreys.eq.1)                                       ! Update Reynolds tensor
     .  call turrey(lnods,shape,veloc,%val(produ_p),work1)  
      if (kturb.ge.3)                                       ! Update production
     .  call upprod(lnods,shape,veloc,%val(produ_p),work1)  ! terms
c
c*** Main iterarive loop
c      
      do while (tuerr.gt.toctu.and.tuite.lt.micou)
        tuite=tuite+1
        jlitu=ilitu
        if(tuite.le.npitu) jlitu=1
c
c*** Laminar flow
c        
        if (kturb.eq.0) then
          tuerr=0.0
c
c*** Mixing length/Smagorinski's model (zero-equations model)
c          
        else if (kturb.eq.1) then     
          tuerr=0.0
c
c*** Solve the K-equation  (one- or two-equations models)
c          
        else if (kturb.ge.2) then
          jturb=kturb
          if((istep.eq.1).and.(iiter-1.le.nkwit)
     .      .and.(erro1.ge.toctu)
     .      .and.(ipase.eq.0    )) jturb=2
          if((jturb.eq.2).or.(tucou.eq.1)) then             ! Solve Per_Blocks
            call tukine(lnods,%val(itufi_p),coord,veloc,    ! Compute K
     .                  presu,tempe,fresu,%val(tkine_p),
     .                  shape,shaga,elpro,rhsid,work1,erro1)
            call upvist(%val(tkine_p),%val(cletu_p),
     .                  %val(visct_p),npoin,turbu)
          end if
c
c*** Solve for W of the K-W model
c
          if (jturb.eq.3) then
            ipase=ipase+1                                   ! Number of passes
            call tutaut(lnods,%val(itufi_p),coord,veloc,
     .                  presu,tempe,fresu,%val(tener_p),
     .                  shape,shaga,elpro,rhsid,work1)
            call upleng(%val(tener_p),%val(tkine_p),        ! Update L
     .                  %val(cletu_p),turbu,npoin,jturb) 
            call relunk(%val(cletu_p),reptu,npoin)
            call tuconv(%val(cletu_p),work1,erro1,npoin,2)  ! Check error
            call upvist(%val(tkine_p),%val(cletu_p),
     .                  %val(visct_p),npoin,turbu)
            write(lulog,105) tuite,erro1
c
c*** Solve for E of the K-E model
c
          else if (jturb.eq.4) then
            ipase=ipase+1                                    ! Number of passes
            if(tucou.eq.1) then                              ! Solve only for E
              call tuepsi(lnods,%val(itufi_p),coord,veloc,
     .                    presu,tempe,fresu,%val(tener_p),
     .                    shape,shaga,elpro,rhsid,work1)
              call upleng(%val(tener_p),%val(tkine_p),       ! Update L
     .                    %val(cletu_p),turbu,npoin,jturb) 
              call relunk(%val(cletu_p),reptu,npoin)
              call tuconv(%val(cletu_p),work1,erro1,npoin,2) ! Check error
              call upvist(%val(tkine_p),%val(cletu_p),
     .                    %val(visct_p),npoin,turbu)
              write(lulog,105) tuite,erro1
            else if(tucou.eq.0) then                         ! Solve for K-E 
              call tukiep(lnods,%val(itufi_p),coord,veloc,
     .                    presu,tempe,fresu,%val(tkine_p),
     .                    %val(tener_p),shape,shaga,elpro,
     .                    rhsid,work1,work1(1+npoin))
              call relunk(%val(tkine_p),reptu,npoin)         ! Relaxation
              call relunk(%val(tener_p),reptu,npoin)         ! Relaxation
              call upleng(%val(tener_p),%val(tkine_p),       ! Update L
     .                    %val(cletu_p),turbu,npoin,jturb)
              call upvist(%val(tkine_p),%val(cletu_p),
     .                    %val(visct_p),npoin,turbu)
              call tuconv(%val(tkine_p),work1,erro1,npoin,2) ! Check error
              call tuconv(%val(tener_p),work1,erro2,npoin,2) ! Check error
              write(lulog,106) tuite,erro1,', E-eq:',erro2
            end if
c
c*** Solve for W of the K-W model
c
          else if (jturb.eq.5) then
            ipase=ipase+1                                    ! Number of passes
            call tuwort(lnods,%val(itufi_p),coord,veloc,
     .                  presu,tempe,fresu,%val(tener_p),
     .                  shape,shaga,elpro,rhsid,work1)
            call upleng(%val(tener_p),%val(tkine_p),        ! Update L
     .                  %val(cletu_p),turbu,npoin,jturb) 
            call relunk(%val(cletu_p),reptu,npoin)
            call tuconv(%val(cletu_p),work1,erro1,npoin,2)  ! Check error
            call upvist(%val(tkine_p),%val(cletu_p),
     .                  %val(visct_p),npoin,turbu)
            write(lulog,105) tuite,erro1
          else if (jturb.ge.6) then
            call runend('TURPAR: MODEL NOT YET IMPLEMENTED')
          end if
        end if
        tuerr=max(erro1,erro2)
      end do
c
c*** Updates Algebraic stress tensor
c
      if (jreys.eq.1)
     .  call turrey(lnods,shape,veloc,%val(produ_p),work1)
c
c*** Write the evolution of the iterative process
c          
      if(kturb.ge.2) write(lulog,110) tuerr
      call cputim(time2)
      cputu=cputu+(time2-time1)
c          
c*** Formats
c
  100 format(/5x,'#### INNER EVOLUTION OF THE TURBULENCE ',
     .  'PARAMETERS IN L2:')
  105 format(11x,'*** Outer iter. for L ',i3,', R = ',e9.3,' %')
  106 format(11x,'*** Iter. ',i3,', K-eq: R = ',e9.3,' %',
     .  a7,' R = ',e9.3,' %')
  110 format(11x,'*** Maximum error at convergence = ',e12.6,' ( % )')
      
      end
