      subroutine turwal(tkine,tener,itufi,vikin,veloc,lpoty,lroty)
c******************************************************************************
c
c**** This routine computes K on the nodes with wall laws as K=U_*^2/sqrt(c_u).
c**** Also, the second turbulence parameter (TENER) is computed in the
c**** boundaries according to:
c
c     KTURB =  3. Compute KT = X y U_* / c_u 
c     KTURB =  4. Compute E  = U_*^3/(X y) on the boundary.
c     KTURB =  5. Compute W  = U_*/[sqrt(c_u) X y]
c
c**** where KTURB acts in this routine a temporary value of KTURB.      
c
c******************************************************************************
      implicit none
      include 'com_prob.f'
      include 'com_auxi.f'      
      integer   ipoin,itotv,ipowa,ibopo,iroty
      integer   itufi(npoin,2), lpoty(npoin), lroty(nbopo)
      real*8    tveno,velfr,vonka
      real*8    tkine(npoin,ncove), tener(npoin,ncove),
     .          vikin(npowa),       veloc(npoin,ncove) 

      ipowa=0
      do ipoin=1,npoin
        velfr=0.0
c
c*** Compute K as K = U_*^2/sqrt(c_u) at the nodes with wall law
c        
        if(itufi(ipoin,1).eq.2) then
          ipowa=ipowa+1
          vonka=0.41
          itotv=(ipoin-1)*ndime+1
          call vecnor(veloc(itotv,1),ndime,tveno,2)
          call nvsvef(turbu(8),turbu(9),tveno,vikin(ipowa),
     .                velfr,vonka,zero3)
          tkine(ipoin,1)=velfr*velfr/(sqrt(turbu(1)))
          tkine(ipoin,1)=max(tkine(ipoin,1),tkcom)
          tkine(ipoin,2)=tkine(ipoin,1)
        end if
c
c*** Compute the second turbulence parameter
c        
        if(kturb.eq.3) then
c
c*** For the K-KT model, KT is prescribed to KT = X y U_*/ c_u
c*** in the part of the boundary with wall laws (y = TURBU(8) is the boundary
c*** layer thickness and X = 0.41 is the von Karman constant).
c*** In the interior of the domain it is unknown, and in the Dirichlet 
c*** boundary given by TURFIX.
c
          if(itufi(ipoin,2).eq.2) then
            tener(ipoin,1)=vonka*turbu(8)*velfr/turbu(1)
            tener(ipoin,1)=max(tener(ipoin,1),tecom)
            tener(ipoin,2)=tener(ipoin,1)
          end if
        else if(kturb.eq.4) then
c
c*** For the K-E model, E is prescribed to E = U_*^3/(X y) in the part of the
c*** boundary with wall laws (y = TURBU(8) is the boundary layer thickness
c*** and X = 0.41 is the von Karman constant). In the interior of the domain
c*** it is unknown, and in the Dirichlet boundary given by TURFIX.
c
          if(itufi(ipoin,2).eq.2) then               
            tener(ipoin,1)=velfr*velfr*velfr/(vonka*turbu(8))
            tener(ipoin,1)=max(tener(ipoin,1),tecom)
            tener(ipoin,2)=tener(ipoin,1)
          end if
        else if(kturb.eq.5) then
c
c*** For the K-W model, W is prescribed to W = U_*/(sqrt(c_u) X y) in the
c*** part of the boundary with wall laws (y = TURBU(8) is the boundary layer
c*** thickness and X = 0.41 is the von Karman constant). In the interior of
c*** the domain it is unknown, and in the Dirichlet boundary given by TURFIX.
c
          if(itufi(ipoin,2).eq.2) then
            tener(ipoin,1)=velfr/(sqrt(turbu(1))*vonka*turbu(8))
            tener(ipoin,1)=max(tener(ipoin,1),tecom)
            tener(ipoin,2)=tener(ipoin,1)
          end if
        else if(kturb.ge.6) then
          call runend('TURWAL: DEFINE HOW TO OBTAIN TENER')
        end if
      end do
c
c*** Take in account the periodicity of turbulent variables (Update V(.,2))
c
      do ipoin=1,npoin
        ibopo=lpoty(ipoin)
        if (ibopo.ne.0) then
          iroty=-lroty(ibopo)
          if (iroty.gt.0) then
            if (itufi(iroty,1).eq.2) then
              tkine(ipoin,1)=tkine(iroty,1)
              tkine(ipoin,2)=tkine(ipoin,1)
            end if
            if (kturb.ge.3) then
              if (itufi(iroty,2).eq.2) then
                tener(ipoin,1)=tener(iroty,1)
                tener(ipoin,2)=tener(ipoin,1)
              end if
            end if
          end if
        end if
      end do

      end
