      subroutine inisol(
     .  imsns,imstu,imshe,imspc,ksyns,ksyhe,irsns,
     .  irstu,irshe,irspc,iasns,iastu,iashe,iaspc,
     .  mians,miatu,miahe,miapc,kryns,krytu,kryhe,
     .  krypc,toans,toatu,toahe,toapc,thrns,thrhe,
     .  thrpc,thrtu,lfins,lfihe,lfipc,lfitu,nscon,
     .  tucou,lenpo)
c***********************************************************************
c
c*** This subroutine initializes some solver parameters. The problems
c*** are numbered as follows:
c
c    Prob. Num. 1 ---> Navier-Stokes equations      
c               2 ---> Heat equation
c               3 ---> Free surface tracking (pseudo-concentration)
c               4 ---> Smoothing of discontinuous variables
c               5 ---> First turbulence parameter T1, always K
c               6 ---> Second turbulence parameter T2 or couple (T1,T2) 
c
c***********************************************************************
      implicit none
      include  'MatMan.h'
      integer
     .  iprob,imsns,imstu,imshe,imspc,ksyns,ksyhe,
     .  irsns,irstu,irshe,irspc,iasns,iastu,iashe,
     .  iaspc,mians,miatu,miahe,miapc,kryns,krytu,
     .  kryhe,krypc,lfins,lfihe,lfipc,lfitu,tucou,
     .  lenpo
      real*8    toans,toatu,toahe,toapc,thrns,thrhe,thrpc,
     .          thrtu
      logical*1 nscon
c
c*** Length of pointers
c
      lengp=lenpo
c
c*** Matrix storage/preconditioning strategy
c
      kmsip(1)=imsns
      kmsip(2)=imshe
      kmsip(3)=imspc
      kmsip(4)=max(max(imsns,imshe),imspc)
      kmsip(5)=imstu
      kmsip(6)=imstu
c
c*** Matrix symmetry
c
      ksymm(1)=ksyns
      ksymm(2)=ksyhe
      ksymm(3)=1
      ksymm(4)=0
      ksymm(5)=1
      ksymm(6)=1
c
c*** Renumber or not
c
      krenu(1)=irsns
      krenu(2)=irshe
      krenu(3)=irspc
      krenu(4)=max(max(irsns,irshe),irspc)
      krenu(5)=irstu
      krenu(6)=irstu
c
c*** Multiplier for sparse matrix dimension
c
      do iprob=1,mprob
        nmult(iprob)=3
      end do
c
c*** Type of solver
c
      kites(1)=iasns
      kites(2)=iashe
      kites(3)=iaspc
      kites(4)=min(min(iasns,iashe),iaspc)
      kites(5)=iastu
      kites(6)=iastu
c
c***  Maximum number of iterations for iterative solvers
c
      itmax(1)=mians
      itmax(2)=miahe
      itmax(3)=miapc
      itmax(4)=min(min(mians,miahe),miapc)
      itmax(5)=miatu
      itmax(6)=miatu
c
c*** Tolerance for iterative solvers
c
      toler(1)=toans
      toler(2)=toahe
      toler(3)=toapc
      toler(4)=max(max(toans,toahe),toapc)
      toler(5)=toatu
      toler(6)=toatu
c
c*** Krylov sub-space for GMRES
c
      kkryl(1)=kryns
      kkryl(2)=kryhe
      kkryl(3)=krypc
      kkryl(4)=max(kryhe,10)
      kkryl(5)=krytu
      kkryl(6)=krytu
c
c*** Initialize some important words
c
      wosol(1)='NAVIER-STOKES  '
      wosol(2)='TEMPERATURE    '
      wosol(3)='PSEUDO-CONCENT.'
      wosol(4)='SMOOTHING      '
      wosol(5)='TURB. KIN. EN. '
      if (tucou.eq.1) then
        wosol(6)='2nd TURB. PAR. '
      else if (tucou.eq.0) then
        wosol(6)='K-E COUPLED SOL'
      end if
c
c*** Bubble nodes elimination
c
      conde(1)= nscon
      conde(2)=.false.
      conde(3)=.false.
      conde(4)=.false.
      conde(5)=.false.
      conde(6)=.false.
c
c*** Threshold and fill-in parameters for preconditioners
c
      thres(1)=thrns
      thres(2)=thrhe
      thres(3)=thrpc
      thres(4)=max(max(thrns,thrhe),thrpc)
      thres(5)=thrtu
      thres(6)=thrtu
      lfill(1)=lfins
      lfill(2)=lfihe
      lfill(3)=lfipc
      lfill(4)=max(max(lfins,lfihe),lfipc)
      lfill(5)=lfitu
      lfill(6)=lfitu

      end
