      subroutine starts(lnods,lnodb,lboel,iffix,ivapo,ivabo,coord,
     .                  presc,tload,veloc,presu,tempe,fresu,shape,
     .                  shaga,shapb,elpro,rhsid,work1)
c*****************************************************************************
c      
c**** This routine prepares to start computations. In what follows, the
c**** order of the problems to be considered is the following:
c
c     Problem 1 : Navier-Stokes equations
c     Problem 2 : Heat equation
c     Problem 3 : Free-surface tracking
c     Problem 4 : Smoothing of discontinuous variables (consistent)
c     Problem 5 : Equation for the turbulent kinetic energy
c     Problem 6 : Equation for the second turbulence parameter (e.g., epsilon)
c
c*****************************************************************************
      implicit real*8(a-h,o-z)
      include 'com_prob.f'
      include 'com_auxi.f'
      include 'def_logi.f'
      integer  lnods(*),lnodb(*),lboel(*),iffix(*),
     .         ivapo(*),ivabo(*)
      real*8   coord(*),presc(*),tload(*),veloc(*),
     .         presu(*),tempe(*),fresu(*),shape(*),
     .         shaga(*),shapb(*),elpro(*),rhsid(*),
     .         work1(*)
c
c*** Read mesh data (only vectors)
c
      call rmevec(lnods,lnodb,lboel,coord,%val(skcos_p),work1)
c
c*** Construct some arrays used throughout the program
c
      call conarr(lnods,lnodb,lboel,coord,work1)
c
c*** Construct the objects (elements & boundaries)
c
      call setobj(lnods,lnodb,lboel,coord,shape,shaga,shapb,work1,0)
c
c*** Construct the vector of exterior normals and compute the number
c*** of nodes on the boundary
c
      call extnor(lnods,work1(isetm(14)))
c
c*** Verify the normals at boundary integration points
c
      call setobj(lnods,lnodb,lboel,coord,shape,shaga,shapb,work1,1)
c
c*** Set the nodal connections of the sub-elements for the smoothing of the
c*** pseudo-concentration      
c
      if(ismpc.eq.1) call cosube(%val(lnodw_p))
c
c*** Allocate memory for the vector LBOTY
c
      call setbou
c
c*** Read fixity data (only vectors)
c
      call rfivec(iffix,presc,ivapo,tload,ivabo,
     .            %val(lroty_p),%val(lpoty_p),%val(lboty_p),
     .            %val(lbcon_p),%val(exnor_p),work1)
c
c*** Construct the rotation matrix for the slave nodes of periodical
c*** boundary conditions 
c
      call moslav(%val(lpoty_p),%val(lroty_p),%val(exnor_p),
     .            %val(skcos_p),coord,work1(1),work1(19),work1(28))
c
c*** Store the initial values for velocity, temperature and free surface
c
      call inival(iffix,lnods,veloc,presu,tempe,fresu,presc,
     .            work1,work1(1+npoin))
c
c*** Set the boundary conditions for turbulence models
c
      call turfix(iffix,lnods,lboel,coord,presc,
     .            %val(lpoty_p),%val(lroty_p),%val(lboty_p),
     .            %val(itufi_p),%val(tkine_p),%val(cletu_p),
     .            %val(tener_p),%val(exnor_p))
c
c*** Write the memory needed for the problem (except for the solver)
c
      call memout
c
c*** Initialize element properties if it is a NEW run
c
      if (.not.rstar)
     .  call elmacc(idumy,lnods,coord,shape,shaga,veloc,presu,tempe,
     .              fresu,elpro,rhsid,
     .              work1(iacel( 1)),work1(iacel( 2)),work1(iacel( 3)),
     .              work1(iacel( 4)),work1(iacel( 5)),work1(iacel( 6)),
     .              work1(iacel( 7)),work1(iacel( 8)),work1(iacel( 9)),
     .               0, 0, 0, 0, 1)
c
c*** Put atmosferic stress
c
      if (latmo.ne.0)
     .  call bouacc(lboel,iffix,lnods,ivabo,tload,coord,shapb,shape,
     .              shaga,veloc,presu,tempe,fresu,elpro,rhsid,
     .              work1(iacel( 1)),work1(iacel( 2)),work1(iacel( 3)),
     .              work1(iacel( 4)),work1(iacel( 5)),work1(iacel( 6)),
     .              work1(iacel( 7)),work1(iacel( 8)),work1(iacel( 9)),
     .                  0, 0, 0, 5)
c
c*** Initialize some solver parameters
c
      call inisol(imsns,imstu,imshe,imspc,ksyns,ksyhe,irsns,
     .            irstu,irshe,irspc,iasns,iastu,iashe,iaspc,
     .            mians,miatu,miahe,miapc,kryns,krytu,kryhe,
     .            krypc,toans,toatu,toahe,toapc,thrns,thrhe,
     .            thrpc,thrtu,lfins,lfihe,lfipc,lfitu,nscon,
     .            tucou,lenpo)
c
c*** Renumber of the equations
c
      call renequ(lnods,iffix,work1)
c
c*** Output geometry for postprocess
c
      call dumpgm(coord,lnods,%val(lpoty_p),%val(lroty_p),
     .            %val(exnor_p),%val(skcos_p),work1)

      end
