      subroutine turfix(
     .  iffix,lnods,lboel,coord,presc,lpoty,lroty,
     .  lboty,itufi,tkine,cletu,tener,exnor)
c***********************************************************************
c        
c**** This routine sets the boundary and initial conditions for the 
c**** turbulence parameters. It also computes the mixing length in the
c**** case it is variable.
c
c***********************************************************************
      implicit none
      include 'com_prob.f'
      include 'com_auxi.f'
      integer
     .  icoun,inodb,inode,ipoin,ibopo,itotv,idime,lbyts,
     .  iroty
      integer
     .  iffix(ntotv),                lnods(nnodt,nelem),
     .  lboel(nnodb+1,nboun),        lpoty(npoin),
     .  lroty(nbopo),                lboty(mboun,nclbo),
     .  itufi(npoin,2)
      real*8
     .  venor,velfr,dista,distn,hdist,y,xmile,time1,time2,
     .  vecdot
      real*8
     .  coord(ndime,npoin),          presc(ntotv),
     .  tkine(npoin,ncove),          tener(npoin,ncove),
     .  cletu(npoin)      ,          exnor(ndime,ndime,nbopo)

      call cputim(time1)
c
c*** Prescription of wall laws. If one of the components of the velocity has
c*** the code 3, the code for the turbulence parameters is set to 2. If this
c*** happens for at least NDIME nodes of the boundary, a stress wall law will
c*** be applied to it.
c
      if (kwall.eq.1) then                                  ! Velocity->Stress
        do iboun=1,nboun                                    ! for wall law
          icoun=0
          ielem=lboel(nnodb+1,iboun)
          do inodb=1,nnodb
            inode=lboel(inodb,iboun)
            ipoin=lnods(inode,ielem)
            ibopo=lpoty(ipoin)
            if (ibopo.ne.0) then
              iroty=-lroty(ibopo)
              if (iroty.gt.0) ipoin=iroty
              itotv=(ipoin-1)*ndime
              if (  iffix(itotv+    1).eq.3
     .          .or.iffix(itotv+    2).eq.3                 ! Wall law 
     .          .or.iffix(itotv+ndime).eq.3) then 
                icoun=icoun+1
                if (kturb.ge.1) then
                  itufi(ipoin,1)=2                          ! Presc. of k
                  if(kturb.eq.3.or.kturb.eq.4.
     .              or.kturb.eq.5) then
                    itufi(ipoin,2)=2                        ! Presc. of eps.
                  else if(kturb.ge.6) then
                    call runend(
     .                'TURFIX: PRESCRIBE TURBULENCE PARAMETER')
                  end if
                end if
              end if
            end if
          end do
          if (icoun.ge.ndime.and.lboty(iboun,1).eq.0)       ! Presc. wall law
     .      lboty(iboun,1)=1                                ! for N-S
        end do
      else
        itotv=0                                             ! Integration up to
        do ipoin=1,npoin                                    ! wall or
          icoun=0                                           ! stress->velocity
          do idime=1,ndime
            itotv=itotv+1
            if (iffix(itotv).eq.3) icoun=1
          end do
          if (icoun.eq.1) then
            if (kturb.ge.1) then
              itufi(ipoin,1)=2
              if (kturb.ge.3) itufi(ipoin,2)=2
            end if
          end if
        end do
      end if
c
c*** Compute the distance from all the points to the walls with a prescribed
c*** stress. These distances are stored in the array CLETU. The maximum to the
c*** walls (DICOM) is also computed. It is used later on as a comparison dis-
c*** tance.
c
      itotv=-ndime
      dicom=0.0d0
      do ipoin=1,npoin               
        dista=0.0d0                    
        icoun=0                      
        do iboun=1,nboun             
          if (lboty(iboun,1).eq.1) then                     ! Boundary with  
            ielem=lboel(nnodb+1,iboun)                      ! stress
            if (icoun.eq.0) then
              inode=lboel(1,iboun)                          ! Distance to the
              do idime=1,ndime                              ! first node of the
                dista=dista+(coord(idime,ipoin)-            ! first boundary
     .            coord(idime,lnods(inode,ielem)))*         ! found with stress
     .            (coord(idime,ipoin)-                      ! (to initialise
     .            coord(idime,lnods(inode,ielem)))          ! DISTA)
              end do
              icoun=1
            end if
            do inodb=1,nnodb                                ! Minimum distance
              distn=0.0d0                                   ! to the boundary
              inode=lboel(inodb,iboun)
              do idime=1,ndime
                distn=distn+(coord(idime,ipoin)-
     .            coord(idime,lnods(inode,ielem)))*
     .            (coord(idime,ipoin)-
     .            coord(idime,lnods(inode,ielem)))
              end do
              dista=min(dista,distn)
            end do
          end if
        end do
        if(kturb.ge.1) then
          cletu(ipoin)=sqrt(dista)+turbu(8)
          dicom=max(dicom,dista)
        end if
      end do
      dicom=sqrt(dicom)+turbu(8)                            ! Maximum distance
c
c*** Now CLETU is modified to be the characteristic length to compute
c*** E = c_u K^(1.5) / L.
c
      if(kturb.ge.1) then
        do ipoin=1,npoin
          if(turbu(2).gt.zero3) then                        ! L constant
            cletu(ipoin)=turbu(2)
          else if(turbu(2).gt.-zero3) then                  ! L = h
            if(kturb.gt.2)
     .        call runend('TURFIX: WRONG VALUE OF TURBU(2)')
          else if(turbu(2).gt.-1.0-zero3) then              ! L from the 
            y=cletu(ipoin)/dicom                            ! Nikuradse model
            xmile=dicom*(0.14d0-0.08d0*(1.0d0-y)*(1.0d0-y)
     .        -0.06d0*(1.0d0-y)*(1.0d0-y)*(1.0d0-y)
     .        *(1.0d0-y))*sqrt(sqrt(turbu(1)))
            cletu(ipoin)=xmile
          else                                              ! Other forms of
            call runend('TURFIX: WRONG VALUE OF TURBU(2)')  ! computing L may
          end if                                            ! be coded here
        end do
      end if
c
c*** For boundary nodes where all the velocity is prescribed, the boundary
c*** conditions are taken as K = a U^2 , where a = TURBU(10) is an user
c*** defined constants. The value of the prescription of the second parameter
c*** is given in UPSETU (The interior values are assumed to be compatible
c*** with the boundary values).
c
      itotv=-ndime
      do ipoin=1,npoin
        if(kturb.ge.2) then
          itotv=itotv+ndime
          ibopo=lpoty(ipoin)
          if (ibopo.ne.0) then
            venor=vecdot(exnor(1,1,ibopo),
     .        presc(itotv+1),ndime)                         ! u_n = u . n
            venor=abs(venor)                                ! |u_n|
            if (   iffix(itotv+    1).eq.1
     .        .and.iffix(itotv+    2).eq.1                    
     .        .and.iffix(itotv+ndime).eq.1 ) then
              if (venor.gt.zero1) then                      ! Normal vel. is 
                itufi(ipoin,1)=1                            ! diferent of zero
                tkine(ipoin,1)=turbu(10)
                tkine(ipoin,2)=tkine(ipoin,1)
                if(kturb.eq.3.or.kturb.eq.4.or.kturb.eq.5) then
                  itufi(ipoin,2)=1
                else if(kturb.ge.6) then
                  call runend(
     .              'TURFIX: SET BC FOR THE TURBULENCE MODEL')
                end if
              else                                          ! Tangent vel. is
                itufi(ipoin,1)=1                            ! prescribed (wall)
                tkine(ipoin,1)=turbu(10)
                tkine(ipoin,2)=tkine(ipoin,1)
                if(kturb.eq.3.or.kturb.eq.4.or.kturb.eq.5) then
                  itufi(ipoin,2)=1
                else if(kturb.ge.6) then
                  call runend(
     .              'TURFIX: SET BC FOR THE TURBULENCE MODEL')
                end if
              end if
            end if
          end if
        end if
      end do
c
c*** Now what is going to be considered "zero" for K and E is determined. 
c*** Also, initial conditions are selected.
c
      if(kturb.ge.2) then
        tkcom=tkine(1,1)
        do ipoin=2,npoin
          tkcom=max(tkine(ipoin,1),tkcom)
        end do
        if (tkcom.lt.zero1) tkcom=turbu(10)
        do ipoin=1,npoin                                    ! K^0 = K_max
          tkine(ipoin,1)=max(tkine(ipoin,1),tkcom)
          tkine(ipoin,2)=tkine(ipoin,1)
        end do
        tkcom=zero1*tkcom
        
        if(kturb.eq.2) then
          tecom=zero1*turbu(1)*tkcom**(1.5)/dicom
        else if(kturb.ge.3) then
          call upsetu(itufi,tkine,cletu,tener,npoin,turbu,kturb)
          tecom=tener(1,1)
          do ipoin=2,npoin 
            tecom=max(tener(ipoin,1),tecom)
          end do
          do ipoin=1,npoin
            tener(ipoin,1)=max(tener(ipoin,1),tecom)
            tener(ipoin,2)=tener(ipoin,1)
          end do
          tecom=zero1*tecom
        end if
        call upvist(%val(tkine_p),%val(cletu_p),
     .              %val(visct_p),npoin,turbu)
      end if
c
c*** Compute the number of nodes on the boundary with wall laws , prescribe
c*** Dirichlet condition for velocity on the wall if it is neccesary, and
c*** allocate memory for the kinematic viscosity at these nodes 
c
      if (kturb.ge.1) then
        npowa=0
        do ipoin=1,npoin
          if(itufi(ipoin,1).eq.2) npowa=npowa+1
        end do

        if(npowa.gt.0) then
          lbyts=8*npowa
          call admemo(0,vikin_p,lbyts)
          call veczer(npowa,%val(vikin_p))
          mepoi=mepoi+lbyts
        end if
      end if
c
c*** Periodicity for turbulent variables
c
      if (kturb.ge.2) then
        do ipoin=1,npoin
          ibopo=lpoty(ipoin)
          iroty=-lroty(ibopo)
          if (iroty.gt.0) then
            itufi(ipoin,1)=1
            tkine(ipoin,1)=tkine(iroty,1)
            tkine(ipoin,2)=tkine(iroty,2)
            if (kturb.ge.3) then
              itufi(ipoin,2)=1
              tener(ipoin,1)=tener(iroty,1)
              tener(ipoin,2)=tener(iroty,2)
            end if
          end if
        end do
      end if
            
      call cputim(time2)
      cpust=cpust+(time2-time1)
      
      end
